      program biharm

! Two-dimensional biharmonic spline interpolation of irregularly
! placed data onto regular grid using Green function algorithm of
! David T. Sandwell (1987).

#define TIMING
#define INDEX_COORDS

      implicit none
      character(len=64) merged_bath_coast, roms_grid_file
      integer max_pts, npts, nx,ny, ncgrd, ierr, i,j,k, lmrg,
     &                                             lgrd,lenstr
      integer, allocatable, dimension(:) :: ip,jp
      real(kind=8), allocatable, dimension(:) :: x,y,z,w,b, xi,eta
      real(kind=8), allocatable, dimension(:,:) :: xlon,ylat, hraw,
     &                                               A,A_bak, bffr
      real(kind=8) xmin,xmax,ymin,ymax, xc,yc,csA,cffX,cffY,r2,err
      real(kind=8), parameter  ::  pi=3.14159265358979323846D0,
     & Eradius=6371315.D0, deg2rad=pi/180.D0, rad2deg=180.D0/pi

#include "netcdf.inc"
#ifdef TIMING
      integer nclk, iclk(2), clk_rate, clk_max
      integer(kind=8) :: inc_clk, net_clk=0
      nclk=1
      call system_clock(iclk(nclk), clk_rate, clk_max)
#endif

      if (iargc() == 2) then
        call getarg(1,merged_bath_coast); call getarg(2,roms_grid_file)
        lmrg=lenstr(  merged_bath_coast);  lgrd=lenstr( roms_grid_file)

        write(*,*) 'arguments: ''', merged_bath_coast(1:lmrg), ''' ''',
     &                                  roms_grid_file(1:lgrd), ''''
      else
        write(*,'(1x,2A/ /8x,2A/)')     'This command-line operator ',
     &           'should be used as', 'biharm merged_bath_coast.dat ',
     &                                           'roms_grid_file_nc'
        stop
      endif

      npts=0; max_pts=256; allocate(x(max_pts),y(max_pts),z(max_pts))

      open(unit=11, file=merged_bath_coast(1:lmrg), status='old')
      ierr=0
      do while(ierr == 0)
        npts=npts+1
        if (npts > max_pts) then
          allocate(bffr(npts-1,3))
          bffr(1:npts-1,1)=x(1:npts-1)
          bffr(1:npts-1,2)=y(1:npts-1)
          bffr(1:npts-1,3)=z(1:npts-1)
          deallocate(x,y,z)
          max_pts=2*max_pts ; write(*,*) 'extended max_pts =',max_pts
          allocate(x(max_pts),y(max_pts),z(max_pts))
          x(1:npts-1)=bffr(1:npts-1,1)
          y(1:npts-1)=bffr(1:npts-1,2)
          z(1:npts-1)=bffr(1:npts-1,3)
          deallocate(bffr)
        endif
        read(11,*,iostat=ierr) y(npts),x(npts),z(npts)
      enddo
      npts=npts-1
      close(unit=11)
      write(*,*) 'Bathymetry data, npts =', npts

      ierr=nf_open(roms_grid_file(1:lgrd), nf_nowrite, ncgrd)
      if (ierr == nf_noerr) then
        call roms_find_dims(ncgrd, roms_grid_file, i,j,k)
        nx=i+2 ; ny=j+2
        allocate(xlon(nx,ny), ylat(nx,ny))
        call get_var_by_name_double(ncgrd, 'lon_rho', xlon)
        call get_var_by_name_double(ncgrd, 'lat_rho', ylat)
        ierr=nf_close(ncgrd)

! Convert into quasi-Cartesian coordinates: this is needed to correctly
! judge distances between the points because displacement in meters by
! the same fraction of a degree in geographical coordinates in E-W
! direction is only cos(lat) of that in N-S.  To evaluate cos(lat) and
! also to minimize roundoff errors determine center (half-way between
! min and max in each direction) and use it as the origin. The resultant
! units of (x,y) are meters.

        xmin=x(1); xmax=x(1); ymin=y(1); ymax=y(1)
        do k=2,npts
          xmin=min(xmin,x(k)) ; xmax=max(xmax,x(k))
          ymin=min(ymin,y(k)) ; ymax=max(ymax,y(k))
        enddo
        xc=0.5D0*(xmin+xmax) ; yc=0.5D0*(ymin+ymax)
        csA=cos(pi*yc/180.D0)
        write(*,*) 'cos(A) =', csA
        cffY=Eradius*pi/180.D0 ; cffX=csA*cffY
        do k=1,npts
          x(k)=cffX*(x(k)-xc) ; y(k)=cffY*(y(k)-yc)
c**       write(*,*) y(k),x(k)
        enddo

        do j=1,ny                         ! apply exactly the
          do i=1,nx                       ! same transformation
            xlon(i,j)=cffX*(xlon(i,j)-xc) ! for ROMS grid
            ylat(i,j)=cffY*(ylat(i,j)-yc) ! coordinates
          enddo
        enddo

! This option transforms both coordinates into ROMS grid index space,
! basically determine to which grid cell [i:i+1]x[j:j+1] of ROMS grid
! each point og the trajectory (x(k),y(k)) belongs, then compute exact
! location (xi,eta) expressed as grid-box franction.

#ifdef INDEX_COORDS
        allocate(ip(npts),jp(npts), xi(npts),eta(npts))
        call search_indices_tile(1,npts, 1,1, nx,ny, xlon,ylat,
     &                                      npts,1, x,y, ip,jp)
        call   comp_offsets_tile(1,npts, 1,1, nx,ny, xlon,ylat,
     &                              npts,1, x,y, ip,jp, xi,eta)
        do k=1,npts
          x(k)=dble(ip(k)-1)+xi(k) ; y(k)=dble(jp(k)-1)+eta(k)
        enddo
        do j=1,ny
          do i=1,nx
            xlon(i,j)=dble(i-1) ; ylat(i,j)=dble(j-1)
          enddo
        enddo
#endif

#ifdef TIMING
        nclk=3-nclk
        call system_clock(iclk(nclk), clk_rate,clk_max)
        inc_clk=iclk(nclk)-iclk(3-nclk)
        if (inc_clk < 0) inc_clk=inc_clk+clk_max
        net_clk=net_clk+inc_clk
        write(*,'(F8.2,1x,A)') dble(net_clk)/dble(clk_rate),
     &                                 'initializing matrix'
#endif
        allocate(w(npts), b(npts), A(npts,npts), A_bak(npts,npts))
        do j=1,npts
          A(j,j)=0.D0 ; b(j)=z(j) ; A_bak(j,j)=0.D0 ; w(j)=0.D0
        enddo
        do j=1,npts
          do i=j+1,npts
            r2=(x(i)-x(j))**2 + (y(i)-y(j))**2
            A(i,j)=r2*(0.5D0*log(r2)-1.D0)
            A(j,i)=A(i,j) ; A_bak(i,j)=A(i,j); A_bak(j,i)=A(i,j)
          enddo
        enddo

#ifdef TIMING
        nclk=3-nclk
        call system_clock(iclk(nclk), clk_rate,clk_max)
        inc_clk=iclk(nclk)-iclk(3-nclk)
        if (inc_clk < 0) inc_clk=inc_clk+clk_max
        net_clk=net_clk+inc_clk
        write(*,'(F8.2,1x,A)') dble(net_clk)/dble(clk_rate),
     &                                           'solving...'
#endif
C$OMP PARALLEL SHARED(npts, A,b, w)
        call pivoted_Gauss_Jordan(npts, A,b, w)
C$OMP END PARALLEL

#ifdef TIMING
        nclk=3-nclk
        call system_clock(iclk(nclk), clk_rate,clk_max)
        inc_clk=iclk(nclk)-iclk(3-nclk)
        if (inc_clk < 0) inc_clk=inc_clk+clk_max
        net_clk=net_clk+inc_clk
        write(*,'(F8.2,1x,A)') dble(net_clk)/dble(clk_rate),
     &                                          'checking...'
#endif
        err=0.D0
        do j=1,npts
          b(j)=z(j)
          do i=1,npts
            b(j)=b(j)-A_bak(i,j)*w(i)
          enddo
          err=max(err,abs(b(j)))
        enddo
        write(*,*) 'maximum error =', err
        deallocate(A_bak,A,b)

#ifdef TIMING
        nclk=3-nclk
        call system_clock(iclk(nclk), clk_rate,clk_max)
        inc_clk=iclk(nclk)-iclk(3-nclk)
        if (inc_clk < 0) inc_clk=inc_clk+clk_max
        net_clk=net_clk+inc_clk
        write(*,'(F8.2,1x,A)') dble(net_clk)/dble(clk_rate),
     &                                     'interpolating...'
#endif
        allocate(hraw(nx,ny))
C$OMP PARALLEL SHARED(npts, x,y,w, nx,ny, xlon,ylat, hraw)
        call sub1(npts, x,y,w, nx,ny, xlon,ylat,hraw)
C$OMP END PARALLEL

#ifdef TIMING
        nclk=3-nclk
        call system_clock(iclk(nclk), clk_rate,clk_max)
        inc_clk=iclk(nclk)-iclk(3-nclk)
        if (inc_clk < 0) inc_clk=inc_clk+clk_max
        net_clk=net_clk+inc_clk
        write(*,'(F8.2,1x,A)') dble(net_clk)/dble(clk_rate),
     &                           'saving into netCDF file...'
#endif
        ierr=nf_open(roms_grid_file(1:lgrd), nf_write, ncgrd)
        call put_var_by_name_double(ncgrd, 'hraw', hraw)
        ierr=nf_close(ncgrd)
      else
        write(*,'(/1x,4A/8x,A)') '### ERROR: Cannot open netCDF ',
     &       'file ''', roms_grid_file(1:lgrd), ''' for reading.',
     &                                         nf_strerror(ierr)
      endif
      end


      subroutine sub1(npts, x,y,w, nx,ny, xlon,ylat,hraw)
      implicit none
      integer npts, nx,ny, i,j,k, ii,kk
      real(kind=8) x(npts),y(npts), w(npts), xlon(nx,ny),ylat(nx,ny),
     &                                      hraw(nx,ny), xk,yk,wk,r2

      integer numthreads, trd, chunk_size, jstr,jend
C$    integer omp_get_num_threads, omp_get_thread_num

      numthreads=1 ; trd=0
C$    numthreads=omp_get_num_threads(); trd=omp_get_thread_num()
      chunk_size=(ny+numthreads-1)/numthreads
      jstr=1+trd*chunk_size ; jend=min(jstr+chunk_size-1, ny)

      do j=jstr,jend
        do i=1,nx
          hraw(i,j)=0.D0
        enddo
        do kk=1,npts,32
          do ii=1,nx,256
            do k=kk,min(kk+31,npts)
              xk=x(k) ; yk=y(k) ; wk=w(k)
              do i=ii,min(ii+255,nx)
                r2=(xlon(i,j)-xk)**2 + (ylat(i,j)-yk)**2
                hraw(i,j)=hraw(i,j) +wk*r2*(0.5D0*log(r2)-1.D0)
              enddo
            enddo
          enddo
        enddo
      enddo
      end

      module pivtd_gs_sync_vars
        real(kind=8), save :: A_max=0.D0
        integer, save :: trd_count=0, i_pvt=0,j_pvt=0
        integer, allocatable, dimension(:), save :: m
      end module pivtd_gs_sync_vars 

      subroutine pivoted_Gauss_Jordan(n, a,b, x)

! Fully-pivoted Gauss-Jordan algorithm for solving linear
! algebra system
!                           A*x=b
!
! where A=A(n,n) is a square matrix; x=x(n) is vector of unknowns;
! and b=b(n) is vector of r.h.s., known.

! Created by Alexander Shchepetkin, old_galaxy@yahoo.com

! Note that in the algorithm below m(j) initially set to all-zeros
! eventually filled with coordinates of pivots, i_pvt=m(j_pvt), where
! j_pvt = 1,2,....,n. Once a nonzero value is assigned to a particular
! m(j)>0 is used as flag to indicate that a pivot was already selected
! within this row, and therefore it excluded from the search for new
! pivots.  Finally, after all m(j) got their values, it is used to
! identify which x(j) is which, because the resultant matrix is not
! diagonal unit matrix, but rather contains unit values in random
! places.  Note that this algorithm employs "in place" approach, so
! no explicit row-swapping is needed as the result.

      use pivtd_gs_sync_vars
      implicit none
      integer n, i,j,k, imx, my_i_pvt,my_j_pvt
      real(kind=8) a(n,n),b(n), x(n), my_A_max, amx, amx2, cff

      integer numthreads, trd, chunk_size, jstr,jend
C$    integeromp_get_num_threads, omp_get_thread_num

c**#define VERBOSE
#ifdef VERBOSE
      integer ncid, varid, ierr, dim(3), start(3),count(3)
# include "netcdf.inc"
#endif
      
      numthreads=1 ; trd=0
C$    numthreads=omp_get_num_threads() ; trd=omp_get_thread_num() 
      chunk_size=(n+numthreads-1)/numthreads
      jstr=1+trd*chunk_size ; jend=min(jstr+chunk_size-1,n)
C$OMP MASTER 
      if (.not.allocated(m)) allocate(m(n))
      trd_count=0 ; A_max=0.D0 ; m=0
#ifdef VERBOSE
      ierr=nf_create('pvtd_gs_chk.nc', nf_clobber, ncid)
      ierr=nf_def_dim(ncid, 'nx', n, dim(1))
      ierr=nf_def_dim(ncid, 'ny', n, dim(2))
      ierr=nf_def_dim(ncid, 'stp', nf_unlimited,  dim(3))
      ierr=nf_def_var(ncid, 'A', nf_double, 3,dim, varid)
      ierr=nf_enddef(ncid); start=1; count=n
      start(3)=1; count(3)=1
      ierr=nf_put_vara_double(ncid,varid, start,count, a)
#endif
C$OMP END MASTER
C$OMP BARRIER

      do k=1,n
        my_A_max=0.D0
        do j=jstr,jend                 ! Criterion for selecting
          if (m(j) == 0) then          ! pivot: not just the largest
            amx=0.D0 ; amx2=0.D0       ! by absolute value coefficient,
            do i=1,n                   ! but the one which dominates
              cff=abs(a(i,j))          ! its row the strongest way.
              if (cff > amx) then      ! Basically compare the largest
                amx=cff ; imx=i        ! coefficient with the second
              elseif (cff > amx2) then ! largest on the same row and
                amx2=cff               ! make the selection based on
              endif                    ! the ratio of the two.
            enddo
            cff=amx/(amx+amx2)         !<-- ratio
            if (cff > my_A_max) then
              my_A_max=cff ; my_i_pvt=imx ; my_j_pvt=j
            endif
          endif
        enddo
C$OMP BARRIER
C$OMP CRITICAL
        if (my_A_max > A_max) then
          A_max=my_A_max ; i_pvt=my_i_pvt ; j_pvt=my_j_pvt
        endif
        trd_count=trd_count+1
        if (trd_count == numthreads) then
          trd_count=0 ; A_max=0.D0
          m(j_pvt)=i_pvt    !<-- record indices for new pivot
        endif
C$OMP END CRITICAL
C$OMP BARRIER
        do j=jend,jstr,-1                  ! subtract row "j_pvt" from
          if (j /= j_pvt) then             ! all others in order to
            cff=a(i_pvt,j)/a(i_pvt,j_pvt)  ! eliminate all coefficients
            do i=1,n                       ! in column "i_pvt" except
              a(i,j)=a(i,j)-cff*a(i,j_pvt) ! on the pivot row itself.

              if (i == i_pvt) a(i,j)=0.D0  !<-- see note **** below
            enddo
            b(j)=b(j)-cff*b(j_pvt)         !<-- same for r.h.s.
          endif
        enddo
C$OMP BARRIER
#ifdef VERBOSE
C$OMP MASTER
        start(3)=start(3)+1
        ierr=nf_put_vara_double(ncid,varid, start,count, a)
C$OMP END MASTER
C$OMP BARRIER
#endif
      enddo !<-- k
c**   write(*,'(A,I3,A,200I4)') 'here',trd, ' m=', m(jstr:jend)
      do j=jstr,jend
        x(m(j))=b(j)/a(m(j),j) !<-- the solution
      enddo
C$OMP BARRIER
#ifdef VERBOSE
C$OMP MASTER
        ierr=nf_close(ncid)
C$OMP END MASTER
C$OMP BARRIER
#endif
      end

! Note *** about setting a(i = i_pvt, j /= j_pvt)=0.D0 by hand vs.
! not doing so: mathematically these coefficients are "eliminated" by
! subtracting a(i,j)=a(i,j)-a(i_pvt,j)*a(i,j_pvt)/a(i_pvt,j_pvt) which
! should yield 0 for all i=i_pvt. However this comes with some roundoff
! error AND these "zeroes" participate in subsequent computations.
! Overall setting them hands improves roundoff error but is not crucial
! for this algorithm.
