!---------------------------------------------------------------------
!
!                                B Y T E S W A P
!
!  Adapted from: Dr. David G. Simpson
!                http://www.davidgsimpson.com/software/byteswap_f90.txt
!                NASA Goddard Space Flight Center
!                Greenbelt, Maryland  20771
!
!  Date:         July 23, 2005
!
!  Description:  A set of subroutines to reverse byte order of integer
!                and real variables. This is useful when reading binary
!                data from a file intended for use on a computer whose
!                byte order is opposite that of the computer on which
!                the binary data was created, BIG- vs. LITTLE-endian.
!
!                   swap_i2    Swap bytes of a 2-byte integer
!                   swap_i4    Swap bytes of a 4-byte integer
!                   swap_i8    Swap bytes of a 8-byte integer
!                   swap_f4    Swap bytes of a 4-byte real
!                   swap_f8    Swap bytes of an 8-byte real
!----------------------------------------------------------------------

      subroutine swap_i2(int2)
      implicit none
      integer(kind=2), intent(inout) :: int2
      integer(kind=1), dimension(2) :: byte_arr,byte_out
      byte_arr=transfer(int2, byte_arr)
      byte_out(1)=byte_arr(2) ; byte_out(2)=byte_arr(1)
      int2=transfer(byte_out, int2)
      end subroutine swap_i2

      subroutine swap_i4(int4)
      implicit none
      integer(kind=4), intent(inout) :: int4
      integer(kind=1), dimension(4) :: byte_arr,byte_out
      byte_arr=transfer(int4, byte_arr)
      byte_out(1)=byte_arr(4) ; byte_out(4)=byte_arr(1)
      byte_out(2)=byte_arr(3) ; byte_out(3)=byte_arr(2)
      int4=transfer(byte_out, int4)
      end subroutine swap_i4

      subroutine swap_i8(int8)
      implicit none
      integer(kind=8), intent(inout) :: int8
      integer(kind=1), dimension(8) :: byte_arr,byte_out
      byte_arr=transfer(int8, byte_arr)
      byte_out(1)=byte_arr(8) ; byte_out(8)=byte_arr(1)
      byte_out(2)=byte_arr(7) ; byte_out(7)=byte_arr(2)
      byte_out(3)=byte_arr(6) ; byte_out(6)=byte_arr(3)
      byte_out(4)=byte_arr(5) ; byte_out(5)=byte_arr(4)
      int8=transfer(byte_out, int8)
      end subroutine swap_i8

      subroutine swap_f4(float4)
      implicit none
      real(kind=4), intent(inout) :: float4
      integer(kind=1), dimension(4) :: byte_arr,byte_out
      byte_arr=transfer(float4, byte_arr)
      byte_out(1)=byte_arr(4) ; byte_out(4)=byte_arr(1)
      byte_out(2)=byte_arr(3) ; byte_out(3)=byte_arr(2)
      float4=transfer(byte_out, float4)
      end subroutine swap_f4

      subroutine swap_f8(float8)
      implicit none
      real(kind=8), intent(inout) :: float8
      integer(kind=1), dimension(8) :: byte_arr,byte_out
      byte_arr=transfer(float8, byte_arr)
      byte_out(1)=byte_arr(8) ; byte_out(8)=byte_arr(1)
      byte_out(2)=byte_arr(7) ; byte_out(7)=byte_arr(2)
      byte_out(3)=byte_arr(6) ; byte_out(6)=byte_arr(3)
      byte_out(4)=byte_arr(5) ; byte_out(5)=byte_arr(4)
      float8=transfer(byte_out, float8)
      end subroutine swap_f8

