      program clim

! Computes climatological monthly, seasonal, and annual averages from
! instantaneous or short-term-averaged records from a long run stored
! in multiple files.

      implicit none
      real(kind=8) time, tdays, cycle_length, tclm, tmin,tmax, dt,
     &                  tpast,tnext, wtmin,wtmax, total_weight,cff
      real(kind=4), allocatable, dimension(:) :: A, buff
      integer nsrcs, ncsrc, nctarg, ndims, nvars, vars_to_process,
     &        varid, vardims, vartype, varatts, ngatts, unlimdimid,
     &        timedimid, nrecs, nuvars, rec, tunits, size, max_size,
     &        combined_size, i,j,k,n, month, ierr, lsrc,
     &                                ltrg, ltvr, lstr, ldim, lvar

      integer, allocatable, dimension(:) :: vid, vsize, start,
     &                                           dimid, dsize
      integer, allocatable, dimension(:) ::  vardimids
      integer  iargc, read_str_dble, lenstr !<-- functions
      integer, parameter :: time_in_seconds=1, time_in_hours=2,
     &                                           time_in_days=3
      logical select
      character(len=64) srcfile, trgfile, str
      character(len=32) time_var_name, varname, dimname
      character(len=64), allocatable, dimension(:) :: srcname
      character(len=32), allocatable, dimension(:) :: vname, uvars

#include "netcdf.inc"

      nuvars=0 ; timedimid=0 ; vars_to_process=0 ; max_size=0
      n=0 ; tunits=0 ; ltvr=0 ; cycle_length=365.25D0 !<-- default

! Sort out arguments within the command line: optional arguments
! starting with dash may specify name of the timing variable, units
! of time, and length of one year.  All other arguments are intended
! to be file names (at least one is required).  Overall the approach
! is to set default values first, and then let them be overwritten.

      nsrcs=iargc()
      if (nsrcs > 0) then
        allocate(srcname(nsrcs))
        do j=1,nsrcs
          call getarg(j,str) ; lstr=lenstr(str)
          if (str(1:1) == '-') then
            i=2
            do while(i < lstr .and. str(i:i) /= '=')
              i=i+1
            enddo
            if ( (i == 11 .and. str(1:i-1) == '--time_var') .or.
     &                   (i == 3 .and. str(1:i-1) == '-t') ) then
              time_var_name=str(i+1:lstr) ; ltvr=lstr-i
              write(*,*)  'user supplied time_var_name = ''',
     &                           time_var_name(1:ltvr), ''''

            elseif ( (i == 7 .and. str(1:i-1) == '--vars') .or.
     &                 (i == 3 .and. str(1:i-1) == '-v') ) then
              nuvars=1
              do k=i+1,lstr
                if (str(k:k) == ',') nuvars=nuvars+1
              enddo
              allocate(uvars(nuvars))
              nuvars=0 ; k=i
              do while(k < lstr)
                k=k+1
                if (str(k:k) == ',') then
                  nuvars=nuvars+1 ; uvars(nuvars)=str(i+1:k-1)
                  i=k
                endif
              enddo
              nuvars=nuvars+1 ; uvars(nuvars)=str(i+1:lstr)
              write(*,*)  'user selected variables =', (' ',
     &            uvars(i)(1:lenstr(uvars(i))), i=1,nuvars)

            elseif ( (i == 15 .and. str(1:i-1) == '--cycle_length')
     &               .or. (i == 3  .and. str(1:i-1) == '-y') ) then
              ierr=read_str_dble(str(i+1:lstr), cycle_length)
              if (ierr == 0) then
                write(*,*)  'user supplied cycle_length =',
     &                                     cycle_length
              else
                write(*,'(/1x,4A/)')  '### ERROR: Cannot read cycle ',
     &                   'length from argument ''', str(1:lstr), '''.'
                stop
              endif
            elseif ( (i == 13 .and. str(1:i-1) == '--time_units')
     &               .or. (i == 3 .and. str(1:i-1) == '-u') ) then
              if (str(i+1:i+6) == 'second') then
                tunits=time_in_seconds
              elseif (str(i+1:i+4) == 'hour') then
                tunits=time_in_hours
              elseif (str(i+1:i+4) == 'days') then
                tunits=time_in_days
              elseif (str(i+1:i+1) == 's') then
                tunits=time_in_seconds
              elseif (str(i+1:i+1) == 'h') then
                tunits=time_in_hours
              elseif (str(i+1:i+1) == 'd') then
                tunits=time_in_days
              else
                write(*,'(/1x,4A/)') '### ERROR: Cannot determine ',
     &            'time units from argument ''', str(1:lstr), '''.'
                stop
              endif
              write(*,*) 'user supplied time units = ',str(i+1:i+1)
            else
              write(*,'(/1x,4A/)')  '### ERROR: Unrecognized ',
     &          'command line argument ''', str(1:lstr), '''.'
              stop
            endif
          else
            n=n+1
            srcname(n)=str(1:lstr) ; lstr=lenstr(srcname(n))
          endif
        enddo
        nsrcs=n  !<-- number of source files to process
      else
        write(*,'(3(/1x,2A),2(/1x,A/5x,A),2(/1x,A/5x,2A))')
     &   'This command-line operator takes a single netCDF file ',
     &   'or a sequence of files', 'and computes monthly, seasonal, ',
     &   'and annual climatological averages placing the',
     &   'results into newly created "monthly.nc" and "seasonal.nc". ',
     &   ' Should be invoked', 'as', 'clim file.nc', 'or',
     &   'clim file1.nc file2.nc ...', 'or', 'clim -t=TIMEVAR ',
     &   '-v=VARS -y=LENGTH -u=UNITS file1.nc file2.nc ...', 'or',
     &   'clim --time_var=TIMEVAR --vars=VARS --cycle_length=LENGTH ',
     &                    '--time_units=UNITS file1.nc file2.nc ...'

        write(*,'(11(/1x,2A)/1x,A,F6.2,1x,A,5(/1x,2A)/)')
     &   'where all files in the list, file1.nc file2.nc etc..., ',
     &   'must be conformal to',  'each other - have the same ',
     &   'dimensions and contain the same set of variables.',
     &   'Wildcards are accepted to specify multiple files.   ',
     &   'Arguments starting with', 'dash are optional to override ',
     &   'internal defaults. TIMEVAR is the name of time',
     &   'variable in netCDF file.   If not specified, the program ',
     &   'attempts to find it',  'automatically as a variable with ',
     &   'its name ending on "time" and having only a',
     &   'single dimension whose name also ending on "time".  ',
     &   'An error message will be',  'issued if no such variable ',
     &   'exists, or if it is not unique.  VARS is a comma-',
     &   'separated list of variables for which climatology be ',
     &   'computed. Omitting this', 'list makes the program process ',
     &   'all the variables which have time dimension.',
     &   'LENGTH is the length of the year expressed in days.  ',
     &   'If not specified, the',  'default value of cycle_length=',
     &    cycle_length,  'is assumed.     UNITS must be either',
     &   '"seconds", "hours", or "days" (only the first character ',
     &   'matters, "s", "h"', 'or "d") specifies the units of ',
     &   'time variable.  If not specified, the program',
     &   'attempts to find an attribute named "units" belonging ',
     &   'to the time variable',  'and infer the units from it. ',
     &   'An error message will be issued if there is no',
     &   'attribute, or if the units are not among "seconds", ',
     &   '"hours", or "days".'
        stop
      endif
      if (nsrcs == 0) stop

! Look at the first source file in the list and use it as template:
! select variables to be processed -- keep only the ones which have
! time dimension. Determine maximum size needed to accommodate one
! time record of the largest variable as well as the aggregate size
! to hold one record of all variables combined. Since  time variable
! requires special treatment, place it as #0 into the list.  At the
! end of this stage allocate array to hold averages and buffer array
! for reading.

      srcfile=srcname(1) ; lsrc=lenstr(srcfile)
      ierr=nf_open(srcfile, nf_nowrite, ncsrc)
      if (ierr == nf_noerr) then
        ierr=nf_inq(ncsrc, ndims, nvars, ngatts, unlimdimid)
        if (ierr == nf_noerr) then

          allocate(dimid(ndims))   ; allocate(dsize(ndims))
          allocate(vid(0:nvars))   ; allocate(vsize(0:nvars))
          allocate(vname(0:nvars)) ; allocate(start(0:nvars))
          allocate(vardimids(ndims))

! Identify time variable and time dimension: At this moment its name
! may be already known (specified by user), or still to be udentified.
! Therefore there are two branches.   In the case when time variable
! needs to be identified automatically, it is presumed (i) to have its
! name end with "time", (ii) to have only a single dimension (not
! necessarily unlimited), and (iii) name of that dimensions must end
! with "time" as well.

          if (ltvr > 0) then
            ierr=nf_inq_varid(ncsrc, time_var_name(1:ltvr), varid)
            if (ierr == nf_noerr) then
              ierr=nf_inq_varndims(ncsrc, varid, vardims)
              if (ierr == nf_noerr) then
                ierr=nf_inq_vardimid(ncsrc, varid, vardimids)
                if (ierr == nf_noerr) then
                  if (vardims == 1) then
                    vid(0)=varid ; timedimid=vardimids(1)
                  else
                    write(*,'(/1x,4A,I3/)')         '### ERROR: User-',
     &             'specified time variable ''', time_var_name(1:ltvr),
     &                   ''' has wrong number of dimensions', vardims
                    stop
                  endif
                else
                  write(*,'(/1x,6A/12x,A/)') '### ERROR: Cannot find ',
     &                                 'dimension IDs for variable ''',
     &                               time_var_name(1:ltvr), ''' in ''',
     &                      srcfile(1:lsrc), '''.',  nf_strerror(ierr)
                endif
              else
                write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot find ',
     &            'number of dimensions for ''', time_var_name(1:ltvr),
     &            ''' in ''',srcfile(1:lsrc),'''.',  nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,6A/12x,A/)')     '### ERROR: Cannot find ',
     &                'variable ''', time_var_name(1:ltvr), ''' in ''',
     &                      srcfile(1:lsrc), '''.',  nf_strerror(ierr)
            endif
            if (ierr /= nf_noerr) stop
          else
            do varid=1,nvars
              ierr=nf_inq_var(ncsrc, varid, varname,  vartype,
     &                            vardims, vardimids, varatts)
              if (ierr == nf_noerr) then
                lvar=lenstr(varname)
                if (vardims == 1 .and. lvar > 3) then
                  if (varname(lvar-3:lvar) == 'time') then
                    ierr=nf_inq_dim(ncsrc,vardimids(1), dimname,nrecs)
                    if (ierr == nf_noerr) then
                      ldim=lenstr(dimname)
                      if (dimname(ldim-3:ldim) == 'time') then
                        if (ltvr == 0) then
                          time_var_name=varname  ; ltvr=lvar
                          vid(0)=varid ; timedimid=vardimids(1)

                          write(*,'(1x,6A,I8,1x,A,I2)')'identified ',
     &                       'time variable ''', time_var_name(1:ltvr),
     &                       ''' and dimension ''',  dimname(1:ldim),
     &                       ''', nrecs =', nrecs
                        else
                          write(*,'(/1x,2A/)') '### ERROR: Ambiguous ',
     &                                  'time variable and dimension.'
                          stop
                        endif
                      endif
                    else
                     write(*,'(/1x,2A,I4,1x,3A/12x,A/)') '### ERROR:',
     &                          ' Cannot make inquiry for dimension #',
     &                          vardimids(1), 'in ''', srcfile(1:lsrc),
     &                                       '''.', nf_strerror (ierr)
                    endif
                  endif
                endif
              else
                write(*,'(/1x,2A,I4,1x,3A/12x,A/)')      '### ERROR: ',
     &                     'Cannot make inquiry for variable #', varid,
     &               'in ''',srcfile(1:lsrc),'''.', nf_strerror (ierr)
              endif
              if (ierr /= nf_noerr) stop
            enddo
          endif
          if (ltvr == 0) then
            write(*,'(/1x,2A/)')   '### ERROR: Cannot find time ',
     &                             'variable and time dimension.'
            stop
          endif

! Identify units for time variable (unless supplied by the user).

          if (tunits == 0) then
            ierr=nf_get_att_text(ncsrc, vid(0), 'units',str)
            if (ierr == nf_noerr) then
              if (str(1:6) == 'second') then
                tunits=time_in_seconds
              elseif (str(1:4) == 'hour') then
                tunits=time_in_hours
              elseif (str(1:4) == 'days') then
                tunits=time_in_days
              else
                write(*,'(/1x,6A/)')  '### ERROR: Unknown units for ',
     &                    'variable ''', time_var_name(1:ltvr), ''':',
     &                                      str(1:lenstr(str)), '''.'
                stop
              endif
              write(*,'(1x,3A,I2,1x,3A)')       'found units for ''',
     &                         time_var_name(1:ltvr),  ''' tunits =',
     &                         tunits, '''',str(1:lenstr(str)),'''.'
            else
              write(*,'(/1x,6A/12x,A/)') '### ERROR: Cannot retrieve ',
     &                           'attribute ''units'' for variable ''',
     &                            time_var_name(1:ltvr),  ''' from ''',
     &                      srcfile(1:lsrc), '''.', nf_strerror (ierr)
            endif
          endif
          if (tunits == 0) stop

! Scan all the variables and select only those which have time
! dimension (obviously, skip time variable itself). Record variable
! IDs and sizes.

          do varid=1,nvars
            if (varid /= vid(0)) then
              ierr=nf_inq_var(ncsrc, varid, varname,  vartype,
     &                            vardims, vardimids, varatts)
              if (ierr == nf_noerr) then
                lvar=lenstr(varname)
                if (nuvars > 0) then
                  select=.false.
                  do i=1,nuvars
                    lstr=lenstr(uvars(i))
                    if (lvar==lstr .and. varname(1:lvar) ==
     &                        uvars(i)(1:lstr)) select=.true.
                  enddo
                else
                 select=.true.
                endif

                if (select .and. ( vartype == nf_double .or.
     &                            vartype == nf_real )) then
                  select=.false. ; size=1
                  i=0
                  do while(i < vardims .and. ierr == nf_noerr)
                    i=i+1
                    if (vardimids(i) == timedimid) then
                      select=.true.
                    else
                      ierr=nf_inq_dimlen(ncsrc, vardimids(i), j)
                      if (ierr == nf_noerr) then
                        size=size*j
                      else
                       write(*,'(/1x,2A,I4,1x,3A/12x,A/)')
     &                    '### ERROR: Cannot determine size  of ',
     &                    'dimension #',   vardimids(i),  'in ''',
     &                    srcfile(1:lsrc),'''.',nf_strerror(ierr)
                      endif
                    endif
                  enddo

                  if (select) then
                    vars_to_process=vars_to_process+1
                    vid(vars_to_process)=varid
                    vname(vars_to_process)=varname
                    vsize(vars_to_process)=size
                    max_size=max(max_size,size)
                  endif
                endif
              else
                write(*,'(/1x,2A,I4,1x,3A/12x,A/)')      '### ERROR: ',
     &            'Cannot make inquiry for variable #', varid, 'in ''',
     &                      srcfile(1:lsrc), '''.', nf_strerror (ierr)
              endif
            endif
          enddo
        else
          write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot make general ',
     &    'inquiry into ''',srcfile(1:lsrc),'''.', nf_strerror (ierr)
        endif
        if (vars_to_process == 0) then
          write(*,'(/1x,2A/)') '### ERROR: No suitable variables are ',
     &                         'selected by user or found in the file.'
          stop
        endif

        max_size=0 ; combined_size=0
        do i=1,vars_to_process
          start(i)=combined_size+1
          combined_size=combined_size+vsize(i)
          max_size=max(max_size, vsize(i))
          lvar=lenstr(vname(i))
          write(*,'(1x,I3,1x,3A,I10)') i, 'found varable ''',
     &              vname(i)(1:lvar), '''  size =', vsize(i)
        enddo

        allocate(A(combined_size), buff(max_size))

      else
        write(*,'(/1x,4A/12x,A/)')   '### ERROR: Cannot open netCDF ',
     &  'file ''',srcfile(1:lsrc),''' for reading:', nf_strerror(ierr)
      endif
      if (ierr /= nf_noerr) stop


!      *****    *********    ******   *******    *********
!    ***   ***  *  ***  *   *   ***   ***   ***  *  ***  *
!    ***           ***     **   ***   ***   ***     ***
!      *****       ***    ***   ***   ***   **      ***
!          ***     ***    *********   ******        ***
!    ***   ***     ***    ***   ***   ***  **       ***
!      *****       ***    ***   ***   ***   ***     ***


! Create output netCDF file:
!------- ------ ------ -----

      trgfile='monthly.nc' ; ltrg=lenstr(trgfile)
      call def_clim_file(srcfile, ncsrc, trgfile, nctarg,
     &               vars_to_process, vid, timedimid, 12,
     &                                      cycle_length)
      ierr=nf_close(ncsrc) ; ncsrc=-1

! Computing monthly averages:  Since array of averages can hold only
!---------- ------- ---------  one time record, the processing is done
! in sparse manner, individually for each month. The sequence of files
! and the number of time records is considered to be arbitrary, and,
! strictly speaking, unknown in advance.  The algorithm employed here
! is sorting: just read time variable for each record, check where it
! fits into the desired time window: if so, read all the corresponding
! variables for that record and add their contributions.  Otherwise
! just skip and go to the next record.

      do month=1,12
        wtmin=dble(month-1)*cycle_length/12.D0    ! bounds for
        wtmax=dble(month  )*cycle_length/12.D0    ! one month

        total_weight=0.D0 ; time=0.D0
        do i=1,combined_size
          A(i)=0.    !< -- initialize summation;
        enddo

        do n=1,nsrcs
          srcfile=srcname(n) ; lsrc=lenstr(srcfile)
          ierr=nf_open(srcfile(1:lsrc), nf_nowrite, ncsrc)
          if (ierr == nf_noerr) then
            ierr=nf_inq_varid(ncsrc, time_var_name(1:ltvr), varid)
            if (ierr == nf_noerr) then
              ierr=nf_inq_varndims(ncsrc, varid, vardims)
              if (ierr == nf_noerr) then
                ierr=nf_inq_vardimid(ncsrc, varid, vardimids)
                if (ierr == nf_noerr) then
                  ierr=nf_inq_dimlen(ncsrc, vardimids(vardims), nrecs)
                  if (ierr == nf_noerr) then
                    write(*,*) 'found', nrecs, ' time records...'
                  else
                    write(*,'(/1x,2A,I4,1x,3A/12x,A/)')  '### ERROR: ',
     &                               'Cannot find size of dimension #',
     &                    vardimids(vardims), 'in ''', srcfile(1:lsrc),
     &                                       '''.',  nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,6A/12x,A/)') '### ERROR: Cannot find ',
     &                                 'dimension IDs for variable ''',
     &                               time_var_name(1:ltvr), ''' in ''',
     &                        srcfile(1:lsrc),'''.', nf_strerror(ierr)
                endif
              else
                write(*,'(/1x,6A/12x,A/)')  '### ERROR: Cannot find ',
     &           'number of dimensions for ''', time_var_name(1:ltvr),
     &           ''' in ''', srcfile(1:lsrc),'''.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot find ',
     &              'variable ''', time_var_name(1:ltvr), ''' in ''',
     &                    srcfile(1:lsrc), '''.',  nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot open ',
     &       'netCDF file ''', srcfile(1:lsrc), ''' for reading.',
     &                                          nf_strerror(ierr)
          endif
          if (ierr /= nf_noerr) stop


          ierr=nf_get_vara_double(ncsrc, varid, 1,1, tdays)
          if (ierr == nf_noerr) then
            if (tunits == time_in_seconds) then   ! read time for
              tdays=tdays/dble(3600*24)           ! the first record
            elseif (tunits == time_in_hours) then ! to start recursion,
              tdays=tdays/dble(24)                ! convert into days 
            endif
          else
            write(*,'(1x,4A,I7,1x,3A/10x,A)')  '### ERROR: Cannot ',
     &          'read ''', time_var_name(1:ltvr),  ''' rec =', rec,
     &          'from ''', srcfile(1:lsrc),'''.', nf_strerror(ierr)
            stop
          endif

          do rec=1,nrecs
            if (rec < nrecs) ierr=nf_get_vara_double(ncsrc, varid,
     &                                             rec+1,1, tnext)
            if (ierr == nf_noerr) then
              if (tunits == time_in_seconds) then   ! convert
                tnext=tnext/dble(3600*24)           ! into days
              elseif (tunits == time_in_hours) then
                tnext=tnext/dble(24)
              endif

              if (rec == 1) then                    ! here "dt" is
                dt=0.5D0*(tnext-tdays)              ! half-interval
              elseif (rec == nrecs) then            ! corresponding 
                dt=0.5*(tdays-tpast)                ! to data of
              else                                  ! current record
                dt=0.25D0*(tnext-tpast)
              endif

              tclm=tdays                            ! fold "tclm" into
              do while(tclm < 0.D0)                 ! one-year range by
                tclm=tclm+cycle_length              ! adding an integer
              enddo                                 ! number of whole
              do while(tclm > cycle_length)         ! years (cycles).
                tclm=tclm-cycle_length
              enddo
              
              tmin=tclm-dt                          ! time bounds for 
              tmax=tclm+dt                          ! current record

              if (wtmin <= tmax .and. tmin < wtmax) then
                cff=(min(tmax,wtmax)-max(tmin,wtmin))/(tmax-tmin)
                total_weight=total_weight+cff ; time=time+cff*tclm
                do i=1,vars_to_process
                  call read_by_name(ncsrc, vname(i), rec, buff)
                  do j=1,vsize(i)
                    A(start(i)+j-1)=A(start(i)+j-1) +cff*buff(j)
                  enddo
                enddo
              endif

              tpast=tdays                           ! recursively 
              tdays=tnext                           ! rotate time
            else
              write(*,'(1x,4A,I7,1x,3A/10x,A)')  '### ERROR: Cannot ',
     &            'read ''', time_var_name(1:ltvr),  ''' rec =', rec,
     &            'from ''', srcfile(1:lsrc),'''.', nf_strerror(ierr)
              stop
            endif
          enddo  !<--  nrecs

          ierr=nf_close(ncsrc) ; ncsrc=-1
        enddo   !<-- ncsrcs

        if (total_weight > 0.D0) then         ! Finalize averaging:
          cff=1.D0/total_weight               ! divide the accumulated
          time=cff*time                       ! sums by the actual
          do i=1,combined_size                ! number of records added
            A(i)=cff*A(i)                     ! up together.
          enddo

          write(*,'(1x,A,I3,1x,A,F12.5,1x,A,F12.5)') 'month =',month,
     &             'tdays =', time, 'averaged recors:', total_weight

          call put_sclr_rec_by_name_double(nctarg, trgfile,
     &                  time_var_name(1:ltvr), month, time)

          do i=1,vars_to_process
            call write_by_name(nctarg, vname(i), month, A(start(i)))
          enddo
          ierr=nf_sync(nctarg)
        endif     !<-- total_weight > 0
      enddo    !<-- month
      ierr=nf_close(nctarg) ; nctarg=-1


! Create seasonal file.  At this moment the output file "trgfile"
!------- -------- -----  becomes input and 12 monthly records are
! combined into 4 (seasons), then the resultant 4 into 1 (annual).

      write(*,'(/1x,A)') 'Computing seasonal and annual climatology...'
      srcfile=trgfile ; trgfile='seasonal.nc'
      ierr=nf_open(srcname(1), nf_nowrite, ncsrc)
      call def_clim_file(srcname(1), ncsrc, trgfile, nctarg,
     &                   vars_to_process, vid, timedimid, 5,
     &                                         cycle_length)
      ierr=nf_close(ncsrc)

      ierr=nf_open(srcfile, nf_nowrite, ncsrc)

      do j=0,11
        if (mod(j,3) == 0) then          !--> initialize summation
          time=0.D0
          do i=1,combined_size
            A(i)=0.    
          enddo
        endif

# ifdef DJF
        rec=j    !--> December, January, February, ..., November
        if (rec < 1) rec=12
# else
        rec=j+1  !--> January, February, March,    ..., December
# endif
        ierr=nf_get_vara_double(ncsrc, varid, rec,1, tdays)
# ifdef DJF
        if (j < 1) tdays=tdays-cycle_length
# endif
        time=time+tdays
        do i=1,vars_to_process
          call read_and_add(ncsrc, vname(i), rec, A(start(i)), buff)
        enddo

        if (mod(j,3) == 2) then          !--> finalize summation
          cff=1.D0/3.D0 ; time=cff*time
          do i=1,combined_size
            A(i)=cff*A(i)
          enddo

          rec=(j+1)/3                    !--> output record
          call put_sclr_rec_by_name_double(nctarg, trgfile,
     &                            time_var_name, rec, time)
          do i=1,vars_to_process
            call write_by_name(nctarg, vname(i), rec, A(start(i)))
          enddo
        endif
      enddo
      ierr=nf_close(ncsrc) ; ncsrc=-1

! Create annual fields: the following code utilizes the fact that
! at this moment scalar variable "time" and array "A" already contain
! rec=4 data corresponding to the last season computed above (e.g,
! October + November + December)/3.  Therefore there is no need to
! re-initialize summation: just add rec=1, 2, and 3 to what is already
! there and normalize.

      do rec=1,3
        ierr=nf_get_vara_double(nctarg, varid, rec,1, tdays)
        time=time+tdays
        do i=1,vars_to_process
          call read_and_add(nctarg, vname(i), rec, A(start(i)), buff)
        enddo
      enddo
      cff=0.25D0 ; time=cff*time
      do i=1,combined_size
        A(i)=cff*A(i)
      enddo
      rec=5
      call put_sclr_rec_by_name_double(nctarg, trgfile,
     &                        time_var_name, rec, time)
      do i=1,vars_to_process
        call write_by_name(nctarg, vname(i), rec, A(start(i)))
      enddo
      ierr=nf_close(nctarg)
      end
