      program coastline2mask

! Reads coastline data from a simple formatted ASCII file containing
! (latitude,longitude) coordinates for one point of the shoreline on
! each line of the file; read ROMS-style grid file, and produce land
! mask from the shoreline data.

      implicit none
      character(len=64) coastline, roms_grid_file
      integer nargs, max_pts,npts, ncgrd, ierr, nx,ny, part(4),
     &     i, ic,jc, xm,xp, x1,y1, ncr1,count, lcst,lgrd,lenstr
      real(kind=8) xmin,xmax,ymin,ymax, xc,yc
      real(kind=8), allocatable :: Points(:,:), bffr(:,:)
      real(kind=8), allocatable, dimension(:,:) :: xlon,ylat
      integer(kind=2), allocatable, dimension(:,:) :: mask
      integer(kind=4), allocatable, dimension(:,:) :: segmpt
      integer(kind=4), allocatable, dimension(:) :: icr1

#include "netcdf.inc"

      nargs=iargc() ; lcst=0 ; lgrd=0
      if (nargs == 2) then
        call getarg(1,coastline); call getarg(2,roms_grid_file)
        lcst=lenstr(  coastline);   lgrd=lenstr(roms_grid_file)
      else
        write(*,'(1x,2A/ /8x,2A/)')   'This command-line operator ',
     &         'should be used as', 'coastline2mask coastline.dat ',
     &                                         'roms_grid_file.nc'
        stop
      endif

! Read coastline data from user-supplied file

      open(unit=12, file=coastline, form='formatted', status='old')
      max_pts=256 ; allocate(Points(2,max_pts))
      npts=0 ; ierr=0
      do while(ierr == 0)
        npts=npts+1
        if (npts > max_pts) then
          allocate(bffr(2,max_pts))
          bffr(1:2,1:npts-1)=Points(1:2,1:npts-1)
          deallocate(Points)
          max_pts=2*max_pts ; write(*,*) 'extended max_pts =',max_pts
          allocate(Points(2,max_pts))
          Points(1:2,1:npts-1)=bffr(1:2,1:npts-1)
          deallocate(bffr)
        endif
        read(12,*,iostat=ierr) Points(2,npts), Points(1,npts)
      enddo
      npts=npts-1
      close(unit=12)

! Convert to integers

      xmin=Points(1,1) ; xmax=Points(1,1)
      ymin=Points(2,1) ; ymax=Points(2,1)
      do i=2,npts
        xmin=min(xmin,Points(1,i)) ; xmax=max(xmax,Points(1,i))
        ymin=min(ymin,Points(2,i)) ; ymax=max(ymax,Points(2,i))
      enddo
      xc=0.5D0*(xmin+xmax) ; yc=0.5D0*(ymin+ymax)

      allocate(segmpt(2,npts),icr1(npts))
      do i=1,npts
        segmpt(1,i)=nint(1.D+8*(Points(1,i)-xc))
        segmpt(2,i)=nint(1.D+8*(Points(2,i)-yc))
      enddo
      write(*,'(1x,A,I5,2(1x,A,F17.12))') 'Coastline data: npts =',
     &                                npts, 'xc =', xc, 'yc =', yc

! Generate land mask....

      ierr=nf_open(roms_grid_file, nf_nowrite, ncgrd)
      if (ierr == nf_noerr) then
        call roms_find_dims(ncgrd, roms_grid_file, ic,jc, i)
        nx=ic+2 ; ny=jc+2
        allocate(xlon(nx,ny), ylat(nx,ny), mask(nx,ny))
        call get_var_by_name_double(ncgrd, 'lon_rho', xlon)
        call get_var_by_name_double(ncgrd, 'lat_rho', ylat)
        ierr=nf_get_att_int(ncgrd, nf_global, 'partition', part)
        if (ierr /= nf_noerr) part(1)=-1 !<-- not found
        ierr=nf_close(ncgrd)

        do jc=1,ny
          do ic=1,nx
            mask(ic,jc)=2 !<-- initialize
            x1=nint(1.D+8*(xlon(ic,jc)-xc))
            y1=nint(1.D+8*(ylat(ic,jc)-yc))

            ncr1=0 ; xm=segmpt(1,1)
            do i=2,npts,+1   !--> recursive
              xp=segmpt(1,i)
              if (xm <= x1 .and. x1 <= xp .or.
     &            xp <= x1 .and. x1 <= xm) then
                ncr1=ncr1+1 ; icr1(ncr1)=i
              endif
              xm=xp               !<-- recursion
            enddo
            if (ncr1 > 0) then
              call count_crossings(npts,segmpt, x1,y1,ncr1,icr1, count)
              if (count /= 0) mask(ic,jc)=mask(ic,jc)+count
            endif
          enddo
        enddo
        call write_mask(nx,ny, mask, part)
      else
        write(*,*) '### ERROR: 1'
      endif
      end

      subroutine count_crossings(npts,segmpt, xs,ys, ncrs,icrs,count)
      implicit none
      integer npts, xs,ys, ncrs, count
      integer segmpt(2,npts), icrs(ncrs)
      integer i,k, xm,ym, xp,yp
      real(kind=8) crss

      count=0
      do k=1,ncrs
        i=icrs(k)
        xp=segmpt(1,i) ; xm=segmpt(1,i-1)
        yp=segmpt(2,i) ; ym=segmpt(2,i-1)

        if (xp /= xm) then
          crss=( dble(ym)*(dble(xp)-dble(xs))
     &          +dble(yp)*(dble(xs)-dble(xm))
     &                  )/(dble(xp)-dble(xm))
# ifdef VERBOSE
          if ((dble(yp)-crss)*(crss-dble(ym)) < 0.D0) write(*,*)
     &      '### ERROR: Search algorithm failure at x,y=', xs,ys
# endif
          if (crss > dble(ys)) then
            if (xs == xm .or. xs == xp) then
              i=1
            else
              i=2
            endif
            if (xp < xm) then
              count=count+i
            elseif (xp > xm) then
              count=count-i
            endif
          endif
        endif
      enddo
      end
