#ifdef REVERT
      program revert_rmask
#else
      program commit_rmask
#endif

! Reads netCDF variable "mask_rho" from ROMS grid file and forces it
! to be either 0 or 1.  This is needed only if land mask was processed
! by "single_connect -show ..." which may set "mask_rho" to values
! other that 0 (negative -1, -2) on land to visualize changes it made.

      implicit none
      character(len=80) grid_file
      real(kind=8), dimension(:,:), allocatable :: rmask
      integer nargs, iargc, ncgrd, Lm,Mm, i,j, ierr, lgrd,lenstr
#include "netcdf.inc"

      nargs=iargc()
      if (nargs==1) then
        call getarg(1,grid_file)
        lgrd=lenstr(grid_file)
        ierr=nf_open (grid_file, nf_write, ncgrd)
        if (ierr==nf_noerr) then
          call roms_find_dims(ncgrd, grid_file, Lm,Mm, i)

          allocate (rmask(0:Lm+1,0:Mm+1))
          call get_var_by_name_double(ncgrd, 'mask_rho', rmask)

          do j=0,Mm+1
            do i=0,Lm+1
#ifdef REVERT
              if (rmask(i,j) > 0.5D0 .or. rmask(i,j) < 0.D0) then
#else
              if (rmask(i,j) > 0.5D0) then
#endif
                rmask(i,j)=1.D0
              else
                rmask(i,j)=0.D0
              endif
            enddo
          enddo

          call put_var_by_name_double (ncgrd, 'mask_rho', rmask)
          ierr=nf_close (ncgrd)
        else
          write(*,'(/1x,4A/12x,A/)')   '### ERROR: Cannot open ROMS ',
     &    'grid file ''', grid_file(1:lgrd), ''' in read-write mode.',
     &                                             nf_strerror(ierr)
        endif
      else
        write(*,'(/1x,A/,/12x,A/1x,A/12x,A/4(/1x,2A),2(/12x,A)/1x,A/)')
     &       'Usage of "commit_rmask" and "revert_rmask" should be:',
     &                 'commit_rmask roms_grid_file.nc',  'or',
     &                 'revert_rmask roms_grid_file.nc',
     &       'They read "mask_rho" from the file and force it to be ',
     &     'either 0 or 1.',   'This is needed only if land mask was ',
     &     'processed by "single_connect" with', '-s or --show flag, ',
     &     'which may set "mask_rho" to land values other than 0,',
     &     'e.g., negative -1, -2 to visualize the modifications ',
     &                               'it made.', 
     &     '"commit_rmask" sets negative values to 0(land)',
     &     '"revert_rmask" resets them back to 1(water), therefore ',
     &       'reverting all the changes made by "single_connect".'
      endif
      end
