! Compute relative vorticity or barotropic PV for different cases.
!-------- -------- --------- -- ---------- -- --- --------- -------
! Legend: mutually exclusive "A"/"C" are for initially non-staggered
! grid (u-, v-, and rho-points are co-located) or Arakawa C-grid; "N"
! indicatee normalization by local Coriolis parameter; "msk" or "spv"
! or nothing (3-way) stand for having land mask array defined at RHO-
! points, or using special values at U- and V-, or no masking at all.

#ifdef NON_STAGG
# ifdef NORMALIZE
#  ifdef MASK_RHO
#   define comp_vort_TYPE_thread comp_vort_ANmsk_thread
#   define comp_vort_TYPE_tile   comp_vort_ANmsk_tile
#  elif defined SPV_UV
#   define comp_vort_TYPE_thread comp_vort_ANspv_thread
#   define comp_vort_TYPE_tile   comp_vort_ANspv_tile
#  else
#   define comp_vort_TYPE_thread comp_vort_AN_thread
#   define comp_vort_TYPE_tile   comp_vort_AN_tile
#  endif
# else
#  ifdef MASK_RHO
#   define comp_vort_TYPE_thread comp_vort_Amsk_thread
#   define comp_vort_TYPE_tile   comp_vort_Amsk_tile
#  elif defined SPV_UV
#   define comp_vort_TYPE_thread comp_vort_Aspv_thread
#   define comp_vort_TYPE_tile   comp_vort_Aspv_tile
#  else
#   define comp_vort_TYPE_thread comp_vort_A_thread
#   define comp_vort_TYPE_tile   comp_vort_A_tile
#  endif
# endif
#else
# ifdef NORMALIZE
#  ifdef MASK_RHO
#   define comp_vort_TYPE_thread comp_vort_CNmsk_thread
#   define comp_vort_TYPE_tile   comp_vort_CNmsk_tile
#  elif defined SPV_UV
#   define comp_vort_TYPE_thread comp_vort_CNspv_thread
#   define comp_vort_TYPE_tile   comp_vort_CNspv_tile
#  else
#   define comp_vort_TYPE_thread comp_vort_CN_thread
#   define comp_vort_TYPE_tile   comp_vort_CN_tile
#  endif
# else
#  ifdef MASK_RHO
#   define comp_vort_TYPE_thread comp_vort_Cmsk_thread
#   define comp_vort_TYPE_tile   comp_vort_Cmsk_tile
#  elif defined SPV_UV
#   define comp_vort_TYPE_thread comp_vort_Cspv_thread
#   define comp_vort_TYPE_tile   comp_vort_Cspv_tile
#  else
#   define comp_vort_TYPE_thread comp_vort_C_thread
#   define comp_vort_TYPE_tile   comp_vort_C_tile
#  endif
# endif
#endif

      subroutine comp_vort_TYPE_thread( Lm,Mm,N, dm_u,dn_v,
     &                                         iA_q, u,v, q
#if defined BAROTR_PV
     &                                       , f_q, h, zeta
# ifdef NORMALIZE
     &                                              , qnorm
# endif
#elif defined NORMALIZE
     &                                       , f_q
#endif
#ifdef MASK_RHO
     &                                   , rmask, FillValue
#elif defined SPV_UV
     &                                     , spv, FillValue
#endif
     &                                                    )
      implicit none
      integer Lm,Mm,N
      real(kind=4), dimension(1:Lm+1,0:Mm+1) :: dm_u
      real(kind=4), dimension(0:Lm+1,1:Mm+1) :: dn_v
      real(kind=4), dimension(1:Lm+1,1:Mm+1) :: iA_q
#if defined BAROTR_PV || defined NORMALIZE
     &                                         , f_q
#endif
#ifdef MASK_RHO
      real(kind=4), dimension(0:Lm+1,0:Mm+1) :: rmask
#endif
#ifdef BAROTR_PV
     &                                      , h, zeta
# ifdef NORMALIZE
      real(kind=4) qnorm !<-- normalization constant
# endif
#endif
#ifdef NON_STAGG
      real(kind=4), dimension(0:Lm+1,0:Mm+1,N) :: u,v
#else
      real(kind=4), dimension(1:Lm+1,0:Mm+1,N) :: u
      real(kind=4), dimension(0:Lm+1,1:Mm+1,N) :: v
#endif
      real(kind=4), dimension(1:Lm+1,1:Mm+1,N) :: q
#ifdef MASK_RHO
      real(kind=4) FillValue
#elif defined SPV_UV
      real(kind=4) spv, FillValue
#endif
      integer numthreads,trd, nsub_x,nsub_y, my_first,my_last,
     &                       range, tile, istr,iend,jstr,jend
C$    integer omp_get_num_threads, omp_get_thread_num
      numthreads=1 ; trd=0
C$    numthreads=omp_get_num_threads() ; trd=omp_get_thread_num()
      call set_tiles(Lm,Mm, nsub_x,nsub_y)
      range=(nsub_x*nsub_y +numthreads-1)/numthreads
      my_first=trd*range
      my_last=min(my_first + range-1, nsub_x*nsub_y-1)

      do tile=my_first,my_last
        call comp_tile_bounds(tile, Lm,Mm, nsub_x,nsub_y,
     &                                 istr,iend,jstr,jend)

        call comp_vort_TYPE_tile(  istr,iend,jstr,jend,   Lm,Mm,N,
     &                                     dm_u,dn_v, iA_q, u,v, q
#if defined BAROTR_PV
     &                                              , f_q, h, zeta
# ifdef NORMALIZE
     &                                                     , qnorm
# endif
#elif defined NORMALIZE
     &                                              , f_q
#endif
#ifdef MASK_RHO
     &                                          , rmask, FillValue
#elif defined SPV_UV
     &                                            , spv, FillValue
#endif
     &                                                           )
      enddo
      end


      subroutine comp_vort_TYPE_tile(istr,iend,jstr,jend, Lm,Mm,N,
     &                                     dm_u,dn_v, iA_q, u,v, q
#if defined BAROTR_PV
     &                                              , f_q, h, zeta
# ifdef NORMALIZE
     &                                                     , qnorm
# endif
#elif defined NORMALIZE
     &                                              , f_q
#endif
#ifdef MASK_RHO
     &                                          , rmask, FillValue
#elif defined SPV_UV
     &                                            , spv, FillValue
#endif
     &                                                           )
      implicit none
      integer istr,iend,jstr,jend, Lm,Mm,N, i,j,k
      real(kind=4), dimension(1:Lm+1,0:Mm+1) :: dm_u
      real(kind=4), dimension(0:Lm+1,1:Mm+1) :: dn_v
      real(kind=4), dimension(1:Lm+1,1:Mm+1) :: iA_q
#if defined BAROTR_PV || defined NORMALIZE
     &                                         , f_q
#endif
#ifdef MASK_RHO
      real(kind=4), dimension(0:Lm+1,0:Mm+1) :: rmask
#endif
#ifdef BAROTR_PV
      real(kind=4), dimension(0:Lm+1,0:Mm+1) :: h, zeta
# ifdef NORMALIZE
      real(kind=4) qnorm !<-- normalization constant
# endif
#endif
#ifdef NON_STAGG
      real(kind=4), dimension(0:Lm+1,0:Mm+1,N) :: u,v
#else
      real(kind=4), dimension(1:Lm+1,0:Mm+1,N) :: u
      real(kind=4), dimension(0:Lm+1,1:Mm+1,N) :: v
#endif
      real(kind=4), dimension(1:Lm+1,1:Mm+1,N) :: q
      real(kind=4) wgt, circ
#ifdef MASK_RHO
      real(kind=4) FillValue
#elif defined SPV_UV
      real(kind=4) FillValue, spv
#endif
#ifdef BAROTR_PV
# if defined MASK_RHO || defined SPV_UV
      real(kind=4) dpth
# endif
#endif

#include "compute_extended_bounds.h"

      wgt=0. ; circ=0.

! Note that so-called "A-grid" code below differs from C-grid only
! by assuming that velocity components u,v are co-located and are at
! the same points as tracers, or free surface, however in both cases
! vorticity is computed at vorticity points - half-grid-interval from
! rho-points in both horizontal directions. As the result, converting
! relative vorticity into barotropic PV takes exactly the same path
! for both types of grid staggering.

    
      do k=1,N
        do j=jstr,jendR
          do i=istr,iendR
#if defined MASK_RHO
            if ( rmask(i,j)+rmask(i-1,j)+rmask(i,j-1)
     &                     +rmask(i-1,j-1) < 0.5 ) then
              q(i,j,k)=FillValue
            else
#elif defined SPV_UV
# ifdef NON_STAGG
            if ( u(i  ,j,k) == spv .or. u(i  ,j-1,k) == spv  .or.
     &           u(i-1,j,k) == spv .or. u(i-1,j-1,k) == spv ) then
# else
            if ( u(i,j,k) == spv .or. u(i,j-1,k) == spv  .or.
     &           v(i,j,k) == spv .or. v(i-1,j,k) == spv ) then
# endif
              q(i,j,k)=FillValue
            else
#endif
#ifdef NON_STAGG
              q(i,j,k)=0.5*iA_q(i,j)*(
     &                  dm_u(i,j-1)*(u(i-1,j-1,k)+u(i  ,j-1,k))
     &                   +dn_v(i,j)*(v(i  ,j-1,k)+v(i  ,j  ,k))
     &                   -dm_u(i,j)*(u(i  ,j  ,k)+u(i-1,j  ,k))
     &                 -dn_v(i-1,j)*(v(i-1,j  ,k)+v(i-1,j-1,k))
     &                                                        )
#else
# ifdef SPV_UV
              wgt=0.  ; circ=0.
              if (u(i,j-1,k) /= spv) then
                wgt=wgt+1. ;  circ=circ +dm_u(i,j-1)*u(i,j-1,k)
              endif
              if (v(i,j,k) /= spv) then
                wgt=wgt+1. ;    circ=circ +dn_v(i,j)*v(i,j,k)
              endif
              if (u(i,j,k) /= spv) then
                wgt=wgt+1. ;  circ=circ   -dm_u(i,j)*u(i,j,k)
              endif
              if (v(i-1,j,k) /= spv) then
                wgt=wgt+1. ;  circ=circ -dn_v(i-1,j)*v(i-1,j,k)
              endif
              q(i,j,k)=iA_q(i,j)*circ
# else
              circ=0.
              if (-1.E+4 < u(i,j-1,k) .and. u(i,j-1,k) < 1.E+4) then
                circ=circ +dm_u(i,j-1)*u(i,j-1,k)
              endif
              if (-1.E+4 < v(i,j,k) .and. v(i,j,k) < 1.E+4) then
                circ=circ +dn_v(i,j)*v(i,j,k)
              endif
              if (-1.E+4 < u(i,j,k) .and. u(i,j,k) < 1.E+4) then
                circ=circ -dm_u(i,j)*u(i,j,k)
              endif
              if (-1.E+4 < v(i-1,j,k) .and. v(i-1,j,k) < 1.E+4) then
                circ=circ -dn_v(i-1,j)*v(i-1,j,k)
              endif
              q(i,j,k)=iA_q(i,j)*circ

c**           q(i,j,k)=iA_q(i,j)*( dm_u(i,j-1)*u(i,j-1,k)
c**  &                              +dn_v(i,j)*v(i,j,k)
c**  &                              -dm_u(i,j)*u(i,j,k)
c**  &                            -dn_v(i-1,j)*v(i-1,j,k))

# endif
#endif

#ifdef BAROTR_PV
# if defined MASK_RHO
              wgt=0. ; dpth=0.
              if (rmask(i,j) > 0.5) then
                wgt=wgt+1. ; dpth=dpth+h(i,j)+zeta(i,j)
              endif
              if (rmask(i-1,j) > 0.5) then
                wgt=wgt+1. ; dpth=dpth+h(i-1,j)+zeta(i-1,j)
              endif
              if (rmask(i,j-1) > 0.5) then
                wgt=wgt+1. ; dpth=dpth+h(i,j-1)+zeta(i,j-1)
              endif
              if (rmask(i-1,j-1) > 0.5) then
                wgt=wgt+1. ; dpth=dpth+h(i-1,j-1)+zeta(i-1,j-1)
              endif
              q(i,j,k)=(q(i,j,k)+f_q(i,j))*wgt/dpth
# elif defined SPV_UV
              wgt=0. ; dpth=0.
              if (zeta(i,j) /= spv) then
                wgt=wgt+1. ;  dpth=dpth+h(i,j)+zeta(i,j)
              endif
              if (zeta(i-1,j) /= spv) then
                wgt=wgt+1. ;  dpth=dpth+h(i-1,j)+zeta(i-1,j)
              endif
              if (zeta(i,j-1) /= spv) then
                wgt=wgt+1. ;  dpth=dpth+h(i,j-1)+zeta(i,j-1)
              endif
              if (zeta(i-1,j-1) /= spv) then
                wgt=wgt+1. ;  dpth=dpth+h(i-1,j-1)+zeta(i-1,j-1)
              endif
              q(i,j,k)=(q(i,j,k)+f_q(i,j))*wgt/dpth
# else
              q(i,j,k)=(q(i,j,k)+f_q(i,j)) * 4./( h(i,j)+zeta(i,j)
     &                  +h(i-1,j)+zeta(i-1,j) +h(i,j-1)+zeta(i,j-1)
     &                                  +h(i-1,j-1)+zeta(i-1,j-1) )
# endif
# ifdef NORMALIZE
              q(i,j,k)=qnorm*q(i,j,k)-1. !<-- normalize and zero-center
# endif
#elif defined NORMALIZE
              q(i,j,k)=q(i,j,k)*f_q(i,j) !<-- normalize by Coriolis
#endif
#if defined MASK_RHO || defined SPV_UV
            endif
#endif
          enddo
        enddo
      enddo   !<-- k
      end

#undef comp_vort_TYPE_thread
#undef comp_vort_TYPE_tile
