      subroutine def_clim_file(srcfile, ncsrc, trgfile, nctarg,
     &        vars_to_process,vid, timedimid, tsize, cycle_length)

! Given netCDF file ID of a file "srcfile" in already open state
! create target netCDF file "trgfile" having the same set of dimensions
! and global attributes are the original file, while containing only a
! subset of variable with netCDF variable IDs (going by ISs in the 
! original file) are listed in array vid(0:vars_to_process).

      implicit none
      character(len=*) srcfile, trgfile
      integer ncsrc,nctarg, vars_to_process,vid(0:vars_to_process),
     &                                             timedimid,tsize
      real(kind=8) cycle_length


      character(len=32) name
      integer ndims, nvars, ngatts, unlimdimid, old_fill_mode,
     &        varid, vartype, vardims, varatts, size, i,j, ierr,
     &                                 lsrc, ltrg, lnme, lenstr
      integer, allocatable, dimension(:) :: dimids

#include "netcdf.inc"

      lsrc=lenstr(srcfile) ; ltrg=lenstr(trgfile)
      write(*,'(/1x,3A)')  'Creating ''', trgfile(1:ltrg), '''....'
      ierr=nf_create(trgfile, nf_netcdf4, nctarg)
c**   ierr=nf_create(trgfile, nf_64bit_offset, nctarg)
      if (ierr == nf_noerr) then
        ierr=nf_set_fill(nctarg, nf_nofill, old_fill_mode)
        ierr=nf_inq(ncsrc, ndims, nvars, ngatts, unlimdimid)
        if (ierr == nf_noerr) then
          allocate(dimids(ndims))

! Copy all dimensions from the original file

c**       write(*,*) 'timedimid =',timedimid,' unlimdimid =',unlimdimid

          if (unlimdimid < 1 .or. ndims < unlimdimid) then
            unlimdimid=timedimid        !<-- unlimited dimension is
          endif                         !    not used for anything, so 
                                        !    use it as time dimension;
          do i=1,ndims
            ierr=nf_inq_dim(ncsrc, i, name, size)
            if (ierr == nf_noerr) then
              lnme=lenstr(name)            ! It is possible that time
              if (i == timedimid) then     ! dimension is not unlimited
                if (i /= unlimdimid) then  ! while unlimited is used
                  size=tsize               ! for something else: set
                else                       ! the size of time dimension
                  size=nf_unlimited        ! to user-specified fixed
                endif                      ! value in this case only; 
              elseif (i == unlimdimid) then
                size=nf_unlimited
              endif

              ierr=nf_def_dim(nctarg, name(1:lnme), size, dimids(i))
              if (ierr == nf_noerr) then
                write(*,'(4x,3A,I4)') 'Created dimension ''',
     &                          name(1:lnme), ''' =', size
              else
                write(*,'(/1x,6A/12x,A/)') '### ERROR: Cannot define ',
     &                        'dimension ''', name(1:lnme), ''' in ''',
     &                      trgfile(1:ltrg),  '''.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,2A,I3,1x,3A/12x,A/)') '### ERROR: Cannot ',
     &            'determine name and size of dimension #', i, 'in ''',
     &                      srcfile(1:lsrc), '''.',  nf_strerror(ierr)
            endif
            if (ierr /= nf_noerr) stop
          enddo

! Copy all global attributes (except signature of nc3to4z-compressed
! file because files created by this program are not compressed.)

          do i=1,ngatts
            ierr=nf_inq_attname(ncsrc, nf_global, i, name)
            if (ierr == nf_noerr) then
              lnme=lenstr(name)
              if (name(1:lnme) == 'nc_format') then
                write(*,'(8x,3A)') 'Skipping global attribute ''',
     &                                        name(1:lnme), '''.'
              else 
                ierr=nf_copy_att(ncsrc, nf_global, name(1:lnme),
     &                                       nctarg, nf_global)
                if (ierr == nf_noerr) then
                  write(*,'(8x,3A)')  'Copied global attribute ''',
     &                                         name(1:lnme), '''.'
                else
                  write(*,'(/1x,8A/12x,A/)') '### ERROR: Cannot copy ',
     &               'global attribute ''', name(1:lnme), ''' from ''',
     &                  srcfile(1:lsrc), ''' into ''', trgfile(1:ltrg),
     &                                        '''.', nf_strerror(ierr)
                endif
              endif
            else
              write(*,'(/1x,2A,I3,1x,3A/12x,A/)')      '### ERROR: ',
     &              'Cannot determine name of global attribute #', i,
     &           'in ''', srcfile(1:lsrc), '''.', nf_strerror(ierr)
            endif
            if (ierr /= nf_noerr) stop
          enddo

! Define variables: only the ones selected by the list source-file
! netCDF ivariable IDs vid(0:vars_to_process). Note that time varable
! is #0 in the list.   Copy all attributes for each variable from the
! original file on "as is" basis, except that time variable units
! becomes "days" no matter what the original, and it alos gets
! "cycle_lenght" attribute.

          do i=0,vars_to_process
            ierr=nf_inq_var(ncsrc, vid(i), name,   vartype,
     &                            vardims, dimids, varatts)
            if (ierr == nf_noerr) then
              lnme=lenstr(name)
              ierr=nf_def_var(nctarg, name(1:lnme),  vartype,
     &                                vardims, dimids, varid)
              if (ierr == nf_noerr) then
                write(*,'(6x,3A)') 'Created variable ''',
     &                               name(1:lnme), '''.'
                do j=1,varatts
                  ierr=nf_inq_attname(ncsrc, vid(i), j, name)
                  lnme=lenstr(name)
                  if (i == 0 .and. name(1:lnme) == 'units') then
                    ierr=nf_put_att_text(nctarg, varid, 'units',
     &                35, 'days from the beginning of the year')
                  else
                    ierr=nf_copy_att(ncsrc, vid(i), name(1:lnme),
     &                                              nctarg, varid)
                  endif
                  if (ierr == nf_noerr) then
                    write(*,'(8x,3A)')   'Copied attribute ''',
     &                                   name(1:lnme), '''.'
                  else
                    write(*,'(/1x,6A/12x,3A/12x,A/)')   '### ERROR: ',
     &               'Cannot copy attribute ''',      name(1:lnme),
     &               ''' for variable ''', name(1:lenstr(name)),
     &               '''',  'into ''',  trgfile(1:ltrg),
     &                                      '''.',  nf_strerror(ierr)
                  endif
                enddo

                if (i == 0) then
                  ierr=nf_put_att_double(nctarg, varid, 'cycle_length',
     &                                     nf_double, 1, cycle_length)
                  if (ierr == nf_noerr) then
                    write(*,'(8x,2A,F8.3)')  'Put attribute ',
     &                            'cycle_length =', cycle_length
                  else
                    write(*,'(/1x,4A/12x,A)')   '### ERROR: Cannot ',
     &                        'put attribute ''cycle_lenght'' into ''',
     &                   trgfile(1:ltrg), '''.', nf_strerror(ierr)
                  endif
                endif
              else
                write(*,'(/1x,4A/12x,3A/12x,A/)') '### ERROR: Cannot ',
     &               'create variable ''', name(1:lnme), ''' in ''',
     &                     trgfile(1:ltrg), '''.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,2A,I3,1x,3A/12x,A/)') '### ERROR: Cannot ',
     &                       'make inquiry for variable #', i, 'in ''',
     &                       srcfile(1:lsrc), '''.', nf_strerror(ierr)
            endif
          enddo
          ierr=nf_enddef(nctarg)
          if (ierr == nf_noerr) then
            write(*,'(1x,A/)') 'Leaving definition mode.' 
            ierr=nf_sync(nctarg)
          else
            write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot switch ''',
     &                   trgfile(1:ltrg), ''' from definition into ',
     &                         'data input mode.', nf_strerror(ierr)
            stop
          endif
          deallocate(dimids)
        else
          write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot make general ',
     &     'inquiry into ''',srcfile(1:lsrc),'''. ',nf_strerror (ierr)
        endif
      else
        write(*,'(/1x,4A/12x,A/)')       '### ERROR: Cannot create ',
     &  'netCDF file ''', trgfile(1:ltrg), ''':', nf_strerror(ierr)
      endif
      end
