#if !defined WIND_VELOCITY && !defined WIND_STRESS

      program test_wind_file

! A self-testing program for the subroutine below. It merely creates
! an empty netCDF (due to zero unlimited dimension) file, however with
! all dimensions, variables, and attributes fully set up.

! Note: at least one of the following three macros must be defined
! for the self-test program to compile.

# define WIND_VELOCITY
c--# define WIND_STRESS

# define WIND_AT_RHO_POINTS

      implicit none
      character(len=64) grid_file, frc_file
      integer Lm,Mm, ncgrd,ncfrc, i,ierr, lgrd,lenstr
# include "netcdf.inc"

      grid_file='grid.nc' ; frc_file='def_wind_test.nc'
      ierr=nf_open(grid_file, nf_nowrite, ncgrd)
      if (ierr == nf_noerr) then
        call roms_find_dims (ncgrd, grid_file, Lm,Mm,i)
        call def_wind_file (frc_file, ncfrc, Lm,Mm)
        ierr=nf_close(ncfrc)
        ierr=nf_close(ncgrd)
      else
        lgrd=lenstr(grid_file)
        write(*,'(/1x,4A/)') '### ERROR: Cannot open netCDF file ''',
     &                   grid_file(1:lgrd), ''' in read-only mode: ',
     &                                          nf_strerror(ierr)
      endif
      end
#endif

! Note that, in principle variable names, WND_TIME, UWIND, and VWIND,
! can be defined externally as CPP-macros to override the definitions
! below which can be viewed as defaults.

c--#define WIND_AT_RHO_POINTS
c--#define WIND_VELOCITY

      subroutine def_wind_file(fname, ncid, Lm,Mm)
      implicit none
      character(len=*) fname
      integer ncid, Lm,Mm, varid, ierr, old_fill_mode,
     &              u2dgrd(3),v2dgrd(3), lfnm, lenstr
#ifndef WND_TIME
# ifdef WIND_VELOCITY
#  define WND_TIME 'wnd_time'
#  define UWIND 'uwnd'
#  define VWIND 'vwnd'
# else
#  define WND_TIME 'sms_time'
#  define UWIND 'sustr'
#  define VWIND 'svstr'
# endif
#endif
#include "spval.h"
#include "netcdf.inc"

      lfnm=lenstr(fname)
      write(*,'(1x,3A)') 'def_wind_file :: creating ''',
     &                          fname(1:lfnm),  ''' ...'
      ierr=nf_create(fname, nf_64bit_offset, ncid)
      if (ierr == nf_noerr) then

! Define dimensions: xi_rho,eta_rho are always required, however the
! other two, xi_u and eta_v are needed in the case C-grided placement
! of wind components only. Note that time dimension has the same name
! as time itself (i.e., following CF-conventions).

        ierr=nf_def_dim(ncid, 'xi_rho',  Lm+2, v2dgrd(1))
#ifndef WIND_AT_RHO_POINTS
        ierr=nf_def_dim(ncid, 'xi_u',    Lm+1, u2dgrd(1))
#else
        u2dgrd(1)=v2dgrd(1)
#endif
        ierr=nf_def_dim(ncid, 'eta_rho', Mm+2, u2dgrd(2))
#ifndef WIND_AT_RHO_POINTS
        ierr=nf_def_dim(ncid, 'eta_v',   Mm+1, v2dgrd(2))
#else
        v2dgrd(2)=u2dgrd(2)
#endif
        ierr=nf_def_dim(ncid, WND_TIME, nf_unlimited, u2dgrd(3))
        v2dgrd(3)=u2dgrd(3)

! Define variables

        ierr=nf_def_var(ncid, WND_TIME, nf_double,1,u2dgrd(3),varid)
        if (ierr == nf_noerr) then
          ierr=nf_put_att_text(ncid, varid, 'long_name', 22,
     &                                      'time for  wind forcing')
          ierr=nf_put_att_text(ncid, varid, 'units',

#if TIME_SHIFT == 2000
     &                          30, 'days since 2000-01-01 00:00:00')
#elif TIME_SHIFT == 1958
     &                          30, 'days since 1958-01-01 00:00:00')
#else
     &                                                    4, 'days')   
#endif
        else
          write(*,*) '### ERROR: def_wind_file :: Cannot define ''',
     &                           WND_TIME, '''.', nf_strerror(ierr)
          stop
        endif

        ierr=nf_def_var(ncid, UWIND, nf_real, 3, u2dgrd, varid)
        if (ierr == nf_noerr) then
#ifdef WIND_VELOCITY
          ierr=nf_put_att_text(ncid, varid, 'long_name', 53,
     &    'wind velocity at 10 m above free surface, U-component')
          ierr=nf_put_att_text(ncid, varid, 'units', 3, 'm/s')
#else
          ierr=nf_put_att_text(ncid, varid, 'long_name', 24,
     &                                 'wind stress, U-component')
          ierr=nf_put_att_text(ncid, varid, 'units',10,'Newton/m^2')
#endif
          ierr=nf_put_att_real(ncid, varid, '_FillValue', nf_real, 1,
     &                                                  FillValue)
        else
          write(*,*) '### ERROR: def_wind_file :: Cannot define ''',
     &                            UWIND, '''. ', nf_strerror(ierr) 
          stop 
        endif

        ierr=nf_def_var(ncid, VWIND, nf_real, 3, v2dgrd, varid)
        if (ierr == nf_noerr) then
#ifdef WIND_VELOCITY
          ierr=nf_put_att_text(ncid, varid, 'long_name', 53,
     &    'wind velocity at 10 m above free surface, V-component')
          ierr=nf_put_att_text(ncid, varid, 'units', 3, 'm/s')
#else
          ierr=nf_put_att_text(ncid, varid, 'long_name', 24,
     &                                 'wind stress, V-component')
          ierr=nf_put_att_text(ncid, varid, 'units',10,'Newton/m^2')
#endif
          ierr=nf_put_att_real(ncid, varid, '_FillValue', nf_real, 1,
     &                                                  FillValue)
        else
          write(*,*) '### ERROR: def_wind_file :: Cannot define ''',
     &                            UWIND, '''. ', nf_strerror(ierr)     
          stop
        endif

        ierr=nf_set_fill(ncid, nf_nofill, old_fill_mode)
        ierr=nf_enddef(ncid)
        ierr=nf_sync(ncid)
        write(*,*) '... return from def_wind_file'
        return    !--> successful return
      else
        write(*,'(/1x,4A/12x,A)') '### ERROR: Cannot create netCDF ',
     &            'file ''', fname(1:lfnm), '''.', nf_strerror(ierr)
        stop
      endif
      end
