      program hmask

! Read landmask (netCDF variable "mask_rho") and topography ("hraw"
! and "vlus") from a named netCDF file specified as command line argument,
! and create masked versions of both "hraw" and "vlus" placing them into
! a separate netCDF file. This is useful for visualization purposes.

      implicit none
      character(len=128) grid
      character(len=128), allocatable :: vname(:)
      real(kind=8), parameter :: FillValue=1.D+33
      real(kind=8), allocatable, dimension(:,:) :: rmask, vlus
      integer iargc, nargs, Lm,Mm,N, ncgrd, nctarg, varid, r2dgrd(4),
     &                            m, i,j, ierr, lgrd, lvar, lenstr

#include "netcdf.inc"

      nargs=iargc()
      if (nargs > 1) then
        call getarg(1,grid) ; lgrd=lenstr(grid)
        allocate(vname(nargs-1))
        do m=1,nargs-1
          call getarg(m+1,vname(m))
        enddo
      else
        write(*,'(/1x,A/ /12x,A/)') 'Usage should be: ',
     &       'hmask grid_file.nc varname [var2 [var3]]'
        stop
      endif

      ierr=nf_open(grid(1:lgrd), nf_nowrite, ncgrd)
      call roms_find_dims(ncgrd, grid, Lm,Mm,N)
      allocate(rmask(0:Lm+1,0:Mm+1)) ; allocate(vlus(0:Lm+1,0:Mm+1))
      call get_var_by_name_double(ncgrd, 'mask_rho', rmask)

      ierr=nf_create('hmask.nc', nf_64bit_offset, nctarg)
      if (ierr==nf_noerr) then
        ierr=nf_def_dim(nctarg, 'xi_rho',  Lm+2, r2dgrd(1))
        ierr=nf_def_dim(nctarg, 'eta_rho', Mm+2, r2dgrd(2))
        do m=1,nargs-1
          lvar=lenstr(vname(m))
          ierr=nf_def_var(nctarg, vname(m)(1:lvar), nf_double,
     &                                       2, r2dgrd, varid)
          ierr=nf_put_att_double(nctarg, varid, '_FillValue',
     &                                nf_double, 1, FillValue)
        enddo
        ierr=nf_enddef(nctarg)
      else
        write(*,'(/1x,4A/12x,A)') '### ERROR: Cannot create netCDF ',
     &                      'file ''hmask.nc''.', nf_strerror(ierr)
        stop
      endif

      do m=1,nargs-1
        lvar=lenstr(vname(m))
        call get_var_by_name_double(ncgrd, vname(m)(1:lvar), vlus)
        do j=0,Mm+1
          do i=0,Lm+1
            if (rmask(i,j) < 0.5D0) then
              vlus(i,j)=FillValue
            endif
          enddo
        enddo
        call put_var_by_name_double(nctarg, vname(m)(1:lvar), vlus)
      enddo
      ierr=nf_close(nctarg)
      ierr=nf_close(ncgrd)
      end
