#if !defined U_TYPE && !defined V_TYPE

C>>>#  define LEAVE_UNMASKED_NANS
C>>>#  define LEAVE_BAD_POINTS

      module horiz_inter_arrays
       real(kind=8), allocatable, dimension(:,:) :: xi_ir, eta_jr,
     &                              xi_iu, eta_ju,  xi_iv, eta_jv
      end module



      subroutine init_interp_2d(lon,lat, xlon,ylat,  !<-- source grid
     &                          Lm,Mm, lon_r,lat_r)  !<-- target grid
      use horiz_inter_arrays
      implicit none
      integer lon,lat, Lm,Mm
      real(kind=8) xlon(lon), lon_r(0:Lm+1,0:Mm+1),
     &             ylat(lat), lat_r(0:Lm+1,0:Mm+1)

#ifdef VERBOSE
      write(*,'(2(1x,A,2I5))') 'enter init_interp_2d, lon,lat =',
     &                               lon,lat,  'Lm,Mm =', Lm,Mm
#endif
      if (.not.allocated(xi_ir)) then
        allocate( xi_ir(0:Lm+1,0:Mm+1), eta_jr(0:Lm+1,0:Mm+1))
        allocate( xi_iu(  Lm+1,0:Mm+1), eta_ju(  Lm+1,0:Mm+1))
        allocate( xi_iv(0:Lm+1,  Mm+1), eta_jv(0:Lm+1,  Mm+1))
      endif

      call init_interp_r2d(lon,lat, xlon,ylat, Lm,Mm, lon_r,lat_r)
      call init_interp_u2d(lon,lat, xlon,ylat, Lm,Mm, lon_r,lat_r)
      call init_interp_v2d(lon,lat, xlon,ylat, Lm,Mm, lon_r,lat_r)
      end


      subroutine comp_grid_bounds(Lm,Mm, iwest,ieast,jsouth,jnorth)
      use horiz_inter_arrays
      implicit none
      integer Lm,Mm, iwest,ieast,jsouth,jnorth, i,j
      real(kind=8) xi_west,xi_east, eta_south,eta_north

#ifdef VERBOSE
      write(*,'(1x,A,2I5)') 'enter comp_grid_bounds, Lm,Mm =',Lm,Mm
#endif

      xi_west=+10.D+33  ;  eta_south=+10.D+33
      xi_east=-10.D+33  ;  eta_north=-10.D+33

      do j=0,Mm+1
        do i=0,Lm+1
          if (xi_ir(i,j) < xi_west) xi_west=xi_ir(i,j)
          if (xi_ir(i,j) > xi_east) xi_east=xi_ir(i,j)

          if (eta_jr(i,j) < eta_south) eta_south=eta_jr(i,j)
          if (eta_jr(i,j) > eta_north) eta_north=eta_jr(i,j)
        enddo
      enddo

c      write(*,*) xi_west, xi_east, eta_south, eta_north
# ifdef XXX
      iwest=int(xi_west)-2  ; jsouth=int(eta_south)-2
      ieast=int(xi_east)+3  ; jnorth=int(eta_north)+3
# else
      iwest=int(xi_west)-1  ; jsouth=int(eta_south)-1
      ieast=int(xi_east)+2  ; jnorth=int(eta_north)+2
# endif

      write(*,'(2(1x,A,2I5))') 'Requested subdomain iwest:ieast =',
     &              iwest,ieast, 'jsouth:jnorth =', jsouth,jnorth
      end
#endif  /* !defined U_TYPE && !defined V_TYPE */




! Macro L_WATER_COND is logical condition for "water" on ROMS grid as
! function of "msk" defined respectively for RHO-, U-, and V-points.
! Masking of U and V is done in a somewhat milder way relatively to the
! actual U- V-point masking algorithm in the model by leaving normal
! velocity points at the coastline be unmasked. Note .OR. logic instead
! of the usual multiplication by products like msk(i,j)*msk(i-1,j),
! which is equivalent to .AND. logical operation.


#if defined U_TYPE
# define interp_TYPE interp_u2d
# define init_interp_TYPE init_interp_u2d
# define xii xi_iu
# define etaj eta_ju
# define imin 1
# define jmin 0
# define L_WATER_COND msk(i,j)==1 .OR. msk(i-1,j)==1
#elif defined V_TYPE
# define interp_TYPE interp_v2d
# define init_interp_TYPE init_interp_v2d
# define xii xi_iv
# define etaj eta_jv
# define imin 0
# define jmin 1
# define L_WATER_COND msk(i,j)==1 .OR. msk(i,j-1)==1
#else
# define interp_TYPE interp_r2d
# define init_interp_TYPE init_interp_r2d
# define xii xi_ir
# define etaj eta_jr
# define imin 0
# define jmin 0
# define L_WATER_COND msk(i,j)==1
#endif

      subroutine init_interp_TYPE(lon,lat,xlon,ylat, Lm,Mm,lon_r,lat_r)
      use horiz_inter_arrays
      implicit none
      integer lon,lat, Lm,Mm,    imax,jmax, i,j, ic,jc
      real(kind=8) xlon(lon), lon_r(0:Lm+1,0:Mm+1), xL,xR,
     &             ylat(lat), lat_r(0:Lm+1,0:Mm+1), xi,eta,
     &             xi_min,xi_max, eta_min,eta_max, cffX,cffE

#ifdef VERBOSE
      write(*,'(2(1x,A,2I5))') 'enter init_interp_TYPE, lon,lat =',
     &                                 lon,lat,  'Lm,Mm =', Lm,Mm
#endif
      imax=Lm+1        ; jmax=Mm+1
      xi_max =-1.D+33  ; eta_max=-1.D+33
      xi_min =+1.D+33  ; eta_min=+1.D+33

      do j=jmin,jmax
        do i=imin,imax
#if defined U_TYPE
          xi= 0.5D0*(lon_r(i,j)+lon_r(i-1,j))
          eta=0.5D0*(lat_r(i,j)+lat_r(i-1,j))
#elif defined V_TYPE
          xi= 0.5D0*(lon_r(i,j)+lon_r(i,j-1))
          eta=0.5D0*(lat_r(i,j)+lat_r(i,j-1))
#else
          xi= lon_r(i,j)   ;   eta=lat_r(i,j)
#endif
          xi_max=max(xi_max, xi)  ; eta_max=max(eta_max,eta)
          xi_min=min(xi_min, xi)  ; eta_min=min(eta_min,eta)
        enddo
      enddo

      write(*,'(2(1x,A,2F12.6))') 'requested bounds:   lon=',
     &               xi_min,xi_max, 'lat=', eta_min,eta_max

      write(*,'(2(1x,A,2F12.6))') 'avail. data bounds: lon=',
     &          xlon(1),xlon(lon), 'lat=', ylat(1),ylat(lat)

      if ( xi_min < xlon(1) .or. xi_max > xlon(lon)) then
        write(*,*) 'folding required'
      endif


      cffX=dble(lon-1)/(xlon(lon)-xlon(1))
      cffE=dble(lat-1)/(ylat(lat)-ylat(1))

c      do j=1,lat-1
c         write(*,*) ylat(j), ylat(j+1)-ylat(j),
c     &          ylat(j+1)-ylat(j)-1.D0/cffE
c      enddo

      do j=jmin,jmax
        do i=imin,imax
#if defined U_TYPE
          xi=0.5D0*(lon_r(i,j)+lon_r(i-1,j))
          eta=0.5D0*(lat_r(i,j)+lat_r(i-1,j))
#elif defined V_TYPE
          xi=0.5D0*(lon_r(i,j)+lon_r(i,j-1))
          eta=0.5D0*(lat_r(i,j)+lat_r(i,j-1))
#else
          xi=lon_r(i,j)
          eta=lat_r(i,j)
#endif
          if (xi < xlon(1)) then
            xi=xi+360.D0
          elseif (xi > xlon(1)+360.D0) then
            xi=xi-360.D0
          endif

          xii(i,j) =1.D0 + cffX*(xi -xlon(1))
          etaj(i,j)=1.D0 + cffE*(eta-ylat(1))

          ic=int(xii(i,j)) ; jc=int(etaj(i,j))

          xL=xlon(ic)
          if (ic < lon) then
            xR=xlon(ic+1)
          else
            xR=xlon(1)+360.D0
          endif

c**      if (i==100) write(*,*) i,j, etaj(i,j)-dble(jc)


          do while ( (xR-xi)*(xi-xL) < 0.D0 .or.
     &     (ylat(jc+1)-eta)*(eta-ylat(jc)) < 0.D0)
            xii(i,j) =dble(ic) + (xi-xL)/(xR-xL)
            etaj(i,j)=dble(jc) + (eta-ylat(jc))/(ylat(jc+1)-ylat(jc))

            ic=int(xii(i,j)) ; jc=int(etaj(i,j))
            xL=xlon(ic)
            if (ic < lon) then
              xR=xlon(ic+1)
            else
              xR=xlon(1)+360.D0
            endif

c**         if (i==100) write(*,*) i,j, etaj(i,j)-dble(jc), '<<<<'
          enddo


          xii(i,j) =dble(ic) + (xi-xL)/(xR-xL)
          etaj(i,j)=dble(jc) + (eta-ylat(jc))/(ylat(jc+1)-ylat(jc))

          if ((xR-xi )*(xi-xL)<0.D0 .or.
     &        (ylat(jc+1)-eta)*(eta-ylat(jc))<0.D0) then
             write(*,'(/1x,2A,2I4,2(1x,A,F10.7)/)')   '### ERROR: ',
     &            'init_interp_2d :: Algorithm failure: i,j =', i,j,
     &            'dxi =',(xi -xL)/(xR-xL),
     &              'deta =', (eta-ylat(jc))/(ylat(jc+1)-ylat(jc))
             stop
          endif
        enddo
      enddo
#ifdef VERBOSE
      write(*,*) 'leaving init_interp_TYPE'
#endif
      end

      subroutine interp_TYPE(lon,lat,src, Lm,Mm, msk, targ)
      use horiz_inter_arrays
      implicit none
      integer lon,lat, Lm,Mm,   imax,jmax
      real(kind=4) src(lon,lat), targ(imin:Lm+1,jmin:Mm+1)
      integer(kind=2) msk(0:Lm+1,0:Mm+1)
!>
      real(kind=8), dimension(:), allocatable :: pth
      integer, dimension(:), allocatable :: ipth,jpth
      integer i,j, ic,ic1, jc,  nmax,ip,np
      real(kind=8) xi,eta, cff,wgt,vlu
#include "spval.h"

      imax=Lm+1 ; jmax=Mm+1

! First stage, apply bi-linear interpolation wherever possible, while
! avoiding special values in "src" field.   Here the 4-point stencil of
! bi-linear interpolation naturally degrades to 3-, 2-point, on single
! nearest point algorithm, when some of "src" values are not available.
! Still, some of the target values cannot computed this way and need to
! be filled in later by a continuation algorithm.  Determine the number
! of these "bad" points,

      nmax=0
      do j=jmin,jmax
        do i=imin,imax
          if (L_WATER_COND) then
            ic=int(xii(i,j))
            if (ic<lon) then
              ic1=ic+1
            else
              ic1=1
            endif

            jc=int(etaj(i,j))
            xi =xii(i,j) -dble(ic)
            eta=etaj(i,j)-dble(jc)

c**         targ(i,j)=(1.D0-xi)*(1.D0-eta)*src(ic ,jc  )
c**  &               +(1.D0-xi)*      eta *src(ic ,jc+1)
c**  &                     +xi *(1.D0-eta)*src(ic1,jc  )
c**  &                     +xi *      eta *src(ic1,jc+1)

            wgt=0.D0
            vlu=0.D0
            if (src(ic,jc)<spv) then
              cff=(1.D0-xi)*(1.D0-eta)
              wgt=wgt +cff ; vlu=vlu +cff*src(ic,jc)
            endif
            if (src(ic,jc+1)<spv) then
              cff=(1.D0-xi)*eta
              wgt=wgt +cff ; vlu=vlu +cff*src(ic,jc+1)
            endif
            if (src(ic1,jc)<spv) then
              cff=xi*(1.D0-eta)
              wgt=wgt +cff ; vlu=vlu +cff*src(ic1,jc)
            endif
            if (src(ic1,jc+1)<spv) then
              cff=xi*eta
              wgt=wgt +cff ; vlu=vlu +cff*src(ic1,jc+1)
            endif
            if (wgt>0.5D0) then
              targ(i,j)=vlu/wgt
            else

#ifdef LEAVE_BAD_POINTS
              targ(i,j)=0.D0
#else
              targ(i,j)=spv_set    !<-- "bad" point
              nmax=nmax+1
#endif
            endif
          else                     ! Temporarily label values under
            targ(i,j)=spv_set      ! ROMS mask as "spv_set", so that
          endif                    ! they can be avoided by the
        enddo                      ! continuation algorithm below.
      enddo
c     write(*,*) 'nmax=',nmax

! Second stage: continuation algorithm for "bad" points, if any.
! From now on, everything what is going on here occurs on ROMS grid,
! without any participation of the original "src" data.  At first,
! allocate arrays necessary for bookkeeping: "ipth" and "jpth" are
! intended to keep indices of bad points;  while "pth" is for patch
! values computed using nearest-point algorithm. Each iteration of
! continuation algorithm has of two stages: first compute all patch
! values which can be computed using only valid nearest points, and,
! subsequently, run through the whole list again to patch 2D "targ"
! field, and re-record the list of remaining bad points.


      if (nmax > 0) then
        allocate(pth(nmax)) ; allocate(ipth(nmax))
                              allocate(jpth(nmax))
        nmax=0
        do j=jmin,jmax
          do i=imin,imax
            if (L_WATER_COND) then
              if (targ(i,j)>spv) then
                nmax=nmax+1
                ipth(nmax)=i         ! record indices
                jpth(nmax)=j         ! of bad points
              endif
            endif
          enddo
        enddo

        cff=0.71D0
        do while(nmax > 0)
c--       write(*,*) nmax
          do ip=1,nmax
            i=ipth(ip) ; wgt=0.D0
            j=jpth(ip) ; vlu=0.D0
            if (i < imax) then
              if (targ(i+1,j) < spv) then
                wgt=wgt+1.D0 ; vlu=vlu+targ(i+1,j)
              endif
            endif
            if (i > imin) then
              if (targ(i-1,j) < spv) then
                wgt=wgt+1.D0 ; vlu=vlu+targ(i-1,j)
              endif
            endif
            if (j < jmax) then
              if (targ(i,j+1) < spv) then
                wgt=wgt+1.D0 ; vlu=vlu+targ(i,j+1)
              endif
            endif
            if (j > jmin) then
              if (targ(i,j-1) < spv) then
                wgt=wgt+1.D0 ; vlu=vlu+targ(i,j-1)
              endif
            endif
            if (i < imax .and. j < jmax) then
              if (targ(i+1,j+1) < spv) then
                wgt=wgt +cff ; vlu=vlu +cff*targ(i+1,j+1)
              endif
            endif
            if (i > imin .and. j < jmax) then
              if (targ(i-1,j+1) < spv) then
                wgt=wgt +cff ; vlu=vlu +cff*targ(i-1,j+1)
              endif
            endif
            if (i > imin .and. j > jmin) then
              if (targ(i-1,j-1) < spv) then
                wgt=wgt +cff ; vlu=vlu +cff*targ(i-1,j-1)
              endif
            endif
            if (i < imax .and. j > jmin) then
              if (targ(i+1,j-1) < spv) then
                wgt=wgt +cff ; vlu=vlu +cff*targ(i+1,j-1)
              endif
            endif
            if (wgt > 0.8D0) then   ! 2.4
              pth(ip)=vlu/wgt
            else
              pth(ip)=spv_set
            endif
          enddo  !<-- ip

          np=0
          do ip=1,nmax,+1    !---> irreversible
            if (pth(ip) < spv) then
              targ(ipth(ip),jpth(ip))=pth(ip)
            else
              np=np+1                ! re-record bad points
              ipth(np)=ipth(ip)      ! remaining after the
              jpth(np)=jpth(ip)      ! present iteration
            endif
          enddo
          if (np < nmax) then
            nmax=np
          else
            write(*,'(/1x,2A/5x,A,I6,1x,A/5x,A/)')   '### WARNING: ',
     &         'Continuation in "interp_2d" is incomplete.',  'Left',
     &         nmax, 'points unfilled.  Setting then to -999999999.',
     &         'Check for isolated data-less areas in land mask.'
            do ip=1,nmax
              targ(ipth(ip),jpth(ip))=-999999999.D0
            enddo
            nmax=0  !<--- to break do while

          endif
        enddo   !<-- while

        deallocate(jpth)
        deallocate(ipth)
        deallocate(pth)
      endif
#ifndef LEAVE_UNMASKED_NANS
      do j=jmin,jmax
        do i=imin,imax
          if (.not.(L_WATER_COND)) then
            targ(i,j)=0.0
          endif
        enddo
      enddo
#endif
      end

#undef interp_TYPE
#undef init_interp_TYPE
#undef xii
#undef etaj
#undef imin
#undef jmin
#undef L_WATER_COND

#if !defined U_TYPE && !defined V_TYPE
# define U_TYPE
# include "interp_2d.F"
# undef U_TYPE
# define V_TYPE
# include "interp_2d.F"
#endif
