      program isoterm

! Read series of ROMS output files, compute depth of a specified
! isoterm and put it into a separate netCDF file.

      implicit none
      real(kind=8) ocean_time, hc, cff
      real(kind=4),dimension(:), allocatable :: T_iso,z_iso, var,bffr
      real(kind=8),dimension(:), allocatable :: Cs_r,Cs_w, h,rmask,
     &                                                         z_r,z_w
      character(len=64),dimension(:), allocatable :: fname
      character trg_fname*64,  string*64, vname*8, VertCoordType*16

      real(kind=4),parameter :: FillValue=+99999.,missing_value=-99999.

      integer nargs,  iarg,   first_file,  last_file,  indx, ierr,
     &        ncsrc,  nctarg, dimids(8),   tg_dimids(8),  tg_ndims,
     &        vid_time, vid_t_iso, vid_z_iso, max_rec, rec, rec_out,
     &        Lm,Mm,N, n_iso, max_horiz_pts, i, lsrc, ltrg, lvar, lstr
      logical miss_file_flag

      integer read_str_dble, lenstr
#include "netcdf.inc"
#define TIMING
#ifdef TIMING
      real(kind=4) tstart, run_time
      integer iclk_start, iclk_end, iclc_rate, iclc_max

      call cpu_time(tstart)
      call system_clock (iclk_start, iclc_rate, iclc_max)
#endif
                                       ! Sort out arguments in the
      n_iso=0                          ! command line: at least two
      nargs=iargc()                    ! arguments must be present:
      miss_file_flag=.true.            ! the first one (or several
      if (nargs > 1) then              ! ones) must be numbers, the
        allocate (T_iso(nargs))        ! remaining are expected to
        iarg=0                         ! be file names.
        ierr=0
        do while (ierr==0 .and. iarg<nargs)
          iarg=iarg+1
          call getarg(iarg,string)
          lstr=lenstr(string)
c??       read(string(1:lstr),*,iostat=ierr) cff
          ierr=read_str_dble(string(1:lstr), cff)
          if (ierr==0) then
            n_iso=n_iso+1
            T_iso(n_iso)=cff
            write(*,'(4x,A,I2,A,F11.4)') 'T_iso(',n_iso,') =',
     &                                         T_iso(n_iso)
          elseif (iarg==1) then
            write(*,'(/1x,4A/)')  '### ERROR: Cannot read isotermal ',
     &       'value from the first argument ''', string(1:lstr), '''.'
          endif
        enddo
        if (n_iso==0) then
          write(*,'(/1x,2A/)')   '### ERROR: At least one isotermal ',
     &                                      'value must be specified.'
        elseif (n_iso<nargs) then
          miss_file_flag=.false.
          last_file=nargs-n_iso-1
          allocate (fname(0:last_file))
          do indx=0,last_file
            iarg=indx+n_iso+1
            call getarg(iarg,fname(indx))
            lsrc=lenstr(fname(indx))
            open(unit=11, file=fname(indx), status='old', iostat=ierr)
            if (ierr==0) then
              close(unit=11)
              write(*,'(4x,I3,1x,3A)')  indx,  'file =''',
     &                                  fname(indx)(1:lsrc), ''''
            else
              write(*,'(/1x,3A/)')   '### ERROR: Cannot find file ''',
     &                                      fname(indx)(1:lsrc), '''.'
              miss_file_flag=.true.
            endif
          enddo
        else
          write(*,'(/1x,A/)') '### ERROR: No file names are given.'
        endif
      else
        write(*,'(2(/1x,2A)/1x,A/ /8x,A/1x,A/8x,2A/3(/1x,2A)/)')
     &     'This command-line operator "isoterm" takes one or more ',
     &     'ROMS-conforming',  'netCDF files and computes depth(s) ',
     &     'of the desired isoterms.   The usage',      'should be:',
     &     'isoterm T1 [T2 T3...]  file1.nc [file2.nc file3.nc ...]',
     &     'or',         'isoterm T1 [T2 T3...]  grid.nc  file1.nc ',
     &     '[file2.nc file3.nc ...]', 'where T1...TN is the set of ',
     &     'desired temperatures, and the first file,',
     &     'either file1.nc or grid.nc, must contain bottom ',
     &     'topography ''h'' and,',    'optionally, land mask data ',
     &                                               '''mask_rho''.'
      endif
      if (n_iso==0 .or. miss_file_flag) stop   !--> ERROR


! The first file, index=0 must have topography, optionally it may
! contain land mask, and it may also contain data as well (i.e., be
! a history file with grid data written in. So, after determining
! horizontal dimensions, and reading topography, also check whether
! or not vertical dimension is present, and if present, then check
! the presence of vertical grid transformation attributes, 'Cs_r',
! 'Cs_w', 'hc': if all of them exist (hence vertical transform is
! fully defined from this file alone) include this file into the
! list of files to be processed (set first_file=0); otherwise use
! it purely as a grid file.

      N=0
      max_horiz_pts=0
      indx=0
      lsrc=lenstr(fname(indx))
      ierr=nf_open (fname(indx), nf_nowrite, ncsrc)
      if (ierr == nf_noerr) then
        call roms_find_dims (ncsrc, fname(indx), Lm,Mm,N)
        max_horiz_pts=(Lm+2)*(Mm+2)
        allocate (h(max_horiz_pts))
        allocate (rmask(max_horiz_pts))
        call get_var_by_name_double(ncsrc, 'h', h)
        if (ierr==nf_noerr) then
          ierr=nf_inq_varid (ncsrc, 'mask_rho', i)
          if (ierr==nf_noerr) then
            ierr=nf_get_var_double (ncsrc, i, rmask)
            if (ierr/=nf_noerr) then
              write(*,*)  '### ERROR: Cannot read ''mask_rho'' ',
     &            'from ROMS grid file ''',  fname(indx)(1:lsrc),
     &                                 ''': ', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/14x,2A/)')   '### WARNING: Cannot ',
     &                             'find land mask from file ''',
     &             fname(indx)(1:lsrc), ''': Assuming rmask = 1',
     &                       'everywhere:  ', nf_strerror(ierr)
            do i=1,max_horiz_pts
              rmask(i)=1.D0
            enddo
           ierr=nf_noerr
          endif
          first_file=min(1,last_file)
          if (N>0) then
            allocate (bffr(2*(N+1)))
            ierr=nf_get_att_double (ncsrc, nf_global,'Cs_r', bffr)
            if (ierr == nf_noerr) then
              ierr=nf_get_att_double (ncsrc, nf_global,'Cs_w',bffr)
              if (ierr == nf_noerr) then
                ierr=nf_get_att_double (ncsrc, nf_global,'hc',bffr)
                if (ierr == nf_noerr) then
                  first_file=0
                endif
              endif
            endif
            deallocate(bffr)
            ierr=nf_noerr
          endif
          ierr=nf_close(ncsrc)
          if (ierr/=nf_noerr) then
            write(*,*)     '### ERROR: Cannot close grid file ''',
     &             fname(indx)(1:lsrc), ''': ', nf_strerror(ierr)
          endif
        else
          write(*,*)  '### ERROR: Cannot read  ROMS grid file ''',
     &             fname(indx)(1:lsrc), ''': ', nf_strerror(ierr)
        endif
      else
        write(*,*) '### ERROR: Cannot open  ROMS grid file ''',
     &               fname(indx)(1:lsrc), ''': ', nf_strerror(ierr)
      endif
      if (ierr /= nf_noerr) stop
!
!      *****    *********    ******   *******    *********
!    ***   ***  *  ***  *   **  ***   ***   ***  *  ***  *
!    ***           ***     **   ***   ***   ***     ***
!      *****       ***    ***   ***   ***   **      ***
!          ***     ***    *********   ******        ***
!    ***   ***     ***    ***   ***   ***  **       ***
!      *****       ***    ***   ***   ***   ***     ***
!
! Note: "check_dims" followed by "find_dims" just below is needed
! because the first netCDF file may be either grid, or data file, so
! it may or may not have vertical dimension (hence resulting in N=0).
! Consequently if all three dimensions are already found, they must
! agree; otherwise, check only horizontal, but find vertical.

      do indx=first_file,last_file
        lsrc=lenstr(fname(indx))
        ierr=nf_open (fname(indx), nf_nowrite, ncsrc)
        if (ierr == nf_noerr) then
          ierr=nf_inq_varid (ncsrc, 'temp', i)
          if (ierr==nf_noerr) then
            call roms_check_dims(ncsrc, fname(indx), Lm,Mm,N)
            call roms_find_dims (ncsrc, fname(indx), Lm,Mm,N)

            if (.not.allocated(Cs_r)) then
              allocate(Cs_r(N))
              allocate(Cs_w(0:N))
              allocate(z_r(max_horiz_pts*N))
              allocate(z_w(max_horiz_pts*(N+1)))
              allocate(var(max_horiz_pts*N))
              allocate(z_iso(max_horiz_pts*n_iso))

              call read_scoord (ncsrc,N, Cs_r,Cs_w,hc, VertCoordType)
              call set_depth   (Lm,Mm,N, Cs_r,Cs_w,hc, VertCoordType,
     &                                                    h, z_r,z_w)
            endif
            max_rec=0
            ierr=nf_inq_varid (ncsrc, 'ocean_time', vid_time)
            if (ierr==nf_noerr) then
              ierr=nf_inq_vardimid (ncsrc, vid_time, dimids)
              if (ierr==nf_noerr) then
                ierr=nf_inq_dimlen (ncsrc, dimids, max_rec)
                if (ierr==nf_noerr) then
                  write(*,'(1x,3A,I4)')        'processing file ''',
     &              fname(indx)(1:lsrc), ''', max_rec =', max_rec
                else
                  write(*,'(/1x,2A,I3,1x,3A/14x,A/)') '### WARNING: ',
     &                'Cannot find length of  dimension #', dimids(1),
     &                 'in file ''',    fname(indx)(1:lsrc),    '''.',
     &                                              nf_strerror(ierr)
                endif
              else
                write(*,'(/1x,4A/14x,A/)')'### WARNING: Cannot find ',
     &              'dimension IDs for variable ''ocean_time'' in ''',
     &               fname(indx)(1:lsrc),  '''.',   nf_strerror(ierr)
              endif
            else
              write(*,'(1x,4A/14x,A)')   '### WARNING: Cannot find ',
     &          'variable ''ocean_time'' in ''', fname(indx)(1:lsrc),
     &                                    '''.',   nf_strerror(ierr)
            endif

! Create target netCDF file: define only those dimensions and
! variables which are actually used, e.g., suppress time dimension
! if only one record is found above, and suppress the dimension
! associated with the number of isoterms, if only one isoterm is
! specified.  Conversely, if multiple isoterms are requested,
! create both vertical dimension AND vertical variable to store
! values of temperature at each isoterm, and use THE SAME name for
! both: following this convention makes "ncview" do display the
! temperature value for the specific isoterm in the dimension box
! instead of just vertical index of the array.

            trg_fname='t_iso_'/ /fname(indx)(1:lsrc)
            ltrg=lenstr(trg_fname)
            ierr=nf_create(trg_fname, nf_64bit_offset, nctarg)
            if (ierr==nf_noerr) then
              ierr=nf_def_dim(nctarg, 'xi_rho',  Lm+2, tg_dimids(1))
              ierr=nf_def_dim(nctarg, 'eta_rho', Mm+2, tg_dimids(2))
              tg_ndims=2
              if (n_iso>1) then
                tg_ndims=tg_ndims+1
                ierr=nf_def_dim(nctarg, 'T_iso', n_iso, tg_dimids(3))
              endif
              if (max_rec>0) then
                tg_ndims=tg_ndims+1
                ierr=nf_def_dim(nctarg, 'time', nf_unlimited,
     &                                       tg_dimids(tg_ndims))
              endif
              if (n_iso>1) then
                ierr=nf_def_var(nctarg, 'T_iso', nf_real, 1,
     &                               tg_dimids(3), vid_t_iso)
              endif
              if (max_rec>0) then
                ierr=nf_def_var(nctarg, 'ocean_time', nf_double, 1,
     &                                      tg_dimids(tg_ndims), i)
                ierr=nf_put_att_text(nctarg, i, 'units', 5,'days')
              endif
              vname='zt_iso'
              lvar=lenstr(vname)
              ierr=nf_def_var (nctarg, vname(1:lvar), nf_real,
     &                           tg_ndims, tg_dimids, vid_z_iso)
              if (n_iso==1) then
                write(string,'(A,F5.2,A)')    'Depth of T=',
     &                                T_iso(1), 'C isoterm'
              else
                write(string,'(A)')    'isoterm depth'
              endif
              lstr=lenstr(string)
              ierr=nf_put_att_text (nctarg, vid_z_iso, 'long_name',
     &                                        lstr, string(1:lstr))

              ierr=nf_put_att_text (nctarg, vid_z_iso, 'units',
     &                                             6, 'meters')
              ierr=nf_put_att_real (nctarg, vid_z_iso, '_FillValue',
     &                                       nf_real, 1, FillValue)
              ierr=nf_enddef(nctarg)
              write(*,'(1x,3A/)')   'Created netCDF file ''',
     &                               trg_fname(1:ltrg), '''.'

            else
              write(*,'(/1x,4A/12x,A)') '### ERROR: Cannot create ',
     &                   'netCDF file ''', trg_fname(1:ltrg), '''.',
     &                                            nf_strerror(ierr)
              stop
            endif
            if (n_iso>1) then
              call write_by_name (nctarg, 'T_iso', 0, T_iso)
            endif

            rec_out=0
            do rec=1,max(max_rec,1)  !<-- at least one time
              rec_out=rec_out+1
              ierr=nf_get_vara_double(ncsrc,vid_time,rec,1,ocean_time)
              if (ierr==nf_noerr) then
                ocean_time=ocean_time/86400.D0 !-> convert sec to days

                call put_sclr_rec_by_name_double(nctarg, trg_fname,
     &                           'ocean_time', rec_out, ocean_time)

              else
                write(*,'(/1x,2A,I4,1x,3A/14x,A/)')  '### WARNING: ',
     &          'Cannot read rec =',rec, 'of ''ocean_time'' from ''',
     &                 fname(indx)(1:lsrc), '''.', nf_strerror(ierr)
              endif
              call read_by_name (ncsrc, 'temp', rec, var)

C$OMP PARALLEL SHARED(Lm,Mm,N,n_iso, rmask,z_r,z_w, var, T_iso,z_iso)

              call compute_isoterm (Lm,Mm,N,n_iso, rmask, z_r,z_w,
     &                                             var, T_iso, z_iso)
C$OMP END PARALLEL
              call write_by_name (nctarg, vname, rec_out, z_iso)

c**           ierr=nf_sync(nctarg)
            enddo   !<-- rec
            ierr=nf_close(nctarg)
            ierr=nf_close(ncsrc)

          else
            write(*,'(/1x,4A/12x,A/)')   '### WARNING: Cannot find ',
     &          'temperature as netCDF variable ''temp'' in file ''',
     &           fname(indx)(1:lsrc),  ''': --> skipping this file.',
     &                                             nf_strerror(ierr)
          endif
        else
          write(*,'(1x,4A/12x,A)')  '### ERROR: Cannot open netCDF ',
     &             'file ''', fname(indx)(1:lsrc), ''' for reading.',
     &                                             nf_strerror(ierr)
        endif
      enddo  !<-- indx
#ifdef TIMING
      call cpu_time(run_time) ; run_time=run_time-tstart 
      write(*,'(/3x,A,F11.2,1x,A)') 'cpu time =', run_time, 'sec'

      call system_clock (iclk_end, iclc_rate, iclc_max)
      if (iclc_rate>0) then
        write(*,'(/1x,A,F20.3,1x,A/)')  'Elapsed wall-clock time =',
     &           float(iclk_end-iclk_start)/float(iclc_rate), 'sec'
      endif
#endif
      end



      subroutine compute_isoterm(Lm,Mm,N, n_iso, rmask, z_r, z_w,
     &                                         var, v_iso, z_iso)
      implicit none
      integer Lm,Mm,N, n_iso, istr,iend,jstr,jend, i,j,k,m
      real(kind=8) rmask(0:Lm+1,0:Mm+1),   z_r(0:Lm+1,0:Mm+1,N),
     &                                     z_w(0:Lm+1,0:Mm+1,0:N)
      real(kind=4) var(0:Lm+1,0:Mm+1,N), z_iso(0:Lm+1,0:Mm+1,n_iso),
     &                           v_iso(n_iso),    value, cff1, cff2
      integer km(0:Lm+1), numthreads,trd, chunk_size,margin
C$    integer omp_get_num_threads, omp_get_thread_num

      numthreads=1 ; trd=0
C$    numthreads=omp_get_num_threads() ; trd=omp_get_thread_num()

      chunk_size=(Mm + numthreads-1)/numthreads
      margin=(chunk_size*numthreads -Mm)/2
      jstr=max(1,   1 + trd*chunk_size -margin)
      jend=min(Mm,  (trd+1)*chunk_size -margin)

      if (jstr==1) jstr=0
      if (jend==Mm) jend=Mm+1

c**   write(*,*) 'trd,jstr,jend =', trd,jstr,jend

      istr=0 ; iend=Lm+1

      do j=jstr,jend
        do m=1,n_iso
          value=v_iso(m)
          do i=istr,iend
            km(i)=0
          enddo
          do k=N,1,-1
            do i=istr,iend
              if (rmask(i,j) > 0.D0 .and. km(i) == 0 .and.
     &                        var(i,j,k) <= value) km(i)=k
            enddo
          enddo
          do i=istr,iend
            if (rmask(i,j) > 0.D0) then
              if (km(i) == N) then
                cff1=value-var(i,j,N)
                cff2=var(i,j,N)-var(i,j,N-1)
                if (cff1*cff2 > 0.) then
                  z_iso(i,j,m)=z_r(i,j,N) + (z_r(i,j,N)-z_r(i,j,N-1))
     &                                                     *cff1/cff2
                else
                  z_iso(i,j,m)=+99999.
                endif
                if (z_iso(i,j,m) > 0.) then
                  z_iso(i,j,m)=+99999.    ! <-- surfacing
                endif
              elseif (km(i) > 0) then
                k=km(i)
                z_iso(i,j,m)=( z_r(i,j,k)*(var(i,j,k+1)-value)
     &                        +z_r(i,j,k+1)*(value-var(i,j,k))
     &                             )/(var(i,j,k+1)-var(i,j,k))

c**                if (var(i,j,k+1)<value .or.
c**     &              var(i,j,k  )>value) then
c**                  write(*,*) '### ERROR: Algorithm failure.'
c**                  stop
c**                endif
              else
                cff1=var(i,j,1)-value
                cff2=var(i,j,2)-var(i,j,1)
                if (cff1*cff2 > 0.) then
                  z_iso(i,j,m)=z_r(i,j,1) - (z_r(i,j,2)-z_r(i,j,1))
     &                                                   *cff1/cff2
                else
                  z_iso(i,j,m)=-99999.
                endif
                if (z_iso(i,j,m)<z_w(i,j,0)) then
                  z_iso(i,j,m)=-99999.    ! <--- bottoming
                endif
              endif
            else
              z_iso(i,j,m)=0.
            endif
          enddo
        enddo
      enddo
      end

