! GSHH data structure:
!----- ---- ----------
! Upon reading GSHHS dataset by either "read_gshhs" (written in C),
! or equivalently by "read_gshhs_data" (in Fortran) three arrays above
! are filled in with data, "nsegm" becomes the total number of polygons
! read and, correspondingly the actual used size of arrays "segmpt" and
! "isegm".
!
! isegm(nsegm) -- arrays of starting indices of polygon vertices
!                                              stored within "segmpt";
! segmhdr(header_size,nsegm) -- array of headers (one set of numbers
!                      per polygon) containing description, see below;
! segmpt(2,actual_total_number_of points) -- array of lon-lat
!                                  coordinates of vertices of polygon;

! The header structure is described in "gshhg.h" and contains 11
! elements stored as integer numbers:
!
!     id -- unique identifier of the polygon
!     np -- number of points in the polygon
!   flag -- various characteristics of the polygon (see below)
!
!    west  \                                                  !
!    east   \ west, east, north, south extents of the polygon !
!    south  /                                                 !
!    north /                                                  !
!
!      area -- polygon area
! area_full -- area of the parent polygon (the finest resolution)
! container -- ID of polygon that encloses this polygon (-1=none)
!  ancestor -- ID of polygon the full resolution set that was
!                         the source of this polygon (-1 if none)
!
! flag contains 6 characteristics of the polygon packed into a single
! integer number.  It is decoded by "decode_flag" in "read_gshhs.c"
! (C-version) or by "dec_segm_flag" in "read_gshhs_data.F"

      module mod_gshhs
       integer, parameter :: header_size=11
       integer(kind=4) nsegm, npts_total
       integer(kind=4), dimension(:), allocatable :: isegm
       integer(kind=4), dimension(:,:), allocatable :: segmhdr,segmpt
      end module mod_gshhs


