      program nc4chunking

! Inspects netCDF file and prints dimensions and chunking of all
! variables (overall comparable in functionality with
!
!                ncdump -s -h  file.nc
!
! but produces less cluttered output (omits attributes completely)
! and in addition (unlike ncdump), it computes and prints the number
! of excess (padded) points if the dimension is not evenly divisible
! by the corresponding chunk size.

      implicit none
      character(len=256) src_fname
      character(len=32) varname, type
      character(len=32), allocatable, dimension(:) :: dimname
      integer, allocatable, dimension(:) :: dimid, vardimids,
     &                      dsize, vardsizes, nblocks, blksize
      integer nargs, ncsrc, ncfmt, ndims, nvars, ngatts,
     &        unlimdimid, vartype, vardims, varatts, vid, id,
     &        ierr, j, lsrc, ldim, lvar, ltyp, lenstr
      integer, parameter :: undefined=-9999
      logical :: nc4format=.false., nc4chunked=.false.

#include "netcdf.inc"

      nargs=iargc()
      if (nargs /= 1) then
        write(*,'(/1x,A,2/20x,A/)')   'nc4chunking usage:',
     &                                'nc4chunking file.nc'
        stop
      endif
      call getarg(1,src_fname) ; lsrc=lenstr(src_fname)

      ierr=nf_open(src_fname(1:lsrc), nf_nowrite, ncsrc)
      if (ierr == nf_noerr) then
        ierr=nf_inq_format(ncsrc, ncfmt)
        if (ierr == nf_noerr) then
          if (ncfmt == nf_format_classic) then
            write(*,'(/1x,3A)') 'classic netCDF format ''',
     &                            src_fname(1:lsrc), ''''
          elseif (ncfmt == nf_format_64bit) then
            write(*,'(/1x,3A)') '64bit netCDF-3 format ''',
     &                            src_fname(1:lsrc), ''''
          elseif (ncfmt == nf_format_netcdf4) then
            write(*,'(/1x,3A)')       'netCDF-4 format ''',
     &                            src_fname(1:lsrc), ''''
            nc4format=.true.
          elseif (ncfmt == nf_format_netcdf4_classic) then
            write(*,'(/1x,3A)') 'netCDF-4 classic format ''',
     &                              src_fname(1:lsrc), ''''
            nc4format=.true.
          else
            write(*,'(/1x,3A/)') '### ERROR: Unknown format ''',
     &                                  src_fname(1:lsrc), ''''
          endif
          unlimdimid=undefined
          ierr=nf_inq(ncsrc, ndims, nvars, ngatts, unlimdimid)
          if (ierr == nf_noerr) then
            allocate(dimid(ndims)) ; allocate(vardimids(ndims))
            allocate(dsize(ndims)) ; allocate(vardsizes(ndims))
            allocate(nblocks(ndims)) ; allocate(blksize(ndims))
            allocate(dimname(ndims))

            write(*,'(1x,A,3(I4,1x,A))') 'found ',ndims,'dimensions,',
     &                  nvars,'variables,', ngatts,'global attibutes'

            write(*,'(/1x,A/)') 'dimensions:'
            do id=1,ndims
              ierr=nf_inq_dim(ncsrc, id, dimname(id), dsize(id))
              if (ierr == nf_noerr) then
                ldim=lenstr(dimname(id))
                if (id == unlimdimid) then
                  type='unlimited'
                else
                  do j=1,len(type)
                    type(j:j)=' '
                  enddo
                endif
                ltyp=max(1, lenstr(type))

                write(*,'(7x,A,I3,2x,A,T24,A,I5,1x,A)')  'dim', id,
     &               dimname(id)(1:ldim), '=', dsize(id), type(1:ltyp)
              else
                write(*,'(/1x,2A,I3/12x,3A/12x,A/)')     '### ERROR: ',
     &                 'Cannot determine name and size of dimension #',
     &                      id, 'in netCDF file ''', src_fname(1:lsrc),
     &                                        '''.', nf_strerror(ierr)
              endif
            enddo

            write(*,'(/1x,A/)') 'variables:'
            do vid=1,nvars
              ierr=nf_inq_var(ncsrc, vid,  varname,  vartype,
     &                           vardims, vardimids, varatts)
              if (ierr == nf_noerr) then
                lvar=lenstr(varname)

                if (vartype == nf_char) then
                  type='char'
                elseif (vartype == nf_byte) then
                  type='byte'
                elseif (vartype == nf_short) then
                  type='short'
                elseif (vartype == nf_int) then
                  type='int'
                elseif (vartype == nf_float) then
                  type='float'
                elseif (vartype==nf_double) then
                  type='double'
                endif
                ltyp=lenstr(type)

                if (vardims > 0) then
                write(*,'(3x,A,1x,16A)') type(1:ltyp), varname(1:lvar),
     &                '(', ( dimname(vardimids(j))( 1:lenstr(
     &                               dimname(vardimids(j)) )), ',',
     &                                                 j=1,vardims-1 ),
     &                       dimname(vardimids(vardims))(1:lenstr(
     &                              dimname(vardimids(vardims)) )), ')'
                else
                  write(*,'(3x,2A)') type(1:ltyp), varname(1:lvar)
                endif
                if (nc4format) then
                  nc4chunked=.false.
                  do j=1,vardims
                    blksize(j)=0
                  enddo
                  ierr=nf_inq_var_chunking(ncsrc, vid, j, blksize)
                  if (ierr == nf_noerr) then
c**                 write(*,'(2(1x,A),I8,1x,A,8I8)') varname(1:lvar),
c**  &                                'j =',j, 'blksize =', blksize
                    if (j == nf_chunked) nc4chunked=.true.
                    if (nc4chunked) then
                      do j=1,vardims
                        vardsizes(j)=dsize(vardimids(j))
                        nblocks(j)=(vardsizes(j)+blksize(j)-1)
     &                                          /blksize(j)
                        write(*,'(10x,A,I3,4(1x,A,I7))')    'dim', j,
     &                          'size =', vardsizes(j), 'chunk size =',
     &                               blksize(j),'chunks =', nblocks(j),
     &                     'padd =',blksize(j)*nblocks(j)-vardsizes(j)
                      enddo
                      write(*,*)
                    else
                      write(*,'(9x,A)')  'not chunked'
                    endif
                  else
                    write(*,'(/1x,4A/12x,A/)')    '### ERROR: Cannot ',
     &                            'determine chunking for variable ''',
     &                      varname(1:lvar), '''.',  nf_strerror(ierr)
                  endif
                endif !<-- nc4format
              else
                write(*,'(/1x,2A,I3/12x,4A/12x,A/)')     '### ERROR: ',
     &               'Cannot make general inquiry for variable #', vid,
     &                      'in ', 'netCDF file ''', src_fname(1:lsrc),
     &                                        '''.', nf_strerror(ierr)
              endif
            enddo
          else
            write(*,'(/1x,4A/12x,A/)')       '### ERROR: Cannot make ',
     &        'general inquiry into netCDF file ''', src_fname(1:lsrc),
     &                                        '''.', nf_strerror(ierr)
          endif
        else
          write(*,'(/1x,4A/12x,A/)')    '### ERROR: Cannot determine ',
     &          'netCDF-3/-4/whatever format of ''', src_fname(1:lsrc),
     &                                       '''.',  nf_strerror(ierr)
        endif
      else
        write(*,'(/1x,4A/12x,A/)')    '### ERROR: Cannot open netCDF ',
     &                  'file ''', src_fname(1:lsrc),''' for reading.',
     &                                               nf_strerror(ierr)
      endif
      ierr=nf_close(ncsrc)
      end
