      program nc_add_att

! A command-line operator to add a user-specified attribute
! to an existing variable within netCDF file.

      implicit none
      character(len=128) fname, vname, attnm
      character(len=2048) str
      integer nargs, arg, ncid, varid, vtype, ndims, natts, dimid(8),
     &            ierr, ip,ic,is,ie,id, nnum, i, lfnm,lvar,latt,lstr,
     &            lenstr, read_str_int, read_str_real, read_str_dble
      integer(kind=2), allocatable, dimension(:) :: i2
      integer(kind=4), allocatable, dimension(:) :: i4
      real(kind=4),    allocatable, dimension(:) :: r4
      real(kind=8),    allocatable, dimension(:) :: r8
#ifdef INT_IARGC
      integer iargc
#endif
#include "netcdf.inc"

c--#define VERBOSE

      nargs=iargc()
      if (nargs < 3) then 
        write(*,'(/1x,3A/1x,A/,/12x,A/1x,A/12x,A)')    'This command ',
     &      'adds or overwrites an attribute to an existing variable ',
     & 'within', 'netCDF file or sequence of files. Should be used as',
     &   'nc_add_att varname attnm=VALUE file.nc [file2.nc file3.nc]',
     &     'or', 'nc_add_att varname attname=v1,v2,...,v1 file*.nc'
        write(*,'(2(/1x,2A))') 'where v1,v2,...,v1 are comma-',
     & 'separated numbers of integer, real, or double', 'type in the ',
     & 'case where the desired attribute is an array of such numbers.'
        write(*,'(8(1x,2A/))')     'In doing so it attempts to match ',
     &                    'the type of added attribute to the type of',
     &       'variable, provided that user supplied expression VALUE ',
     &            'can be interpreted as', 'an object of the desired ',
     &            'type. If not, the attribute added is of text type.',
     &          'If attribute with specified name already exists its ',
     &            'value (content if text)', 'will be overwritten by ',
     &          'VALUE.   To add or modify a global attribute replace',
     &       'variable name with keyword ''global'' or ''nc_global'' ',
     &     'or ''nf_global'' in all', 'lowercase letters.  Wildcards ',
     &      'are accepted to specify multiple files.',
     &      'To delete an attribute completely and without any ',
     &     'replacement use nc_del_att.'
        stop
      endif

      call getarg(1,vname) ; lvar=lenstr(vname)
      call getarg(2,str)   ; lstr=lenstr(str)
      i=1
      do while(i < lstr .and. str(i:i) /= '=')
        i=i+1
      enddo
      if (str(i:i) == '=') then
        attnm=str(1:i-1)  ; latt=lenstr(attnm)
        str=str(i+1:lstr) ; lstr=lstr-i
      else
        write(*,'(/1x,2A/)')   '### ERROR: The second argument ',
     &          'should be  attnm=VALUE  or  attnm=v1,v2,...,vN'
        stop
      endif

! Interpretation of input string: if it can be read in as a number or
! comma-separated sequence of numbers it should be interpteted as such;
! otherwise treat it as a string of characters;  The type of numbers,
! integer, real, or double precision is distinguished by XX.DYY

      ierr=0; ip=0; ic=0; ie=0; id=0
      do i=1,lstr
        if (str(i:i) == '.') then
          ip=ip+1
        elseif (str(i:i) == ',') then
          ic=ic+1
        elseif (str(i:i) == 'e' .or. str(i:i) == 'E') then
          ie=ie+1
        elseif (str(i:i) == 'd' .or. str(i:i) == 'D') then
          id=id+1
        elseif ( (str(i:i) < '0'   .or. str(i:i) > '9') .and.
     &            str(i:i) /= '+' .and. str(i:i) /= '-' ) then
          ierr=ierr+1
        endif
      enddo
#ifdef VERBOSE
      write(*,'(5(2x,A,I3))') 'ierr =',ierr, 'ip =',ip, 'ic =',ic,
     &                                       'ie =',ie, 'id =',id
#endif

      nnum=ic+1
      if (ierr > 0) then
        vtype=nf_char ; ierr=0
      elseif (ip == 0 .and. ie == 0 .and. id == 0) then
        vtype=nf_int
      elseif ( ip > 0 .and. ie == 0 .and. id == 0) then
        vtype=nf_real
      elseif (                             id > 0) then
        vtype=nf_double
      elseif (               ie > 0              ) then
        vtype=nf_real
      else
        vtype=nf_char
      endif

      if (vtype /= nf_char) then
        if (vtype == nf_int) then
          allocate(i4(nnum))
        elseif (vtype == nf_real) then
          allocate(r4(nnum))
        elseif (vtype == nf_double) then
          allocate(r8(nnum))
        endif

        i=0 ; ic=0 ; ierr=0
        do while(i < nnum .and. ierr == 0)
          i=i+1 ; is=ic+1 ; ic=is     
          do while(ic < lstr .and. str(ic:ic) /= ',')
            ic=ic+1
          enddo
          if (str(ic:ic) == ',') then
            ie=ic-1
          else
            ie=ic
          endif
          if (vtype == nf_int) then
            ierr=read_str_int(str(is:ie), i4(i))
          elseif (vtype == nf_real) then
            ierr=read_str_real(str(is:ie), r4(i))
          elseif (vtype == nf_double) then
            ierr=read_str_dble(str(is:ie), r8(i))
          endif
        enddo
      endif
#ifdef VERBOSE
      if (vtype == nf_char) then
        write(*,*) 'nf_char ',  attnm(1:latt), '=', str(1:lstr) 
      elseif (vtype == nf_int) then 
        write(*,*) 'nf_int ',  attnm(1:latt), '=', i4
      elseif (vtype == nf_real) then
        write(*,*) 'nf_real ',  attnm(1:latt), '=', r4
      elseif (vtype == nf_double) then
        write(*,*) 'nf_double ',  attnm(1:latt), '=', r8
      endif
#endif
      if (ierr /= 0) stop

      do arg=3,nargs
        call getarg(arg,fname) ; lfnm=lenstr(fname)
        ierr=nf_open(fname(1:lfnm), nf_write, ncid)
        if (ierr == nf_noerr) then
          if (vname == 'global' .or. vname == 'nc_global' .or.
     &                               vname == 'nf_global') then
            varid=nf_global
          else
            ierr=nf_inq_varid(ncid, vname, varid)
            if (ierr == nf_noerr) then
              ierr=nf_inq_var(ncid, varid, vname, i, ndims,
     &                                        dimid, natts)
              if (ierr /= nf_noerr) then
                write(*,'(/1x,6A/12x,A/)')  '### ERROR: Cannot make ',
     &               'general inquiry for variable ''', vname(1:lvar),
     &            ''' in ''', fname(1:lfnm), '''.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,6A/12x,A/)')    '### ERROR: Cannot find ',
     &                   'variable ''', vname(1:lvar), ''' within ''',
     &                        fname(1:lfnm), '''.', nf_strerror(ierr)
            endif
          endif

          if (ierr == nf_noerr) then
            ierr=nf_redef(ncid)
            if (ierr == nf_noerr) then
              if (vtype == nf_char) then
                ierr=nf_put_att_text(ncid, varid, attnm(1:latt),
     &                                        lstr, str(1:lstr))
              elseif (vtype == nf_int2) then
                ierr=nf_put_att_int2(ncid, varid, attnm(1:latt),
     &                                        nf_int2, nnum, i2)
              elseif (vtype == nf_int) then
                ierr=nf_put_att_int(ncid, varid, attnm(1:latt),
     &                                        nf_int, nnum, i4)
              elseif (vtype == nf_real) then
                ierr=nf_put_att_real( ncid, varid, attnm(1:latt),
     &                                        nf_real, nnum, r4)
              elseif (vtype == nf_double) then
                ierr=nf_put_att_double(ncid,varid, attnm(1:latt),
     &                                      nf_double, nnum, r8)
              endif
              if (ierr == nf_noerr) then
                ierr=nf_enddef(ncid)
                if (ierr == nf_noerr) then
                  ierr=nf_close(ncid)
                  if (ierr == nf_noerr) then
                    if (vname=='global' .or. vname=='nc_global' .or.
     &                                       vname=='nf_global') then
                      if (vtype == nf_char) then 
                        write(*,'(8(1x,A))') 'added global attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                             'of type char into', fname(1:lfnm)
                      elseif (vtype == nf_int2) then
                        write(*,'(8(1x,A))') 'added global attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                             'of type int2 into', fname(1:lfnm)
                      elseif (vtype == nf_int) then
                        write(*,'(8(1x,A))') 'added global attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                              'of type int into', fname(1:lfnm)
                      elseif (vtype == nf_real) then
                        write(*,'(8(1x,A))') 'added global attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                             'of type real into', fname(1:lfnm)
                      elseif (vtype == nf_double) then
                        write(*,'(8(1x,A))') 'added global attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                           'of type double into', fname(1:lfnm)
                      endif
                    else
                      if (vtype == nf_char) then
                        write(*,'(8(1x,A))')        'added attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                     'of type char for variable', vname(1:lvar),
     &                                            'in', fname(1:lfnm)
                      elseif (vtype == nf_int2) then
                        write(*,'(8(1x,A))')        'added attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                     'of type int2 for variable', vname(1:lvar), 
     &                                            'in', fname(1:lfnm)
                      elseif (vtype == nf_int) then
                        write(*,'(8(1x,A))')        'added attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                      'of type int for variable', vname(1:lvar),
     &                                            'in', fname(1:lfnm)
                      elseif (vtype == nf_real) then
                        write(*,'(8(1x,A))')        'added attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                     'of type real for variable', vname(1:lvar),
     &                                            'in', fname(1:lfnm)
                      elseif (vtype == nf_double) then
                        write(*,'(8(1x,A))')        'added attribute',
     &                                attnm(1:latt), '=', str(1:lstr),
     &                   'of type double for variable', vname(1:lvar),
     &                                            'in', fname(1:lfnm)
                      endif
                    endif
                  else
                    write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot ',
     &                         'close netCDF file ''', fname(1:lfnm),
     &                                      '''.', nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot end ',
     &                           'definition mode for netCDF file ''',
     &                        fname(1:lfnm), '''.', nf_strerror(ierr)
                endif
              else
                write(*,'(/1x,8A/12x,A/)')   '### ERROR: Cannot add ',
     &            'attribute ''', attnm(1:latt), ''' for variable ''',
     &                                    vname(1:lvar), ''' into ''',
     &                        fname(1:lfnm), '''.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,4A/12x,A/)')     '### ERROR: Cannot put ',
     &                                'netCDF file ''', fname(1:lfnm),
     &                ''' into redefinition mode.', nf_strerror(ierr)
            endif
          else
            ierr=nf_close(ncid)!<-- just close file without any action

          endif
        else
          write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot open netCDF ',
     &                    'file ''', fname(1:lfnm), ''' for writting.',
     &                                              nf_strerror(ierr)
        endif
      enddo  !<-- loop over files
      end
