      program nccp

! Generic tool to copy content of a netCDF variable from one file to
! another, where the variable is assumed to be apriori defined in both
! source and target files (hence this tool makes no attempt to redefine
! the structure of target netCDF file).
!
! Usage: nccp needs 3 arguments:
!-------
!               nccp source.nc target.nc varname
! where
!          soure.nc   is name of source netCDF file
!          target.nc             target netCDF file
!          varname               variable to be copied


      implicit none
      character(len=80) srcfile,trgfile, varname,varname1
      character(len=16) dimname(8), dimname1(8)
      integer ncsrc,nctrg, varid,vtarg, unlimdimid,unlimdimid1,
     &        vartype,vartype1, vardims,vardims1, varatts,varatts1,
     &        dimids(8),dimids1(8),  start(8),count(8), count1(8),
     &        size, nrecs, irec, i, ierr, lsrc,ltrg, lvar
      integer iargc, lenstr !<-- functions

      character(len=1), allocatable, dimension(:) :: cbfr
      integer(kind=1),  allocatable, dimension(:) :: ibfr1
      integer(kind=2),  allocatable, dimension(:) :: ibfr2
      integer(kind=4),  allocatable, dimension(:) :: ibffr
      real(kind=4), allocatable, dimension(:) :: bffr4
      real(kind=8), allocatable, dimension(:) :: bffr8

c--#define VERBOSE
#include "netcdf.inc"

      i=iargc()
      if (i == 3) then
        call getarg(1,srcfile) ; lsrc=lenstr(srcfile)
        call getarg(2,trgfile) ; ltrg=lenstr(trgfile)
        call getarg(3,varname) ; lvar=lenstr(varname)
      else
        write(*,'(/1x,3A/1x,A/ /8x,A/ /1x,3A/1x,2A/)') 'Operation ''',
     &          'nccp'' copies content of a user-specified variable ',
     &          'between two',  'netCDF files. It should be used as:',
     &          'nccp source.nc target.nc varname','where the ',
     &          'second file should have variable ''varname'' pre-',
     &          'defined to have',    'the same type and sizes of ',
     &          'all dimensions (except record dimension, if any).'
        stop
      endif

      unlimdimid=-1024              ! <-- nonexisting dimension id
      size=1  ; nrecs=1             ! <-- all are initialized to
      start=1 ; count=1 ; count1=1  ! <-- meaningful default values


! Open source file, inquire about variable and determine the size of
!----- ------ ----- buffer needed to read it: either the whole netCDF
! variable or just one time record if it consists of records.
! Note that variable "size" is initialized to 1 above, and thereafter
! is multiplied by the length of each dimension of the variable (except
! record dimension), so positive value of "size" is used as signal of
! success. Conversely, if error occurs at any stage, "size" is set to
! zero and Remains so thereafter to signal about error.

      ierr=nf_open(srcfile, nf_nowrite, ncsrc)
      if (ierr == nf_noerr) then
        ierr=nf_inq_unlimdim(ncsrc, unlimdimid)
        if (ierr == nf_noerr) then
          lvar=lenstr(varname)
          ierr=nf_inq_varid (ncsrc, varname, varid)
          if (ierr == nf_noerr) then
            ierr=nf_inq_var(ncsrc, varid, varname, vartype,
     &                          vardims,  dimids, varatts)
            if (ierr == nf_noerr) then
#ifdef VERBOSE
              write(*,'(3A,3(1x,A,I4)/A,8I4)')    'varname = ''',
     &               varname(1:lvar), '''', 'vartype =', vartype,
     &              'vardims =', vardims,   'varatts =', varatts,
     &                      'dimids =', (dimids(i), i=1,vardims)
#endif
              do i=1,vardims
                ierr=nf_inq_dim(ncsrc,dimids(i), dimname(i),count(i))
                if (ierr == nf_noerr) then
#ifdef VERBOSE
                  write(*,*) dimname(i)(1:lenstr(dimname(i))),
     &                                       ' =', count(i)
#endif
                  if (dimids(i) == unlimdimid) then
                    nrecs=count(i)
                    count(i)=1
                  else
                    size=size*count(i)
                  endif
                else
                  write(*,'(/1x,2A,I3,1x,3A/12x,A/)')   '### ERROR: ',
     &               'Cannot make inquiry for dimension #', dimids(i),
     &               'in input netCDF file ''',       srcfile(1:lsrc),
     &                                      '''. ', nf_strerror(ierr)
                  size=0
                endif
              enddo
#ifdef VERBOSE
              write(*,*) 'size =',size, ' nrecs =',nrecs
#endif
            else
              write(*,'(/1x,2A,I3,A/12x,4A/)') '### ERROR: Cannot ',
     &                'make general inquiry for variable with ID =',
     &                 varid,     ' within',       'netCDF file ''',
     &                 srcfile(1:lsrc), ''': ', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,4A/)')    '### ERROR: Cannot get ',
     &              'netCDF ID for variable ''',    varname(1:lvar),
     &              ''' within netCDF', 'file ''', srcfile(1:lsrc),
     &              ''': ', nf_strerror(ierr)
          endif
        else
          write(*,'(1x,4A/12x,A)')  '### ERROR: Cannot make inquiry ',
     &               'for unlimited dimension in ''', srcfile(1:lsrc),
     &                                       '''.', nf_strerror(ierr)
        endif
      else
        write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot open netCDF ',
     &                 'file ''', srcfile(1:lsrc), ''' for reading,',
     &                                             nf_strerror(ierr)
      endif
      if (ierr /= nf_noerr .or. size == 0) stop

! Open target file:  Compare variable with the same name against the
!----- ------ -----  one in source file: its type, rank (number of
! dimensions), and sizes of each dimension should be the same, with
! the, exception of record dimension (if any), which, in principle
! may be anything (incl. 0 records)  as the target file may be just
! created as a placeholder without any actual data written yet. 

      ierr=nf_open(trgfile, nf_write, nctrg)
      if (ierr == nf_noerr) then
        ierr=nf_inq_unlimdim(nctrg, unlimdimid1)
        if (ierr == nf_noerr) then
          ierr=nf_inq_varid(nctrg, varname, vtarg)
          if (ierr == nf_noerr) then
            ierr=nf_inq_var(nctrg, vtarg, varname1, vartype1,
     &                          vardims1,  dimids1, varatts1)
            if (ierr == nf_noerr) then
              if (vartype1 == vartype) then
                do i=1,vardims1
                  ierr=nf_inq_dim(nctrg, dimids1(i), dimname1(i),
     &                                                count1(i))
                  if (ierr == nf_noerr) then
                    if ( dimids(i) == unlimdimid  .and.
     &                  dimids1(i) == unlimdimid1 ) then
                      count1(i)=1
                    elseif (count1(i) /= count(i)) then
                      write(*,'(/1x,A,I4,1x,6A,I5,A/28x,3A,I5/)')
     &                  '### ERROR: Dimension #', i, 'of input-file ',
     &                      'variable ''', varname(1:lenstr(varname)),
     &                       ''', ', dimname(i)(1:lenstr(dimname(i))),
     &                                          ' =', count(i), ',',
     &                  'differs from that of target-file variable, ',
     &                            dimname1(i)(1:lenstr(dimname1(i))),
     &                                             ' =', count1(i)
                      size=0
                    endif
                  else
                    write(*,'(/1x,2A,I3,1x,3A/12x,A/)') '### ERROR: ',
     &              'Cannot make inquiry for dimension #', dimids1(i),
     &               'in target netCDF file ''',     trgfile(1:ltrg),
     &                                      '''. ', nf_strerror(ierr)
                    size=0
                  endif
                enddo
              else
                write(*,'(/1x,4A,I4,A/25x,2A,I4,A/)')  '### ERROR: ',
     &          'Type of of variable ''', varname(1:lenstr(varname)),
     &          ''' in source file,', vartype, ',',  'differs from ',
     &                         'that in target file,', vartype1, '.'
              endif
            else
              write(*,'(/1x,2A,I3/12x,3A,1x,A/)') '### ERROR: Cannot ',
     &               'make general inquiry for variable with #', vtarg,
     &                    'in target file ''',  trgfile(1:ltrg), '''.',
     &                                               nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,3A,1x,A/)')  '### ERROR: Cannot get ',
     &               'netCDF ID for variable ''',      varname(1:lvar),
     &                ''' within', 'target file ''',  trgfile(1:ltrg),
     &                                       '''.',  nf_strerror(ierr)
          endif
        else
          write(*,'(1x,4A/12x,A)')   '### ERROR: Cannot make inquiry ',
     &              'for unlimited dimension in ''', trgfile(1:ltrg),
     &                                        '''.', nf_strerror(ierr)
        endif
      else
        write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot open netCDF ',
     &             'file ''',  trgfile(1:ltrg), ''' for writing,',
     &                                            nf_strerror(ierr)
      endif

! Once all the sized are determined and all checks complete allocate
! workspace of the proper type and do the actual copying.

      if (ierr == nf_noerr .and. size > 0) then
        if (vartype == nf_char) then
          allocate(cbfr(size))
#ifndef VERBOSE
# define write !
#endif
          write(*,*) 'allocated nf_char workspace', size
        elseif (vartype == nf_byte) then
          allocate(ibfr1(size))
          write(*,*) 'allocated nf_byte workspace', size
        elseif (vartype == nf_short) then
          allocate(ibfr2(size))
          write(*,*) 'allocated nf_int2 workspace', size
        elseif (vartype == nf_int) then
          allocate(ibffr(size))
          write(*,*) 'allocated nf_int workspace',  size
        elseif (vartype == nf_float) then
          allocate(bffr4(size))
          write(*,*) 'allocated nf_real workspace', size
        elseif (vartype == nf_double) then
          allocate(bffr8(size))
          write(*,*) 'allocated nf_double workspace',size
#ifdef write
# undef write
#endif
        else
          write(*,'(/1x,2A)') '### ERROR: Unknown variable type.'
          stop
        endif

        ierr=nf_set_fill(nctrg, nf_nofill, i)
        if (ierr /= nf_noerr) then
           write(*,'(/1x,A/)') '### ERROR: Cannot switch ''',
     &              trgfile(1:ltrg), ''' into no fill mode.'
           stop
#ifdef VERBOSE
        else
           write(*,'(1x,3A)')      'set ''', trgfile(1:ltrg),
     &                                ''' into no fill mode.'
#endif
        endif

        do irec=1,nrecs
          do i=1,vardims
            if (dimids(i) == unlimdimid) then
              start(i)=irec ; count(i)=1
            endif
          enddo

          if (vartype == nf_char) then
            ierr=nf_get_vara_text (ncsrc,varid, start,count, cbfr)
          elseif (vartype == nf_byte) then
            ierr=nf_get_vara_int1 (ncsrc,varid, start,count, ibfr1)
          elseif (vartype == nf_short) then
            ierr=nf_get_vara_int2  (ncsrc,varid, start,count, ibfr2)
          elseif (vartype == nf_int) then
            ierr=nf_get_vara_int   (ncsrc,varid, start,count, ibffr)
          elseif (vartype == nf_float) then
            ierr=nf_get_vara_real  (ncsrc,varid, start,count, bffr4)
          elseif (vartype == nf_double) then
            ierr=nf_get_vara_double(ncsrc,varid, start,count, bffr8)
          endif
          if (ierr == nf_noerr) then
            if (vartype == nf_char) then
              ierr=nf_put_vara_text  (nctrg,vtarg, start,count, cbfr)
            elseif (vartype == nf_byte) then
              ierr=nf_put_vara_int1  (nctrg,vtarg, start,count, ibfr1)
            elseif (vartype == nf_short) then
              ierr=nf_put_vara_int2  (nctrg,vtarg, start,count, ibfr2)
            elseif (vartype == nf_int) then
              ierr=nf_put_vara_int   (nctrg,vtarg, start,count, ibffr)
            elseif (vartype == nf_float) then
              ierr=nf_put_vara_real  (nctrg,vtarg, start,count, bffr4)
            elseif (vartype == nf_double) then
              ierr=nf_put_vara_double(nctrg,vtarg, start,count, bffr8)
            endif
            if (ierr == nf_noerr) then
              if (nrecs > 1) then
                write(*,'(1x,2(A,I5,1x),7A)') 'Copied record',irec,'/',
     &            nrecs, 'of variable ''',varname(1:lvar),''' from ''',
     &            srcfile(1:lsrc), ''' into ''',trgfile(1:ltrg), '''.'
              else
                write(*,'(1x,7A)')                'Copied variable ''',
     &                  varname(1:lvar), ''' from ''', srcfile(1:lsrc),
     &                            ''' into ''', trgfile(1:ltrg), '''.'
              endif
            else
              write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot write ',
     &                   'variable ''', varname(1:lvar), ''' into ''',
     &                     trgfile(1:ltrg), '''.', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,6A/12x,A/)')      '### ERROR: Cannot read ',
     &                   'variable ''', varname(1:lvar), ''' from ''',
     &                     srcfile(1:lsrc), '''.', nf_strerror(ierr)
          endif
        enddo

        ierr=nf_close(nctrg)
        ierr=nf_close(ncsrc)
      endif
      end

