      program nccpstruct

#define COPY_TIMELESS_VARS

! A generic tool to copy skeleton structure of netCDF file without
! filling its content.  This is a stand-alone, command-line operator
! designed to be used as
!
!          nccpstruct source.nc target.nc
!
! Basically it makes a general inquiry into "source" netCDF file and
! creates "target" netCDF file with exactly the same dimensions,
! variables and attributes.

      implicit none
      character(len=80) src_fname, trg_fname
      character(len=32) dimname, attname, varname
      integer, allocatable, dimension(:) :: dimid, vardimids
      integer nargs, ncsrc, nctarg, ndims, nvars, ngatts,
     &        unlimdimid, varid, vartype, vardims, varatts,
     &        size, i,j, ierr, lsrc, ltrg, lvar, latt, lenstr

#ifdef COPY_TIMELESS_VARS

! Buffer arrays for reading/writing, and their allocated sizes

      integer :: alloc_cbfr=0,  alloc_ibfr1=0, alloc_ibfr2=0,
     &           alloc_ibffr=0, alloc_bffr4=0, alloc_bffr8=0
      character(len=1), allocatable, dimension(:) :: cbfr
      integer(kind=1),  allocatable, dimension(:) :: ibfr1
      integer(kind=2),  allocatable, dimension(:) :: ibfr2
      integer(kind=4),  allocatable, dimension(:) :: ibffr
      real(kind=4), allocatable, dimension(:) :: bffr4
      real(kind=8), allocatable, dimension(:) :: bffr8
#endif

#include "netcdf.inc"

      nargs=iargc()
      if (nargs == 2) then
        call getarg(1,src_fname) ; lsrc=lenstr(src_fname)
        call getarg(2,trg_fname) ; ltrg=lenstr(trg_fname)

        ierr=nf_open(src_fname(1:lsrc), nf_nowrite, ncsrc)
        if (ierr == nf_noerr) then
          ierr=nf_inq(ncsrc, ndims, nvars, ngatts, unlimdimid)
          if (ierr == nf_noerr) then
            allocate(dimid(ndims), vardimids(ndims))
            ierr=nf_create(trg_fname(1:ltrg), nf_64bit_offset, nctarg)
            if (ierr == nf_noerr) then
              write(*,*)     'created netCDF file ''',
     &                        trg_fname(1:ltrg), '''.'

! Sort out dimensions:

              do i=1,ndims
                ierr=nf_inq_dim(ncsrc, i, dimname, size)
                if (ierr == nf_noerr) then
                  lvar=lenstr(dimname)
                  if (i == unlimdimid) size=nf_unlimited
                  ierr=nf_def_dim(nctarg, dimname(1:lvar), size,
     &                                                 dimid(i))
                  if (ierr == nf_noerr) then
                    write(*,'(3x,3A,I4)') 'created dimension ''',
     &                              dimname(1:lvar), '''=', size
                  else
                    write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot ',
     &                         'define dimension ''', dimname(1:lvar),
     &                      ''' in netCDF file ''', trg_fname(1:ltrg),
     &                                       '''.', nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,2A,I3/12x,3A/12x,A/)')  '### ERROR: ',
     &                'Cannot determine name and size of dimension #',
     &                  i, 'in netCDF file ''',     src_fname(1:lsrc),
     &                                       '''.', nf_strerror(ierr)
                endif
              enddo

! Copy global attributes

              do i=1,ngatts
                ierr=nf_inq_attname(ncsrc, nf_global, i, attname)
                if (ierr == nf_noerr) then
                  latt=lenstr(attname)
                  if (attname(1:latt) == 'nc_format') then
                    write(*,'(8x,3A)') 'skipping global attribute ''',
     &                                         attname(1:latt), '''.'
                  else
                    ierr=nf_copy_att(ncsrc, nf_global, attname(1:latt),
     &                                              nctarg, nf_global)
                    if (ierr == nf_noerr) then
                      write(*,'(8x,3A)') 'copied global attribute ''',
     &                                         attname(1:latt), '''.'
                    else
                      write(*,'(/1x,4A/12x,3A/12x,A/)')  '### ERROR: ',
     &                               'Cannot copy global attribute ''',
     &                    attname(1:latt), '''', 'into netCDF file ''',
     &                    trg_fname(1:ltrg), '''.',  nf_strerror(ierr)
                    endif
                  endif
                else
                  write(*,'(/1x,2A,I3,1x,3A/12x,A/)')   '### ERROR: ',
     &                  'Cannot determine name of global attribute #',
     &                   i, 'from netCDF file ''',  src_fname(1:lsrc),
     &                                       '''.', nf_strerror(ierr)
                endif
              enddo

! Define variables and their attributes.

              do i=1,nvars
                ierr=nf_inq_var(ncsrc, i, varname, vartype, vardims,
     &                                           vardimids, varatts)
                if (ierr == nf_noerr) then
                  lvar=lenstr(varname)
                  ierr=nf_def_var(nctarg, varname(1:lvar),  vartype,
     &                                    vardims, vardimids, varid)
                  if (ierr == nf_noerr) then
                    write(*,'(3x,3A)') 'created variable ''',
     &                                 varname(1:lvar), '''.'
                    do j=1,varatts
                      ierr=nf_inq_attname(ncsrc, i, j, attname)
                      latt=lenstr(attname)
                      ierr=nf_copy_att(ncsrc, i,  attname(1:latt),
     &                                             nctarg, varid)
                      if (ierr == nf_noerr) then
                        write(*,'(8x,3A)')    'copied attribute ''',
     &                                         attname(1:latt), '''.'
                      else
                        write(*,'(/1x,5A/12x,3A/12x,A/)')
     &                          '### ERROR: Cannot copy attribute ''',
     &                           attname(1:latt),''' for variable ''',
     &                                    varname(1:lvar), ''' into ',
     &                          'netCDF file ''',  trg_fname(1:ltrg),
     &                                     '''.',  nf_strerror(ierr)
                      endif
                    enddo
                  else
                    write(*,'(/1x,4A/12x,3A/12x,A/)')  '### ERROR: ',
     &                  'Cannot create variable ''', varname(1:lvar),
     &                  '''', 'in netCDF file ''', trg_fname(1:ltrg),
     &                                      '''.', nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,2A,I3/12x,4A/12x,A/)') '### ERROR: ',
     &            'Cannot make general inquiry for variable # ', i,
     &            'in ', 'netCDF file ''', src_fname(1:lsrc),  '''.',
     &                                            nf_strerror(ierr)
                endif
              enddo

              ierr=nf_enddef(nctarg)
              write(*,'(2x,3A)')  'leaving definition mode for ''',
     &                                    trg_fname(1:ltrg), '''.'
              goto 1 !--> proceed to copying variables 
            else
              write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot create ',
     &                     'netCDF file ''', trg_fname(1:ltrg), '''.',
     &                                              nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,A/)')      '### ERROR: Cannot make ',
     &                          'general inquiry into netCDF file ''',
     &                    src_fname(1:lsrc), '''.', nf_strerror(ierr)
          endif
        else
          write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot open netCDF ',
     &                 'file ''', src_fname(1:lsrc),''' for reading.',
     &                                              nf_strerror(ierr)
        endif
      else
        write(*,'(/1x,A/ /12x,A/)') 'Usage of nccpstruct should be:',
     &                    'nccpstruct source_file.nc target_file.nc'
      endif
      stop
  1   continue

!--> Copy variables

#ifdef COPY_TIMELESS_VARS

! Copy variables which do not have unlimited dimension: most likely
! they are environmental variables)

      do i=1,nvars
        ierr=nf_inq_var(ncsrc, i, varname, vartype, vardims,
     &                                   vardimids, varatts)
        if (ierr == nf_noerr) then
          size=1
          do j=1,vardims
            if (vardimids(j) == unlimdimid) then
              size=0
            else 
              ierr=nf_inq_dimlen(ncsrc, vardimids(j), lvar)
              if (ierr == nf_noerr) then
                size=size*lvar
              else
                write(*,'(/1x,2A,I4/12x,A/)') '### ERROR: Cannot find',
     &          ' size of dimesion #', vardimids(j), nf_strerror(ierr)
                stop
              endif
            endif
          enddo
          if (size > 0) then
            if (vartype == nf_char .and. size > alloc_cbfr) then
              if (allocated(cbfr)) deallocate(cbfr)
              allocate(cbfr(size)) ;  alloc_cbfr=size
              write(*,*) 'allocated nf_char workspace', size
            elseif (vartype == nf_byte .and. size > alloc_ibfr1) then
              if (allocated(ibfr1)) deallocate(ibfr1)
              allocate(ibfr1(size)) ; alloc_ibfr1=size
              write(*,*) 'allocated nf_byte workspace', size
            elseif (vartype == nf_short  .and. size > alloc_ibfr2) then
              if (allocated(ibfr2)) deallocate(ibfr2)
              allocate(ibfr2(size)) ; alloc_ibfr2=size
              write(*,*) 'allocated nf_int2 workspace', size
            elseif (vartype == nf_int .and. size > alloc_ibffr) then
              if (allocated(ibffr)) deallocate(ibffr)
              allocate(ibffr(size)) ; alloc_ibffr=size
              write(*,*)  'allocated nf_int workspace', size
            elseif (vartype == nf_float .and. size > alloc_bffr4) then
              if (allocated(bffr4)) deallocate(bffr4)
              allocate(bffr4(size)) ; alloc_bffr4=size
              write(*,*) 'allocated nf_real workspace', size
            elseif (vartype == nf_double .and. size > alloc_bffr8) then
              if (allocated(bffr8)) deallocate(bffr8)
              allocate(bffr8(size)) ; alloc_bffr8=size
              write(*,*)'allocated nf_double workspace',size
            endif

            lvar=lenstr(varname)
            if (vartype == nf_char) then
              ierr=nf_get_var_text(ncsrc, i, cbfr)
            elseif (vartype == nf_byte) then
              ierr=nf_get_var_int1  (ncsrc, i, ibfr1)
            elseif (vartype == nf_short) then
              ierr=nf_get_var_int2  (ncsrc, i, ibfr2)
            elseif (vartype == nf_int) then
              ierr=nf_get_var_int   (ncsrc, i, ibffr)
            elseif (vartype == nf_float) then
              ierr=nf_get_var_real  (ncsrc, i, bffr4)
            elseif (vartype == nf_double) then
              ierr=nf_get_var_double(ncsrc, i, bffr8)
            endif
            if (ierr == nf_noerr) then
              write(*,'(9x,3A)') 'read ''', varname(1:lvar), ''''
              if (vartype == nf_char) then
                ierr=nf_put_var_text(nctarg, i,  cbfr)
              elseif (vartype == nf_byte) then
                ierr=nf_put_var_int1 (nctarg, i,  ibfr1)
              elseif (vartype == nf_short) then
                ierr=nf_put_var_int2 (nctarg, i,  ibfr2)
              elseif (vartype == nf_int) then
                ierr=nf_put_var_int  (nctarg, i,  ibffr)
              elseif (vartype == nf_float) then
                ierr=nf_put_var_real (nctarg, i,  bffr4)
              elseif (vartype == nf_double) then
                ierr=nf_put_var_double(nctarg, i, bffr8)
              endif
              if (ierr == nf_noerr) then
                write(*,'(8x,3A)') 'wrote ''', varname(1:lvar), ''''
              else
                write(*,*) '### ERROR: Cannot write variable ''',
     &                  varname(1:lvar), '''', nf_strerror(ierr)
              endif
            else
              write(*,*)   '### ERROR: Cannot read variable ''',
     &                  varname(1:lvar), '''', nf_strerror(ierr)
            endif
          endif  !<-- size > 0
        else
          write(*,'(/1x,2A,I3,1x,3A/12x,A/)')     '### ERROR: Cannot ',
     &   'make general inquiry for variable #', i, 'in netCDF file ''',
     &                     src_fname(1:lsrc), '''.', nf_strerror(ierr)
        endif
      enddo !<-- i=1,nvars
#endif
      ierr=nf_close(nctarg)
      write(*,*)  'closed netCDF file ''', trg_fname(1:ltrg), '''.'
      ierr=nf_close(ncsrc)
      end
