#ifndef TYPE

      program ncmaxval

! Find the minimum and maximum unmasked values of a user-specified
! variable inside a netCDF file or a sequence of files and print their
! spatial indices, record number, and filename in the case of sequence.
!
! Usage:
!          ncmaxval varname file.nc [file2.nc file3.nc ...]

c--#define VERBOSE

      implicit none
      character(len=80) fname
      character(len=32) vname, dimname(8)
      character(len=256) maxstr, minstr 
      integer nargs, iarg, ncid, varid, vtype,  nvdims, vatts,
     &        unlimdimid, rec_dim, nrecs, dimids(8), dimsize(8),
     &        start(8),count(8), size, irec, ierr, lstr, lfnm,
     &        i, spvtype, lspv, lvar, lenstr, iargc
      real(kind=4),  allocatable, dimension(:) :: bffr4
      real(kind=8),   allocatable, dimension(:) :: bffr8
      integer(kind=4), allocatable, dimension(:) :: ibffr
      integer(kind=2),  allocatable, dimension(:) :: ibff2
      integer :: bffr4_size=0, bffr8_size=0,
     &           ibffr_size=0, ibff2_size=0

      integer(kind=4) :: imin=-1, jmin=-1, kmin=-1, rmin=-1, ifmin=-1,
     &                   imax=-1, jmax=-1, kmax=-1, rmax=-1, ifmax=-1
      real(kind=4) :: spv4=1.E+33, vmin4, vmax4
      real(kind=8) :: spv8=1.D+33, vmin8, vmax8

! Note: ispv,ispv2 are initialized to the largerst possible positive
! values for 4- and 2-byte integer numbers respectively.

      integer(kind=2) :: ispv2=32767, ivmin2, ivmax2
      integer(kind=4) :: ispv=2147483647, ivmin, ivmax

#include "netcdf.inc"

      ivmin2=ispv2 ; ivmax2=-ispv2 ; ivmin=ispv ; ivmax=-ispv
      vmin4=spv4   ; vmax4=-spv4   ; vmin8=spv8 ; vmax8=-spv8

      nargs=iargc()
      if (nargs > 1) then
        call getarg(1,vname) ; lvar=lenstr(vname)
      else
        write(*,'(3(/1x,2A),/1x,A/16x,A/1x,A/16x,A/,/1x,A/)')
     &  'Operation ''ncmaxval'' finds minimum and maximum values ',
     &  'among unmasked points',   '(points with missing/special ',
     &           'values are excluded) for the specified variable',
     &  'stored in netCDF file or sequence of files and prints ',
     &                    'their locations.',      'Usage:',
     &                    'ncmaxval varname file.nc',         'or',
     &                    'ncmaxval varname file1.nc file2.nc ...',
     &                    'wildcards like file*.nc are accepted.'
        stop
      endif

      do iarg=2,nargs
        call getarg(iarg,fname) ; lfnm=lenstr(fname)
        dimsize=0 ; dimids=-1 ; rec_dim=0 ; nrecs=1 ; start=1 ; count=1

        ierr=nf_open(fname, nf_nowrite, ncid)
        if (ierr == nf_noerr) then
          ierr=nf_inq_varid(ncid, vname, varid)
          if (ierr == nf_noerr) then
            ierr=nf_inq_var(ncid, varid,  vname,  vtype,
     &                            nvdims, dimids, vatts)
            if (ierr == nf_noerr) then
#ifdef VERBOSE
              write(*,'(1x,3A,I4,3(1x,A,I4),3I4)') 'vname = ''',
     &                      vname(1:lvar),  ''' type =',  vtype,
     &                     'nvdims =', nvdims, 'vatts =', vatts,
     &                     'dimids =', (dimids(i), i=1,nvdims)
#endif
#define SPV_NAME '_FillValue'

              ierr=nf_inq_att(ncid, varid, SPV_NAME, spvtype, lspv)
              if (ierr == nf_noerr) then
                if (vtype == nf_real) then
                  ierr=nf_get_att_real(ncid, varid,  SPV_NAME,  spv4)
                elseif (vtype == nf_double) then
                  ierr=nf_get_att_double(ncid, varid, SPV_NAME, spv8)
                elseif (vtype == nf_int) then
                  ierr=nf_get_att_int(ncid,  varid,   SPV_NAME, ispv)
                elseif (vtype == nf_short) then
                  ierr=nf_get_att_int2(ncid, varid,  SPV_NAME, ispv2)
                endif
                if (ierr == nf_noerr) then
                  if (vtype == nf_real) then
                    write(*,*) 'retrieved 'SPV_NAME' =', spv4,
     &                                        ' type nf_real.'
                  elseif (vtype == nf_double) then
                    write(*,*) 'retrieved 'SPV_NAME' =', spv8,
     &                                      ' type nf_double.'
                  elseif (vtype == nf_int) then
                    write(*,*) 'retrieved 'SPV_NAME' =', ispv,
     &                                         ' type nf_int.'
                  elseif (vtype == nf_short) then
                    write(*,*) 'retrieved 'SPV_NAME' =', ispv2,
     &                                        ' type nf_short.'
                  endif
                endif 
#ifdef VERBOSE
              else
                write(*,*) 'No 'SPV_NAME' for variable ''',
     &                                 vname(1:lvar), '''.' 
#endif
              endif

              unlimdimid=-1
              ierr=nf_inq_unlimdim(ncid, unlimdimid)
              if (ierr /= nf_noerr) then
                write(*,'(1x,2A/12x,A/)')    '### ERROR: Cannot ',
     &                    'determine unlimited dimension number.',
     &                                         nf_strerror(ierr)
                goto 99
#ifdef VERBOSE
              elseif (unlimdimid /= -1) then
                write(*,'(1x,A,I4,1x,A)') 'identified dimension ',
     &                                unlimdimid, 'as unlimited.'
#endif
              endif

              do i=1,nvdims
                ierr=nf_inq_dim(ncid, dimids(i), dimname(i),
     &                                           dimsize(i))
                if (ierr == nf_noerr) then
                  lstr=lenstr(dimname(i))
                  if (i == nvdims) then
                    if (dimids(i) == unlimdimid) then
                      rec_dim=i
                    elseif (lstr > 3) then
                      if (dimname(i)(lstr-3:lstr) == 'time') then
                        rec_dim=i
                      endif
                    endif
                  endif
                  count(i)=dimsize(i)
#ifdef VERBOSE
                  write(*,'(8x,3A,I5)')  'dimension ',
     &                    dimname(i)(1:lstr), ' =', dimsize(i)
#endif
                else
                  write(*,'(/1x,2A,I3,A/12x,4A/)')     '### ERROR: ',
     &               'Cannot determine name and size of dimension #',
     &                dimids(i),     ' within netCDF',    'file ''',
     &                fname(1:lfnm), ''': ', nf_strerror(ierr)
                  count(i)=0
                endif
              enddo

              if (rec_dim > 0) then
                nrecs=count(rec_dim) ; nvdims=rec_dim-1
              endif
              size=1
              do i=1,nvdims
                size=size*count(i)
              enddo
              if (vtype == nf_real) then
                if (size > bffr4_size) then
                  if (allocated(bffr4)) deallocate(bffr4)
                  allocate(bffr4(size)) ; bffr4_size=size
                endif
              elseif (vtype == nf_double) then
                if (size > bffr8_size) then
                  if (allocated(bffr8)) deallocate(bffr8)
                  allocate(bffr8(size)) ; bffr8_size=size
                endif
              elseif (vtype == nf_int) then
                if (size > ibffr_size) then
                  if (allocated(ibffr)) deallocate(ibffr)
                  allocate(ibffr(size)) ; ibffr_size=size
                endif
              elseif (vtype == nf_short) then
                if (size > ibff2_size) then
                  if (allocated(ibff2)) deallocate(ibff2)
                  allocate(ibff2(size)) ; ibff2_size=size
                endif
              else
                write(*,'(/1x,5A/)')    '### ERROR: Variable ''',
     &                  vname(1:lvar), ''' from ''', fname(1:lfnm),
     &                                    ''' is of unknown type.'
                goto 99
              endif
            else
              write(*,'(/1x,2A,I3,A/12x,4A/)') '### ERROR: Cannot ',
     &                  'make general inquiry for variable with ID =',
     &                   varid,     ' within',       'netCDF file ''',
     &                 fname(1:lfnm), ''': ', nf_strerror(ierr)
              goto 99
            endif
          else
            write(*,'(/1x,4A/12x,4A/)')     '### ERROR: Cannot find ',
     &                  'netCDF variable ID for ''',    vname(1:lvar),
     &                  ''' within netCDF', 'file ''', fname(1:lfnm),
     &                                    ''': ', nf_strerror(ierr)
            goto 99
          endif
        else
          write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot open netCDF ',
     &            'netCDF file ''',  fname(1:lfnm),  ''' for reading,',
     &                                               nf_strerror(ierr)
          goto 99
        endif


!         *****    *********    ******   *******    *********
!       ***   ***  *  ***  *   *   ***   ***   ***  *  ***  *
!       ***           ***     **   ***   ***   ***     ***
!         *****       ***    ***   ***   ***   **      ***
!             ***     ***    *********   ******        ***
!       ***   ***     ***    ***   ***   ***  **       ***
!         *****       ***    ***   ***   ***   ***     ***


        do irec=1,nrecs
          if (rec_dim > 0) then
            start(rec_dim)=irec ; count(rec_dim)=1
          endif

          if (vtype == nf_real) then
            ierr=nf_get_vara_real(ncid,varid,  start,count, bffr4)
          elseif (vtype == nf_double) then
            ierr=nf_get_vara_double(ncid,varid, start,count, bffr8)
          elseif (vtype == nf_int) then
            ierr=nf_get_vara_int  (ncid,varid,  start,count, ibffr)
          elseif (vtype == nf_short) then
            ierr=nf_get_vara_int2(ncid,varid,  start,count, ibff2)
          endif

          if (ierr == nf_noerr) then
            if (vtype == nf_real) then
              call compute_max_real (bffr4, spv4, count, irec,iarg,
     &                            vmin4, imin,jmin,kmin, rmin,ifmin,
     &                            vmax4, imax,jmax,kmax, rmax,ifmax)
            elseif (vtype == nf_double) then
              call compute_max_double(bffr8, spv8, count, irec,iarg,
     &                            vmin8, imin,jmin,kmin, rmin,ifmin,
     &                            vmax8, imax,jmax,kmax, rmax,ifmax)
            elseif (vtype == nf_int) then
              call compute_max_int (ibffr, ispv, count, irec,iarg,
     &                           ivmin, imin,jmin,kmin, rmin,ifmin,
     &                           ivmax, imax,jmax,kmax, rmax,ifmax)
            elseif (vtype == nf_short) then
              call compute_max_int2 (ibff2, ispv2, count, irec,iarg,
     &                           ivmin2, imin,jmin,kmin, rmin,ifmin,
     &                           ivmax2, imax,jmax,kmax, rmax,ifmax)
            endif
          else
            write(*,'(/1x,6A/12x,A/)')    '### ERROR: Cannot read ',
     &              'netCDF variable ''',vname(1:lvar),''' from ''',
     &                      fname(1:lfnm), '''.', nf_strerror(ierr)
          endif
        enddo !<-- irec

  99    if (ncid /= -1) then
          i=nf_close(ncid)
          if (i /= nf_noerr) then
            write(*,'(/1x,4A/12x,A/)')   '### ERROR: Cannot close ',
     &              'netCDF file ''', fname(1:lenstr(fname)), '''.',
     &                                               nf_strerror(i)
          endif
          if (ierr /= nf_noerr) stop
        endif
      enddo !<-- iarg

! Report the findings: at first print everything into buffer character
! string using natural format (*) for each type of number resulting in
! an arbitrary number of digits, no overflows, and no artificial round
! offs, them post-process the string to remove redundant information
! and print it out.

      if (vtype == nf_real) then
        call getarg(ifmin,fname) ; lfnm=lenstr(fname)
        write(minstr,*) 'min', vmin4, imin,jmin,kmin,
     &                       rmin, ' ', fname(1:lfnm)
        call getarg(ifmax,fname) ; lfnm=lenstr(fname)
        write(maxstr,*) 'max', vmax4, imax,jmax,kmax,
     &                       rmax, ' ', fname(1:lfnm)
      elseif (vtype == nf_double) then
        call getarg(ifmin,fname) ; lfnm=lenstr(fname)
        write(minstr,*) 'min', vmin8, imin,jmin,kmin,
     &                       rmin, ' ', fname(1:lfnm)
        call getarg(ifmax,fname) ; lfnm=lenstr(fname)
        write(maxstr,*) 'max', vmax8, imax,jmax,kmax,
     &                       rmax, ' ', fname(1:lfnm)
      elseif (vtype == nf_int) then
        call getarg(ifmin,fname) ; lfnm=lenstr(fname)
        write(minstr,*) 'min', ivmin, imin,jmin,kmin,
     &                       rmin, ' ', fname(1:lfnm)

        call getarg(ifmax,fname) ; lfnm=lenstr(fname)
        write(maxstr,*) 'max', ivmax, imax,jmax,kmax,
     &                       rmax, ' ', fname(1:lfnm)
      elseif (vtype == nf_short) then
        call getarg(ifmin,fname) ; lfnm=lenstr(fname)
        write(minstr,*) 'min', ivmin2,imin,jmin,kmin,
     &                       rmin, ' ', fname(1:lfnm)

        call getarg(ifmax,fname) ; lfnm=lenstr(fname)
        write(maxstr,*) 'max',ivmax2, imax,jmax,kmax,
     &                       rmax, ' ', fname(1:lfnm)
      endif
      write(*,*)
      if (imin > -1 .and. imax > -1) then
        call print_string(minstr, nvdims, rec_dim, nargs-1)
        call print_string(maxstr, nvdims, rec_dim, nargs-1)
      elseif (imin == -1 .and. imax == -1) then
        write(*,*) 'All values are labelled as missing values.'
      else
         write(*,*) '### ERROR: ncmaxval :: Algorithm failure.'
      endif
      write(*,*)
      end

! Post-process character string: suppress printing useless spatial
! indices beyond the actual number of spatial dimensions "nvdims", and
! record number if the variable does not have records.  Print file name
! only if multiple files are present; also reduce excessive white space
! between words and insert words to identify meaning of each number
! printed. The expected word sequence within the string is
!
!              key value  i  j  k  rec  file
!               1    2    3  4  5   6    7

      subroutine print_string(str, nvdims, rec_dim, nfiles)
      implicit none
      character(len=*) str
      character(len=256) pstr
      integer nvdims, rec_dim, nfiles, lstr, i,j, is,ie, kword,lenstr
      logical blank
      lstr=lenstr(str)
      do i=1,256
        pstr(i:i)=' ' !<-- blank out output string.
      enddo
      blank=.true. ; kword=0 ; j=1 ; is=0
      do i=1,lstr+1
        if (blank .and. str(i:i) /= ' ') then
          is=i   ; blank=.false.
        elseif (.not.blank .and. str(i:i) == ' ') then
          ie=i-1 ; blank=.true. ; kword=kword+1
c**       write(*,*) '''', str(is:ie), ''''
          if (kword == 1) then
            pstr(j:j+ie-is)=str(is:ie) ; j=j+ie-is+2
            pstr(j:j)='=' ; j=j+2
          elseif (kword == 2) then
            pstr(j:j+ie-is)=str(is:ie) ; j=j+ie-is+4
            pstr(j:j+1)='at' ; j=j+3
          elseif (kword < nvdims+3) then
            pstr(j:j+ie-is)=str(is:ie) ; j=j+ie-is+2
          elseif (kword == 6 .and. rec_dim > 0) then 
            j=j+2 ; pstr(j:j+4)='rec =' ; j=j+6
            pstr(j:j+ie-is)=str(is:ie) ; j=j+ie-is+2
          elseif (kword == 7 .and. nfiles > 1) then
            pstr(j:j+5)='file =' ; j=j+7
            pstr(j:j+ie-is)=str(is:ie)
          endif
        endif
      enddo
c**   write(*,*) '-------------------'
c**   write(*,'(2x,A)') str(1:lstr)    !<-- string as is
c**   write(*,*) '-------------------'
      lstr=lenstr(pstr)
      write(*,'(2x,A)') pstr(1:lstr)
      end
#endif

#ifndef TYPE
# define TYPE 1
# define compute_max_TYPE compute_max_int2
#elif TYPE == 1
# undef TYPE
# define TYPE 2
# undef compute_max_TYPE
# define compute_max_TYPE  compute_max_int
#elif TYPE == 2
# undef TYPE
# define TYPE 4
# undef compute_max_TYPE
# define compute_max_TYPE compute_max_real
#elif TYPE == 4
# undef TYPE
# define TYPE 8
# undef compute_max_TYPE
# define compute_max_TYPE compute_max_double
#endif

      subroutine compute_max_TYPE(bffr, spv, count, irec,ifile,
     &                        vmin, imin,jmin,kmin, rmin,ifmin,
     &                        vmax, imax,jmax,kmax, rmax,ifmax)
      implicit none
#if TYPE == 1
      integer(kind=2) bffr(*), spv, vmin,vmax, qmin,qmax
#elif TYPE == 2
      integer(kind=4) bffr(*), spv, vmin,vmax, qmin,qmax
#else
      real(kind=TYPE) bffr(*), spv, vmin,vmax, qmin,qmax
#endif
      integer count(3), irec,ifile, imin,jmin,kmin, rmin,ifmin,
     &                              imax,jmax,kmax, rmax,ifmax,
     &                    iqmin,jqmin,kqmin, iqmax,jqmax,kqmax,
     &                           i,j,k, nx,ny,nz, kshft,jshft

                               ! Note that "spv" may be both positive
      qmax=spv ; qmin=-spv     ! or negative, so switch their roles if
      if (qmax > qmin) then    ! needed to make sure that qmax is set
        qmax=-spv ; qmin=spv   ! to an unrealistically large negative
      endif                    ! value, while qmin is large positive.

      iqmin=-1 ; iqmax=-1      ! This initialization is needed only
      jqmin=-1 ; jqmax=-1      ! to handle the situation where all
      kqmin=-1 ; kqmax=-1      ! values in the arrays are missing.

      nx=max(1,count(1)) ; ny=max(1,count(2)) ; nz=max(1,count(3))

#ifdef VERBOSE
      write(*,*) 'before search: qmin =', qmin,'  qmax =', qmax
#endif
      do k=1,nz
        kshft=(k-1)*nx*ny
        do j=1,ny
          jshft=(j-1)*nx +kshft
          do i=1,nx
            if (bffr(i+jshft)  /= spv) then
              if (bffr(i+jshft) < qmin) then
                qmin=bffr(i+jshft)
                iqmin=i ; jqmin=j ; kqmin=k
              elseif (bffr(i+jshft) > qmax) then
                qmax=bffr(i+jshft)
                iqmax=i ; jqmax=j ; kqmax=k
              endif
            endif
          enddo
        enddo
      enddo
#ifdef VERBOSE
      write(*,*) 'rec', irec,' max =', qmax, ' at', iqmax,jqmax,kqmax
#endif
      if (qmin < vmin) then
        vmin=qmin ; imin=iqmin ; jmin=jqmin ; kmin=kqmin
        rmin=irec ; ifmin=ifile
      endif
      if (qmax > vmax) then
        vmax=qmax ; imax=iqmax ; jmax=jqmax ; kmax=kqmax
        rmax=irec ; ifmax=ifile
      endif
      end

#if TYPE < 8
# include "ncmaxval.F"
#endif
