      program ncvardiff

! A generic tool to compare fields with the same name in two netCDF
! files. It takes three arguments,
!
!              ncvardiff file1.nc file2.nc varname
!
! reads variable "varname" from both files, computes their difference
!
!              varname(from file1.nc) - varname(from file2.nc)
!
! and writes it into a newly created netCF file named
!
!                   diff_<varname>.nc.

c--#define VERBOSE

      implicit none
      logical matched_dims
      character(len=80) fname(2), tgname
      character(len=32) vname, attname, dimname(8)
      integer ncid, ncsrc, ncrfr, nctarg, varid, vidsr, vidrf, vtarg,
     &        vtype, vtype1, nvdims, vatts, unlimdimid, rec_dim, nrecs,
     &        dimids(8), idim(8), dimsize(8), start(8),count(8), size,
     &        ipass, irec, ierr, lstr, lfnm, i, lvar, lenstr, iargc
      real(kind=4),  allocatable, dimension(:) :: bffr4, rfrc4
      real(kind=8),   allocatable, dimension(:) :: bffr8, rfrc8
      integer(kind=4), allocatable, dimension(:) :: ibffr, irfrc
      integer(kind=2),  allocatable, dimension(:) :: ibff2, irfr2
      character(len=1), allocatable, dimension(:) :: bffrc, rfrcc

#include "netcdf.inc"

      if (iargc() == 3) then         ! Initialize/reset everything:
        call getarg(1,fname(1))      ! note that some variables below
        call getarg(2,fname(2))      ! are arrays.
        call getarg(3, vname)
        lvar=lenstr(vname)

        matched_dims=.true.   ; ncsrc=-1 ; nctarg=-1 ; ncrfr=-1
        dimsize=0 ; dimids=-1 ; rec_dim=-1 ; nrecs=1 ; vtype1=0
        start=1 ; count=1

        do ipass=1,2
          lfnm=lenstr(fname(ipass))
          ierr=nf_open(fname(ipass), nf_nowrite, ncid)
          if (ierr == nf_noerr) then
            ierr=nf_inq_varid(ncid, vname, varid)
            if (ierr == nf_noerr) then
              ierr=nf_inq_unlimdim(ncid, unlimdimid)
              if (ierr /= nf_noerr) then
                unlimdimid=-1
#ifdef VERBOSE
              else
                write(*,'(A,I4)') 'Found unlimited dimension #',
     &                                              unlimdimid
#endif
              endif
              ierr=nf_inq_var(ncid, varid,  vname,  vtype,
     &                              nvdims, dimids, vatts)
              if (ierr == nf_noerr) then
                if (ipass==1) then
                  ncsrc=ncid ; vidsr=varid ; vtype1=vtype
                else
                  ncrfr=ncid ; vidrf=varid
                endif

#ifdef VERBOSE
                write(*,'(A,3I4/A,8I4)') vname(1:lvar), vtype,
     &          nvdims, vatts, 'dimids=', (dimids(i), i=1,nvdims)
#endif
                do i=1,nvdims
                  ierr=nf_inq_dim (ncid, dimids(i), dimname(i),
     &                                              dimsize(i))
                  if (ierr == nf_noerr) then
                    lstr=lenstr(dimname(i))
                    if (i==nvdims) then
                      if (ipass==1) rec_dim=0
                      if (dimids(i) == unlimdimid) then
                        rec_dim=i
                      elseif (lstr > 3) then
                        if (dimname(i)(lstr-3:lstr) =='time') then
                          rec_dim=i
                        endif
                      endif
                    endif
                    if (ipass==1) then
                      count(i)=dimsize(i)
                    elseif (i==rec_dim) then
                      count(i)=min(count(i),dimsize(i))
                    elseif (dimsize(i) /= count(i)) then
                      matched_dims=.false.
                    endif
#ifdef VERBOSE
                    write(*,*)  dimname(i)(1:lstr), dimsize(i)
#endif
                  else
                    write(*,'(/1x,2A,I3,A/12x,4A/)')     '### ERROR: ',
     &                 'Cannot determine name and size of dimension #',
     &                   dimids(i),     ' within netCDF',    'file ''',
     &                 fname(ipass)(1:lfnm), ''': ', nf_strerror(ierr)
                    count(i)=0
                  endif
                enddo
                if (ipass==1) then
                  vtype1=vtype 
                elseif (matched_dims .and. vtype==vtype1) then

                  if (rec_dim > 0) then
                    nrecs=count(rec_dim) ; nvdims=rec_dim-1 
                  endif 
                  size=1
                  do i=1,nvdims
                    size=size*count(i)
                  enddo

                  if (size > 0) then 
                    if (vtype == nf_real) then
                      allocate(bffr4(size)) ; allocate(rfrc4(size))
                    elseif (vtype == nf_double) then
                      allocate(bffr8(size)) ; allocate(rfrc8(size))
                    elseif (vtype == nf_int) then
                      allocate(ibffr(size)) ; allocate(irfrc(size))
                    elseif (vtype == nf_short) then
                      allocate(ibff2(size)) ; allocate(irfr2(size))
                    elseif (vtype == nf_char) then
                      allocate(bffrc(size)) ; allocate(rfrcc(size))
                    else
                      write(*,'(/1x,5A/)')  '### ERROR: Variable ''',
     &                                   vname(1:lvar), ''' from ''',
     &                                  fname(1)(1:lenstr(fname(1))),
     &                                ''' has wrong or unknown type.'
                      goto 99
                    endif

                    tgname='diff_'/ /vname(1:lvar)/ /'.nc'
                    ierr=nf_create(tgname, nf_64bit_offset,nctarg)
                    do i=1,nvdims
                      lstr=lenstr(dimname(i))
                      ierr=nf_def_dim(nctarg, dimname(i)(1:lstr),
     &                                          count(i), idim(i))
                    enddo
                    if (rec_dim > 0) then
                      lstr=lenstr(dimname(rec_dim))
                      ierr=nf_def_dim(nctarg, dimname(rec_dim)(1:lstr),
     &                                    nf_unlimited, idim(rec_dim))
                    endif
                    ierr=nf_def_var(nctarg, vname(1:lvar), vtype,
     &                          max(nvdims,rec_dim), idim, vtarg)
                    do i=1,vatts
                      ierr=nf_inq_attname(ncid, varid, i, attname)
                      lstr=lenstr(attname)
                      ierr=nf_copy_att(ncid, varid, attname(1:lstr),
     &                                               nctarg, vtarg)
                    enddo
                    ierr=nf_enddef(nctarg)
#ifdef VERBOSE
                    write(*,*) 'created ', tgname(1:lenstr(tgname))
#endif
                  endif
                else
                  write(*,'(/1x,2A/)') '### ERROR: Detected missmatch',
     &                          ' in variable type or its dimensions.'
                endif
              else
                write(*,'(/1x,2A,I3,A/12x,4A/)') '### ERROR: Cannot ',
     &                  'make general inquiry for variable with ID =',
     &                   varid,     ' within',       'netCDF file ''',
     &                 fname(ipass)(1:lfnm), ''': ', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,4A/12x,4A/)')   '### ERROR: Cannot find ',
     &                  'netCDF variable ID for ''',    vname(1:lvar),
     &            ''' within netCDF', 'file ''', fname(ipass)(1:lfnm),
     &                ''': ', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,A/)')      '### ERROR: Cannot open ',
     &      'netCDF file ''', fname(ipass)(1:lfnm), ''' for reading,',
     &                                             nf_strerror(ierr)
          endif
        enddo !<-- ipass



!         *****    *********    ******   *******    *********
!       ***   ***  *  ***  *   *   ***   ***   ***  *  ***  *
!       ***           ***     **   ***   ***   ***     ***
!         *****       ***    ***   ***   ***   **      ***
!             ***     ***    *********   ******        ***
!       ***   ***     ***    ***   ***   ***  **       ***
!         *****       ***    ***   ***   ***   ***     ***


        if (nctarg /= -1  .and.  ierr == nf_noerr) then
          write(*,*) ! <-- just a blank line
          do irec=1,nrecs
            if (rec_dim > 0) then
              start(rec_dim)=irec ; count(rec_dim)=1
            endif

            if (vtype == nf_real) then
              ierr=nf_get_vara_real (ncsrc,vidsr, start,count, bffr4)
            elseif (vtype == nf_double) then
              ierr=nf_get_vara_double(ncsrc,vidsr, start,count, bffr8)
            elseif (vtype == nf_int) then
              ierr=nf_get_vara_int   (ncsrc,vidsr, start,count, ibffr)
            elseif (vtype == nf_short) then
              ierr=nf_get_vara_int2  (ncsrc,vidsr, start,count, ibff2)
            elseif (vtype == nf_char) then
              ierr=nf_get_vara_text (ncsrc, vidsr, start,count, bffrc)
            endif
            if (ierr == nf_noerr) then
              if (vtype == nf_real) then
                ierr=nf_get_vara_real(ncrfr,vidrf, start,count, rfrc4)
              elseif (vtype == nf_double) then
                ierr=nf_get_vara_double(ncrfr,vidrf,start,count,rfrc8)
              elseif (vtype == nf_int) then
                ierr=nf_get_vara_int (ncrfr,vidrf, start,count, irfrc)
              elseif (vtype == nf_short) then
                ierr=nf_get_vara_int2(ncrfr,vidrf, start,count, irfr2)
              elseif (vtype == nf_char) then
                ierr=nf_get_vara_text(ncrfr,vidrf, start,count, rfrcc)
              endif
              if (ierr == nf_noerr) then
                if (vtype == nf_real) then
                  call compute_diff_real (bffr4, rfrc4, size, irec)
                elseif (vtype == nf_double) then
                  call compute_diff_double (bffr8, rfrc8, size, irec)
                elseif (vtype == nf_int) then
                  call compute_diff_int4   (ibffr, irfrc, size, irec)
                elseif (vtype == nf_short) then
                  call compute_diff_int2   (ibff2, irfr2, size, irec)
                elseif (vtype == nf_char) then
                  call compute_diff_text   (bffrc, rfrcc, size, irec)
                endif

                if (vtype == nf_real) then
                  ierr=nf_put_vara_real(nctarg,vtarg, start,count,
     &                                                       bffr4)
                elseif (vtype == nf_double) then
                  ierr=nf_put_vara_double(nctarg,vtarg, start,count,
     &                                                        bffr8)
                elseif (vtype == nf_int) then
                  ierr=nf_put_vara_int   (nctarg,vtarg, start,count,
     &                                                        ibffr)
                elseif (vtype == nf_short) then
                  ierr=nf_put_vara_int2  (nctarg,vtarg, start,count,
     &                                                        ibff2)
                elseif (vtype == nf_char) then
                  ierr=nf_put_vara_text  (nctarg,vtarg, start,count,
     &                                                       bffrc)
                endif

                if (ierr /= nf_noerr) then
                  write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot ',
     &                    'write netCDF variable ''', vname(1:lvar),
     &                                    '''.',  nf_strerror(ierr)
                endif
              else
                write(*,'(/1x,6A/12x,A/)')  '### ERROR: Cannot read ',
     &          'netCDF variable ''',   vname(1:lvar),   ''' from ''',
     &          fname(2)(1:lenstr(fname(2))),'''.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,6A/12x,A/)')    '### ERROR: Cannot read ',
     &          'netCDF variable ''',   vname(1:lvar),   ''' from ''',
     &          fname(1)(1:lenstr(fname(1))),'''.', nf_strerror(ierr)
            endif
          enddo !<-- irec
          write(*,*) ! <-- just a blank line
        endif

  99    if (ncsrc /= -1) then
          ierr=nf_close(ncsrc)
          if (ierr /= nf_noerr) then
            write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot close ',
     &              'netCDF file ''', fname(1)(1:lenstr(fname(1))),
     &                                   ''':', nf_strerror(ierr)
          endif
        endif
        if (ncrfr/= -1) then
          ierr=nf_close(ncrfr)
          if (ierr /= nf_noerr) then
            write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot close ',
     &              'netCDF file ''', fname(2)(1:lenstr(fname(2))),
     &                                    ''':', nf_strerror(ierr)
          endif
        endif
        if (nctarg /= -1) then
          ierr=nf_close (nctarg)
          if (ierr /= nf_noerr) then
            write(*,'(/1x,4A/12x,A/)')  '### ERROR: Cannot close ',
     &                  'netCDF file ''', tgname(1:lenstr(tgname)),
     &                                    ''':', nf_strerror(ierr)
          else
c            call system('ncview '/ /tgname(1:lenstr(tgname)))
          endif
        endif
      else
        write(*,'(/1x,A,1x,A/32x,A/)') 'The usage of ''ncvardiff''',
     &          'should be:', 'ncvardiff file1.nc file2.nc varname'
      endif
      end


      subroutine compute_diff_double(bffr, rfrc, size, irec)
      implicit none
      integer size, i, ndiff, irec
      real(kind=8) bffr(size), rfrc(size),  max_diff
      ndiff=0 ; max_diff=0.D0
      do i=1,size
        if (bffr(i)==rfrc(i)) then
          bffr(i)=0.D0
        else
          bffr(i)=bffr(i)-rfrc(i)
          max_diff=max(max_diff,abs(bffr(i)))
          ndiff=ndiff+1
        endif
      enddo
      if (ndiff > 0) then
        write(*,*) 'rec', irec, ndiff, 'points different, ',
     &                   'maximum difference ', max_diff
      else
        write(*,*) 'rec', irec, ' --> no difference'
      endif
      end

      subroutine compute_diff_real(bffr, rfrc, size, irec)
      implicit none
      integer size, i, ndiff, irec
      real(kind=4) bffr(size), rfrc(size),  max_diff
      ndiff=0 ; max_diff=0.0
      do i=1,size
        if (bffr(i)==rfrc(i)) then
          bffr(i)=0.0
        else
          bffr(i)=bffr(i)-rfrc(i)
          max_diff=max(max_diff,abs(bffr(i)))
          ndiff=ndiff+1
        endif
      enddo
      if (ndiff > 0) then
        write(*,*) 'rec', irec, ndiff, 'points different, ',
     &                   'maximum difference ', max_diff
      else
        write(*,*) 'rec', irec, ' --> no difference'
      endif
      end

      subroutine compute_diff_int4 (bffr, rfrc, size, irec)
      implicit none
      integer size, i, ndiff, irec
      integer(kind=4) bffr(size), rfrc(size), max_diff
      ndiff=0 ; max_diff=0
      do i=1,size
        if (bffr(i)==rfrc(i)) then
          bffr(i)=0
        else
          bffr(i)=bffr(i)-rfrc(i)
          max_diff=max(max_diff,abs(bffr(i)))
          ndiff=ndiff+1
        endif
      enddo
      if (ndiff > 0) then
        write(*,*) 'rec', irec, ndiff, 'points different, ',
     &                   'maximum difference ', max_diff
      else
        write(*,*) 'rec', irec, ' --> no difference'
      endif
      end

      subroutine compute_diff_int2 (bffr, rfrc, size, irec)
      implicit none
      integer size, i, ndiff, irec
      integer(kind=2) bffr(size), rfrc(size), max_diff
      ndiff=0 ; max_diff=0
      do i=1,size
        if (bffr(i)==rfrc(i)) then
          bffr(i)=0
        else
          bffr(i)=bffr(i)-rfrc(i)
          max_diff=max(max_diff,abs(bffr(i)))
          ndiff=ndiff+1
        endif
      enddo
      if (ndiff > 0) then
        write(*,*) 'rec', irec, ndiff, 'points different, ',
     &                   'maximum difference ', max_diff
      else
        write(*,*) 'rec', irec, ' --> no difference'
      endif
      end


      subroutine compute_diff_text (bffr, rfrc, size, irec)
      implicit none
      integer size, i, ndiff, irec
      character(len=1), dimension(size) :: bffr, rfrc
      ndiff=0
      do i=1,size
        if (bffr(i)==rfrc(i)) then
          bffr(i)=' '
        else
          ndiff=ndiff+1
        endif
      enddo
      if (ndiff > 0) then
        write(*,*) 'rec', irec, ndiff, 'points in byte ',
     &                            'field are different.'
      else
        write(*,*) 'rec', irec, ' --> no difference'
      endif
      end
