      program plot_coast_traj

! A program to visualize shoreline and trajectory for point
! measurements using NCAR graphics.

      implicit none
      character(len=64)  shoreline, point_data_file
      integer nargs, max_pts, ncst,npts, ierr, i,lcst,lpnt,lenstr
      real(kind=8), allocatable :: Points(:,:), bffr(:,:)
      real(kind=8) xmin,ymin,xmax,ymax, xc,yc,csA
      real(kind=4), allocatable, dimension(:) :: x,y
      real(kind=4) xl,xr,yb,yt, ratio

#define POSTSCRIPT
#ifdef POSTSCRIPT
      integer(kind=4) ngpswk, unused
#endif

      nargs=iargc() ; lcst=0 ; lpnt=0
      if (nargs > 0) then
        call getarg(1,shoreline) ; lcst=lenstr(shoreline)
        if (nargs == 2) then
          call getarg(2,point_data_file) ; lpnt=lenstr(point_data_file)
        endif
      else
        write(*,'(1x,2A/ /8x,2A/)')   'This command-line operator ',
     &         'should be used as', 'plot_coast_traj costline.dat ',
     &                                      '[point_data_file.dat]'
        stop
      endif

      open(unit=11, file=shoreline, status='old', iostat=ierr)
      if (ierr == 0) then
        ncst=0 ; max_pts=256 ; allocate(Points(2,max_pts))
        do while(ierr == 0)
          ncst=ncst+1
          if (ncst > max_pts) then
            allocate(bffr(2,max_pts))
            bffr(1:2,1:ncst-1)=Points(1:2,1:ncst-1)
            deallocate(Points)
            max_pts=2*max_pts
            allocate(Points(2,max_pts))
            Points(1:2,1:ncst-1)=bffr(1:2,1:ncst-1)
            deallocate(bffr)
          endif
          read(11,*,iostat=ierr) Points(2,ncst), Points(1,ncst)
        enddo
        close(unit=11)
        ncst=ncst-1
        xmin=Points(1,1) ; ymin=Points(2,1)
        xmax=Points(1,1) ; ymax=Points(2,1);
        do i=2,ncst
          xmin=min(xmin,Points(1,i)) ; ymin=min(ymin,Points(2,i))
          xmax=max(xmax,Points(1,i)) ; ymax=max(ymax,Points(2,i))
        enddo
        xc=0.5D0*(xmin+xmax) ; yc=0.5D0*(ymin+ymax)
        csA=cos(3.14159265358979323846D0*yc/180.D0)

        write(*,*) 'xmin =',xmin, '   ymin=',ymin
        write(*,*) 'xmax =',xmax, '   ymax=',ymax
        write(*,*) 'cos(MidLat=', yc, ') =', csA

        allocate(x(max_pts),y(max_pts))
        xl=xmin-xc ; xr=xmax-xc ; yb=ymin-yc ; yt=ymax-yc
        do i=1,ncst             ! subtract lat-lon of the center
          x(i)=Points(1,i)-xc   ! to avoid roundoff errors before
          y(i)=Points(2,i)-yc   ! converting from double to single
        enddo                   ! precision.

        ratio=csA*(xr-xl)/(yt-yb)      ! Adjust aspect ratio
        if (ratio > 1.D0) then         ! of visual portion of
          yt=yt*ratio ; yb=yb*ratio    ! the viewport to make
        else                           ! the contour appear
          xl=xl/ratio ; xr=xr/ratio    ! naturally when using
        endif                          ! Mercator projection.

#ifdef POSTSCRIPT
        call gopks(6,unused) ;   call ngsetc('ME','file.ps')
        call gopwk(1,2, ngpswk('PS','PORTRAIT','MONOCHROME'))
        call gacwk(1)
#else
        call opngks
#endif
        call set(0.01,0.99,0.01,0.99, xl,xr,yb,yt, 1)
        write(*,*) 'Plotting shoreline, num points =',ncst
        call curved(x,y, ncst)

      else
        write(*,*) '### ERRORL: Cannot open shoreline file ''',
     &                    shoreline(1:lcst), ''' for reading.'
        stop
      endif

      if (nargs == 2) then
        open(unit=12, file=point_data_file, status='old', iostat=ierr)
        if (ierr == 0) then
          npts=0
          do while(ierr == 0)
            npts=npts+1
            if (npts > max_pts) then
              allocate(bffr(2,max_pts))
              bffr(1:2,1:npts-1)=Points(1:2,1:npts-1)
              deallocate(Points)
              max_pts=2*max_pts
              allocate(Points(2,max_pts))
              Points(1:2,1:npts-1)=bffr(1:2,1:npts-1)
              deallocate(bffr)
            endif
            read(12,*,iostat=ierr) Points(2,npts), Points(1,npts)
          enddo
          close(unit=12)
          npts=npts-1
          write(*,*) 'Plotting boat trajectory, npts =', npts

          do i=1,npts
            x(i)=Points(1,i)-xc ; y(i)=Points(2,i)-yc
          enddo
          call bullet(x(1),y(1))
          do i=2,npts
            call line(x(i-1),y(i-1), x(i),y(i))
            call bullet(x(i),y(i))
          enddo
        else
          write(*,*) 'Skip plotting boat trajectory: ',
     &                          'input file not found'
        endif
      endif

      call frame
#ifdef POSTSCRIPT
      call gdawk(1) ; call gclwk(1) ; call gclks
      call system('ps2epsi file.ps')
      call system('epstopdf file.epsi')
      call system('/bin/rm -f file.ps file.epsi')
#else
      call clsgks
#endif
      end


      subroutine bullet(x,y)            ! Draw bullet (a circle filled
      implicit none                     ! with default color) centered
      real(kind=4) x,y                  ! at user coordinates (x,y).
      integer i
      integer, parameter :: M=16
      real(kind=4), parameter :: r=0.003125  !<-- radius of bullet
      real(kind=4) cffX,cffY, xi(M),eta(M), wL,wR,wB,wT, xL,xR,yB,yT
      real(kind=4), save :: cffX_bak=0., cffY_bak=0., xi0(M), eta0(M)
      real(kind=8) cff,arg
                                                   ! initialize only
      call getset(wL,wR,wB,wT, xL,xR,yB,yT, i)     ! when viewport
      cffX=r*(xR-xL)/(wR-wL)                       ! has been changed
      cffY=r*(yT-yB)/(wT-wB)
      if (cffX /= cffX_bak .or. cffY /= cffY_bak) then
        write(*,*) 'initialing bullet, r =',r
        cff=6.2831853071795862D0/dble(M)
        do i=1,M
          arg=cff*dble(i)
          xi0(i)= cffX*cos(arg)
          eta0(i)=cffY*sin(arg)
        enddo
        cffX_bak=cffX
        cffY_bak=cffY
      endif

c**     write(*,*) x,y
      do i=1,M
        xi(i) =x+xi0(i)
        eta(i)=y+eta0(i)
      enddo

c**   call gscr(1,2, 0.,0.,0.)  !<-- color must be
c**   call gsfaci(1)            !<-- set externally
      call gfa(M,xi,eta)
      end
