      program read_esri_data

! Reads shoreline data from ESRI-style shape file and converts it into
! a simple ASCII format containing (latitude,longitude) coordinates for
! one point of the shoreline on each line of the file.

      implicit none
      integer nargs, ierr, i, lesr,lshr, lenstr
      character(len=64) esri_shape_file, shoreline

! Variables associated with ESRI file header, for specification see
! http://www.esri.com/library/whitepapers/pdfs/shapefile.pdf

      integer(kind=4) FileCode, unused(5), FileLength,Version,ShapeType
      real(kind=8) Xmin,Ymin,Xmax,Ymax, Zmin,Zmax, Mmin,Mmax
      integer(kind=4) RecordNum,ContentLength !<-- record header
      real(kind=8) Box(4)                     !--> variables associated
      integer(kind=4) NumParts, NumPoints     !    with record content
      integer(kind=4), allocatable :: Parts(:)
      real(kind=8), allocatable :: Points(:,:)

      nargs=iargc() ; lesr=0 ; lshr=0
      if (nargs == 2) then
        call getarg(1,esri_shape_file) ; call getarg(2,shoreline)
        lesr=lenstr(esri_shape_file)   ; lshr=lenstr(shoreline)
      else
        write(*,'(/1x,A/,/12x,A/,/1x,A/,/4x,A/3(/1x,2A)/)')
     &               'This command-line operator should be used as',
     &                   'read_esri_data esri_file.shp shoreline.dat',
     &     'It reads user-specified ESRI-style binary shapefile (see',
     &   'http://www.esri.com/library/whitepapers/pdfs/shapefile.pdf',
     &   'for technical specification), and converts it into simple ',
     &  'ASCII','format containing (latitude,longitude) coordinates ',
     &   'for each point','of the shoreline (one pair of numbers on ',
     &                                     'each line of the file).'
        stop
      endif

      open(unit=11, file=esri_shape_file(1:lesr), form='unformatted',
     &                   access='stream', status='old', iostat=ierr)
      if (ierr == 0) then
        read(11,iostat=ierr) FileCode, unused, FileLength, Version,
     &        ShapeType, Xmin,Ymin,Xmax,Ymax, Zmin,Zmax, Mmin,Mmax
        if (ierr == 0) then
          call swap_i4(FileCode)
          call swap_i4(FileLength)
          write(*,*) 'FileCode = ', FileCode,
     &            ' FileLength = ', FileLength
          write(*,*)  'Version = ', Version,
     &             ' ShapeType = ', ShapeType
          write(*,*) 'Xmin = ',Xmin, ' Ymin = ',Ymin
          write(*,*) 'Xmax = ',Xmax, ' Ymax = ',Ymax
          write(*,*) 'Zmin = ',Zmin, ' Zmax = ',Zmax
          write(*,*) 'Mmin = ',Mmin, ' Mmax = ',Mmax

          read(11,iostat=ierr) RecordNum,ContentLength
          if (ierr == 0) then
            call swap_i4(RecordNum)
            call swap_i4(ContentLength)
            write(*,*) 'RecordNum = ', RecordNum,
     &            ' ContentLength = ', ContentLength

            read(11,iostat=ierr) ShapeType, Box, NumParts, NumPoints
            if (ierr == 0) then
              write(*,*) 'ShapeType = ', ShapeType
              write(*,*) 'Box = ', Box
              write(*,*) 'NumParts = ', NumParts,
     &                 ' NumPoints = ', NumPoints

              allocate(Parts(NumParts),Points(2,NumPoints))
              read(11,iostat=ierr) Parts,Points
              if (ierr == 0) then
c**             write(*,*) 'Parts:'
c**             do i=1,NumParts
c**               write(*,*) Parts(i)
c**             enddo
c**             write(*,*) 'Points:'
c**             do i=1,NumPoints
c**               write(*,*) i, Points(2,i), Points(1,i)
c**             enddo
                close(unit=11)

                open(unit=12, file=shoreline, form='formatted')
                do i=1,NumPoints
                  write(12,*) Points(2,i), Points(1,i)
                enddo
                close(unit=12)
              else
                write(*,'(/1x,2A/)')     '### ERROR: Cannot read ',
     &                   'Parts,Points content of ESRI shapefile.' 
              endif
            else
              write(*,'(/1x,3A/)')   '### ERROR: Cannot read the ',
     &                      'third element of ESRI-shape header, ',
     &                      'ShapeType, Box, NumParts, NumPoints.'
            endif
          else
            write(*,'(/1x,3A/)')     '### ERROR: Cannot read the ',
     &                     'second element of ESRI-shape header, ',
     &                                  'RecordNum,ContentLength.'
          endif
        else
          write(*,'(/1x,2A/)') '### ERROR: Cannot read the first ',
     &     'element of ESRI-shape header, FileCode,...,Mmin,Mmax.'
        endif
      else
        write(*,'(/1x,4A/)')     '### ERROR: Cannot open ESRI ',
     & 'shapefile ''',esri_shape_file(1:lesr),''' for reading.'
      endif
      end
