#if !defined U_TYPE && !defined V_TYPE
c--#define BILINEAR
c-# define BIHALF
# define HALFSHIFT
c--#define REF_WITH_CORR


      module multi_grid_arrays
        integer isize2,jsize2, isize4,jsize4,   isize8,jsize8,
     &                         isize16,jsize16, isize32,jsize32
        real(kind=8), dimension(:), allocatable ::  xlon2,ylat2,
     &                                xlon4,ylat4,  xlon8,ylat8,
     &                              xlon16,ylat16, xlon32,ylat32
        real(kind=4), dimension(:,:), allocatable :: wrk2, wrk4,
     &                                        wrk8, wrk16,wrk32
# ifdef REF_WITH_CORR
     &                         , res,crr2, res2,crr4, res4,crr8
     &                                , res8,crr16, res16,crr32
# endif
# ifdef HALFSHIFT
        real(kind=8), dimension(:), allocatable ::  xlons,ylats
        real(kind=4), dimension(:,:), allocatable :: wrks
# endif
      end module

      module ref2d_patch_arrays
        real(kind=8), allocatable, dimension(:) :: pth
        integer, allocatable, dimension(:) :: ipth,jpth
      end module



      subroutine init_frac_interp(ref_ratio, isize,jsize, xlon,ylat,
     &                                           Lm,Mm, lon_r,lat_r)
      use multi_grid_arrays
      use ref2d_patch_arrays

      implicit none
      integer ref_ratio, isize,jsize, Lm,Mm, npth
      real(kind=8) xlon(isize), lon_r(0:Lm+1,0:Mm+1),
     &             ylat(jsize), lat_r(0:Lm+1,0:Mm+1)

      if (ref_ratio > 1) then
        isize2=2*(isize-1) ; allocate(xlon2(isize2))
        jsize2=2*(jsize-1) ; allocate(ylat2(jsize2))
        call ref_axis(isize, xlon, xlon2)
        call ref_axis(jsize, ylat, ylat2)
# ifdef REF_WITH_CORR
        allocate (res(isize,jsize))
        allocate(crr2(isize2,jsize2))
# endif
        allocate(wrk2(isize2,jsize2))
      endif

      if (ref_ratio > 3) then
        isize4=2*(isize2-1) ; allocate(xlon4(isize4))
        jsize4=2*(jsize2-1) ; allocate(ylat4(jsize4))
        call ref_axis(isize2, xlon2, xlon4)
        call ref_axis(jsize2, ylat2, ylat4)
# ifdef REF_WITH_CORR
        allocate(res2(isize2,jsize2))
        allocate (crr4(isize4,jsize4))
# endif
        allocate (wrk4(isize4,jsize4))
      endif

      if (ref_ratio > 7) then
        isize8=2*(isize4-1) ; allocate(xlon8(isize8))
        jsize8=2*(jsize4-1) ; allocate(ylat8(jsize8))
        call ref_axis(isize4, xlon4, xlon8)
        call ref_axis(jsize4, ylat4, ylat8)
# ifdef REF_WITH_CORR
        allocate(res4(isize4,jsize4))
        allocate(crr8(isize8,jsize8))
# endif
        allocate(wrk8(isize8,jsize8))
      endif

      if (ref_ratio > 15) then
        isize16=2*(isize8-1) ; allocate(xlon16(isize16))
        jsize16=2*(jsize8-1) ; allocate(ylat16(jsize16))
        call ref_axis(isize8, xlon8, xlon16)
        call ref_axis(jsize8, ylat8, ylat16)
# ifdef REF_WITH_CORR
        allocate(res8(isize8,jsize8))
        allocate(crr16(isize16,jsize16))
# endif
        allocate(wrk16(isize16,jsize16))
      endif

      if (ref_ratio > 31) then
        isize32=2*(isize16-1) ; allocate(xlon32(isize32))
        jsize32=2*(jsize16-1) ; allocate(ylat32(jsize32))
        call ref_axis(isize16, xlon16, xlon32)
        call ref_axis(jsize16, ylat16, ylat32)
# ifdef REF_WITH_CORR
        allocate(res16(isize16,jsize16))
        allocate(crr32(isize32,jsize32))
# endif
        allocate(wrk32(isize32,jsize32))
      endif



      if (ref_ratio == 1) then
        call init_interp_2d(isize,jsize,  xlon,ylat,
     &                          Lm,Mm, lon_r,lat_r)
        npth=isize*jsize
      else
# ifdef HALFSHIFT
        if (ref_ratio == 2) then
          allocate(xlons(isize2-1)) ; allocate(ylats(jsize2-1))
          call half_axis(isize2, xlon2, xlons)
          call half_axis(jsize2, ylat2, ylats)
          call init_interp_2d(isize2-1,jsize2-1, xlons,ylats,
     &                                      Lm,Mm, lon_r,lat_r)
          allocate(wrks(isize2-1,jsize2-1))
          npth=(isize2-1)*(jsize2-1)
        elseif (ref_ratio == 4) then
          allocate(xlons(isize4-1)) ; allocate(ylats(jsize4-1))
          call half_axis(isize4, xlon4, xlons)
          call half_axis(jsize4, ylat4, ylats)
          call init_interp_2d(isize4-1,jsize4-1, xlons,ylats,
     &                                      Lm,Mm, lon_r,lat_r)
          allocate(wrks(isize4-1,jsize4-1))
          npth=(isize4-1)*(jsize4-1)
        elseif (ref_ratio == 8) then
          allocate(xlons(isize8-1)) ; allocate(ylats(jsize8-1))
          call half_axis(isize8, xlon8, xlons)
          call half_axis(jsize8, ylat8, ylats)
          call init_interp_2d(isize8-1,jsize8-1, xlons,ylats,
     &                                      Lm,Mm, lon_r,lat_r)
          allocate(wrks(isize8-1,jsize8-1))
          npth=(isize8-1)*(jsize8-1)
        elseif (ref_ratio == 16) then
          allocate(xlons(isize16-1)) ; allocate(ylats(jsize16-1))
          call half_axis(isize16, xlon16, xlons)
          call half_axis(jsize16, ylat16, ylats)
          call init_interp_2d(isize16-1,jsize16-1, xlons,ylats,
     &                                      Lm,Mm, lon_r,lat_r)
          allocate(wrks(isize16-1,jsize16-1))
          npth=(isize16-1)*(jsize16-1)
        elseif (ref_ratio == 32) then
          allocate(xlons(isize32-1)) ; allocate(ylats(jsize32-1))
          call half_axis(isize32, xlon32, xlons)
          call half_axis(jsize32, ylat32, ylats)
          call init_interp_2d(isize32-1,jsize32-1, xlons,ylats,
     &                                        Lm,Mm, lon_r,lat_r)
          allocate(wrks(isize32-1,jsize32-1))
          npth=(isize32-1)*(jsize32-1)
# else
        if (ref_ratio == 2) then
          call init_interp_2d(isize2,jsize2, xlon2,ylat2,
     &                                Lm,Mm, lon_r,lat_r)
          npth=isize2*jsize2
        elseif (ref_ratio == 4) then
          call init_interp_2d(isize4,jsize4, xlon4,ylat4,
     &                                Lm,Mm, lon_r,lat_r)
          npth=isize4*jsize4
        elseif (ref_ratio == 8) then
          call init_interp_2d(isize8,jsize8, xlon8,ylat8,
     &                                Lm,Mm, lon_r,lat_r)
          npth=isize8*jsize8
        elseif (ref_ratio == 16) then
          call init_interp_2d(isize16,jsize16, xlon16,ylat16,
     &                                  Lm,Mm, lon_r,lat_r)
          npth=isize16*jsize16
        elseif (ref_ratio == 32) then
          call init_interp_2d(isize32,jsize32, xlon32,ylat32,
     &                                Lm,Mm, lon_r,lat_r)
          npth=isize32*jsize32
# endif
        else
          write(*,'(/1x,2A,I4,1x,A)') '### ERROR: Wrong  value of ',
     &      'ref_ratio =', ref_ratio, 'must be 1,2,4,8,16, or 32.'
          stop
        endif
      endif

      allocate(pth(npth))
      allocate(ipth(npth))
      allocate(jpth(npth))
      end

#endif



#if defined U_TYPE
# define frac_int_TYPE frac_int_u2d
# define interp_TYPE interp_u2d
#elif defined V_TYPE
# define frac_int_TYPE frac_int_v2d
# define interp_TYPE interp_v2d
#else
# define frac_int_TYPE frac_int_r2d
# define interp_TYPE interp_r2d
#endif

      subroutine frac_int_TYPE(ref_ratio, isize,jsize, src,
     &                                    Lm,Mm, msk, targ)
      use multi_grid_arrays
      implicit none
      integer ref_ratio, isize,jsize, Lm,Mm
      real(kind=4) src(isize,jsize)
      integer(kind=2) msk(0:Lm+1,0:Mm+1)
#if defined U_TYPE
      real(kind=4) targ(1:Lm+1,0:Mm+1)
#elif defined V_TYPE
      real(kind=4) targ(0:Lm+1,1:Mm+1)
#else
      real(kind=4) targ(0:Lm+1,0:Mm+1)
#endif

      call lm_mask_patch (isize, jsize, src)
      if (ref_ratio > 1) then
        call ref_2d(isize,jsize,  src, wrk2)
        call lm_mask_patch(isize2, jsize2, wrk2)
#ifdef REF_WITH_CORR
        call coarse_res(isize, jsize, wrk2, src, res)
        call ref_2d(isize, jsize,  res,    crr2)
        call lm_mask_patch(isize2, jsize2, crr2)
        wrk2=wrk2+crr2
#endif
      endif

      if (ref_ratio > 3) then
        call ref_2d(isize2,jsize2,  wrk2, wrk4)
        call lm_mask_patch(isize4,jsize4, wrk4)
#ifdef REF_WITH_CORR
        call coarse_res(isize2,jsize2, wrk4, wrk2, res2)
        call ref_2d(isize2, jsize2,  res2, crr4)
        call lm_mask_patch(isize4, jsize4, crr4)
        wrk4=wrk4+crr4
#endif
      endif

      if (ref_ratio > 7) then
        call ref_2d(isize4,jsize4,  wrk4,wrk8)
        call lm_mask_patch(isize8, jsize8,wrk8)
#ifdef REF_WITH_CORR
        call coarse_res(isize4,jsize4, wrk8,wrk4, res4)
        call ref_2d(isize4,jsize4,   res4,crr8)
        call lm_mask_patch(isize8,jsize8, crr8)
        wrk8=wrk8+crr8
#endif
      endif

      if (ref_ratio > 15) then
        call ref_2d(isize8,jsize8,  wrk8,wrk16)
        call lm_mask_patch(isize16,jsize16, wrk16)
#ifdef REF_WITH_CORR
        call coarse_res(isize8,jsize8, wrk16,wrk8, res8)
        call ref_2d (isize8,jsize8,   res8, crr16)
        call lm_mask_patch(isize16,jsize16, crr16)
        wrk16=wrk16+crr16
#endif
      endif

      if (ref_ratio > 31) then
        call ref_2d(isize16,jsize16,  wrk16,wrk32)
        call lm_mask_patch(isize32,jsize32, wrk32)
#ifdef REF_WITH_CORR
        call coarse_res(isize16,jsize16, wrk32,wrk16, res16)
        call ref_2d (isize16,jsize16,   res16, crr32)
        call lm_mask_patch(isize32,jsize32, crr32)
        wrk32=wrk32+crr32
#endif
      endif



      if (ref_ratio == 1) then
        call interp_TYPE(isize,jsize,     src,   Lm,Mm, msk,targ)
      else
#ifdef HALFSHIFT
        if (ref_ratio == 2) then
          call halfshift(isize2,    jsize2,    wrk2, wrks)
          call interp_TYPE(isize2-1,jsize2-1,  wrks, Lm,Mm, msk,targ)
        elseif (ref_ratio == 4) then
          call halfshift(isize4,    jsize4,    wrk4, wrks)
          call interp_TYPE(isize4-1,jsize4-1,  wrks, Lm,Mm, msk,targ)
        elseif (ref_ratio == 8) then
          call halfshift(isize8,    jsize8,    wrk8, wrks)
          call interp_TYPE(isize8-1,jsize8-1,  wrks, Lm,Mm, msk,targ)
        elseif (ref_ratio == 16) then
          call halfshift(isize16,    jsize16,  wrk16, wrks)
          call interp_TYPE(isize16-1,jsize16-1,wrks, Lm,Mm, msk,targ)
        elseif (ref_ratio == 32) then
          call halfshift(isize32,  jsize32,    wrk32, wrks)
          call interp_TYPE(isize32-1,jsize32-1,wrks, Lm,Mm, msk,targ)
        endif
#else
        if (ref_ratio == 2) then
          call interp_TYPE(isize2,jsize2,   wrk2,  Lm,Mm, msk,targ)
        elseif (ref_ratio == 4) then
          call interp_TYPE(isize4,jsize4,   wrk4,  Lm,Mm, msk,targ)
        elseif (ref_ratio == 8) then
          call interp_TYPE(isize8,jsize8,   wrk8,  Lm,Mm, msk,targ)
        elseif (ref_ratio == 16) then
          call interp_TYPE(isize16,jsize16, wrk16, Lm,Mm, msk,targ)
        elseif (ref_ratio == 32) then
          call interp_TYPE(isize32,jsize32, wrk32, Lm,Mm, msk,targ)
        endif
#endif
      endif
      end

#undef interp_TYPE
#undef frac_int_TYPE

#ifndef U_TYPE
# ifndef V_TYPE

      subroutine ref_axis(isize, src, targ)
      implicit none
      integer isize, i,ic
      real(kind=8) src(isize), targ(2*(isize-1))
      write(*,*) 'ref_axis', isize, ' -->', 2*(isize-1)
      do i=1,isize-1
        ic=2*i-1
        targ(ic  )=0.75D0*src(i) +0.25D0*src(i+1)
        targ(ic+1)=0.25D0*src(i) +0.75D0*src(i+1)
      enddo
      end

      module sync_vars_ref2d
      integer :: comm_j_indx=0
      end module sync_vars_ref2d

      subroutine ref_2d(isize,jsize, src,targ)
      use sync_vars_ref2d
      implicit none
      integer isize,jsize
      real(kind=4) src(isize,jsize), targ(2*(isize-1),2*(jsize-1))
      comm_j_indx=0  !<-- initialize
C$OMP PARALLEL SHARED(isize,jsize, src,targ)
      call ref_2d_thread(isize,jsize, src,targ)
C$OMP END PARALLEL
      end


      subroutine ref_2d_thread(isize,jsize, src,targ)
      use sync_vars_ref2d
      implicit none
      integer isize,jsize, i,j, itg,jtg
      real(kind=4) src(isize,jsize), targ(2*(isize-1),2*(jsize-1))
      real(kind=8) wg00,vu00, wg10,vu10, wg01,vu01, wg11,vu11
#  ifdef BILINEAR
      real(kind=8), parameter :: Cntr=9.D0, Side=3.D0, Corn=1.D0
#  elif defined BIHALF
      real(kind=8), parameter :: Cntr=8.5D0, Side=3.5D0, Corn=0.5D0
#  else
      real(kind=8), parameter :: Cntr=8.D0, Side=4.D0, Corn=0.D0
#  endif

#  include "spval.h"
#  define THRESHOLD 6.5D0

      do while(.true.)  !<-- originally do j=1,jsize-1
C$OMP CRITICAL(ref_2d_sync)
        comm_j_indx=comm_j_indx+1 ; j=comm_j_indx
C$OMP END CRITICAL(ref_2d_sync)
        if (j > jsize-1) return
#  ifdef VERBOSE
        if (j == 1) write(*,*) 'refined to -->',2*(isize-1),2*(jsize-1)
#  endif

        jtg=2*j-1
        do i=1,isize-1
          itg=2*i-1

          vu01=0.D0 ; wg01=0.D0 ; vu11=0.D0 ; wg11=0.D0
          vu00=0.D0 ; wg00=0.D0 ; vu10=0.D0 ; wg10=0.D0

          if (src(i,j) < spv) then
            wg00=wg00+Cntr ; vu00=vu00+Cntr*src(i,j)

            wg10=wg10+Side ; vu10=vu10+Side*src(i,j)

            wg01=wg01+Side ; vu01=vu01+Side*src(i,j)

            wg11=wg11+Corn ; vu11=vu11+Corn*src(i,j)
          endif

          if (src(i+1,j) < spv) then
            wg00=wg00+Side ; vu00=vu00+Side*src(i+1,j)

            wg10=wg10+Cntr ; vu10=vu10+Cntr*src(i+1,j)

            wg01=wg01+Corn ; vu01=vu01+Corn*src(i+1,j)

            wg11=wg11+Side ; vu11=vu11+Side*src(i+1,j)
          endif

          if (src(i,j+1) < spv) then
            wg00=wg00+Side ; vu00=vu00+Side*src(i,j+1)

            wg10=wg10+Corn ; vu10=vu10+Corn*src(i,j+1)

            wg01=wg01+Cntr ; vu01=vu01+Cntr*src(i,j+1)

            wg11=wg11+Side ; vu11=vu11+Side*src(i,j+1)
          endif

          if (src(i+1,j+1) < spv) then
            wg00=wg00+Corn ; vu00=vu00+Corn*src(i+1,j+1)

            wg10=wg10+Side ; vu10=vu10+Side*src(i+1,j+1)

            wg01=wg01+Side ; vu01=vu01+Side*src(i+1,j+1)

            wg11=wg11+Cntr ; vu11=vu11+Cntr*src(i+1,j+1)
          endif

          if (wg00 > THRESHOLD) then
            targ(itg,jtg)=vu00/wg00
          else
            targ(itg,jtg)=spv_set
          endif

          if (wg10 > THRESHOLD) then
            targ(itg+1,jtg)=vu10/wg10
          else
            targ(itg+1,jtg)=spv_set
          endif

          if (wg01 > THRESHOLD) then
            targ(itg,jtg+1)=vu01/wg01
          else
            targ(itg,jtg+1)=spv_set
          endif

          if (wg11 > THRESHOLD) then
            targ(itg+1,jtg+1)=vu11/wg11
          else
            targ(itg+1,jtg+1)=spv_set
          endif
        enddo
      enddo
      end

#  ifdef HALFSHIFT
      subroutine half_axis(isize, src, targ)
      implicit none
      integer isize, i
      real(kind=8) src(isize), targ(isize-1)
      write(*,*) 'half_axis', isize, ' -->', isize-1
      do i=1,isize-1
        targ(i)=0.5D0*(src(i)+src(i+1))
      enddo
      end

      subroutine halfshift(isize,jsize, src,targ)
      use sync_vars_ref2d
      implicit none
      integer isize,jsize
      real(kind=4) src(isize,jsize), targ(isize-1,jsize-1)
      comm_j_indx=0  !<-- initialize
C$OMP PARALLEL SHARED(isize,jsize, src,targ)
      call hlfsh_thread(isize,jsize, src,targ)
C$OMP END PARALLEL
      end

      subroutine hlfsh_thread(isize,jsize, src,targ)
      use sync_vars_ref2d
      implicit none
      integer isize,jsize, i,j
      real(kind=4) src(isize,jsize), targ(isize-1,jsize-1)
      real(kind=8) wgt,vlu
#   include "spval.h"

      do while(.true.)  !<-- originally do j=1,jsize-1
C$OMP CRITICAL(ref_2d_sync)
        comm_j_indx=comm_j_indx+1 ; j=comm_j_indx
C$OMP END CRITICAL(ref_2d_sync)
        if (j > jsize-1) return
#   ifdef VERBOSE
        if (j == 1) write(*,*) 'refined to -->',2*(isize-1),2*(jsize-1)
#   endif
        do i=1,isize-1
          wgt=0.D0 ; vlu=0.D0
          if (src(i,j) < spv) then
            wgt=wgt+1.D0 ; vlu=vlu+src(i,j)
          endif
          if (src(i+1,j) < spv) then
            wgt=wgt+1.D0 ; vlu=vlu+src(i+1,j)
          endif
          if (src(i+1,j+1) < spv) then
            wgt=wgt+1.D0 ; vlu=vlu+src(i+1,j+1)
          endif
          if (src(i,j+1) < spv) then
            wgt=wgt+1.D0 ; vlu=vlu+src(i,j+1)
          endif
          if (wgt > 0.5D0) then
            targ(i,j)=vlu/wgt
          else
            targ(i,j)=spv_set
          endif
        enddo
      enddo
      end
#  endif



      subroutine lm_mask_patch(isize,jsize, src)
      use ref2d_patch_arrays
      implicit none
      integer isize,jsize, i,j, itrans, ip,np
      real(kind=4) src(isize,jsize)
      real(kind=8) wgt,vlu
      logical lland
#  include "spval.h"
#  ifdef VERBOSE
      write(*,*) 'lm_mask_patch ', isize,jsize
#  endif

      np=0
      do j=2,jsize-1
        do i=2,isize-1
          if (src(i,j) > spv) then

            if (src(i+1,j-1) < spv) then
              lland=.false.
            else
              lland=.true.
            endif

            wgt=0.D0 ; vlu=0.D0
            itrans=0

            if (src(i+1,j) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+src(i+1,j)
              if (lland) itrans=itrans+1
              lland=.false.
            else
              lland=.true.
            endif

            if (src(i+1,j+1) < spv) then
              wgt=wgt+0.71D0 ; vlu=vlu+0.71D0*src(i+1,j+1)
              if (lland) itrans=itrans+1
              lland=.false.
            else
              lland=.true.
            endif

            if (src(i,j+1) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+src(i,j+1)
              if (lland) itrans=itrans+1
              lland=.false.
            else
              lland=.true.
            endif

            if (src(i-1,j+1) < spv) then
              wgt=wgt+0.71D0 ; vlu=vlu+0.71D0*src(i-1,j+1)
              if (lland) itrans=itrans+1
              lland=.false.
            else
              lland=.true.
            endif

            if (src(i-1,j) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+src(i-1,j)
              if (lland) itrans=itrans+1
              lland=.false.
            else
              lland=.true.
            endif

            if (src(i-1,j-1) < spv) then
              wgt=wgt+0.71D0 ; vlu=vlu+0.71D0*src(i-1,j-1)
              if (lland) itrans=itrans+1
              lland=.false.
            else
              lland=.true.
            endif

            if (src(i,j-1) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+src(i,j-1)
              if (lland) itrans=itrans+1
              lland=.false.
            else
              lland=.true.
            endif

            if (src(i+1,j-1) < spv) then
              wgt=wgt+0.71D0 ; vlu=vlu+0.71D0*src(i+1,j-1)
              if (lland) itrans=itrans+1
            endif

            if (itrans < 2 .and. wgt > 3.2D0) then
              np=np+1
              ipth(np)=i             ! Threshold settings:
              jpth(np)=j             ! 3.42 convex corners
              pth(np)=vlu/wgt        ! 2.71 diagonal lines
            endif                    ! 2.41 straight lines
          endif
        enddo
      enddo
      do ip=1,np
        src(ipth(ip),jpth(ip))=pth(ip)
      enddo
      end






#  ifdef REF_WITH_CORR
      subroutine coarse_res(lon,lat, targ,src, res)
      implicit none
      integer lon,lat, i,j,ic,jc
      real(kind=4) targ(2*(lon-1),2*(lat-1)), src(lon,lat),
     &                               res(lon,lat), wgt,vlu
#   include "spval.h"
#   ifdef VERBOSE
      write(*,*) 'entered coarse_res, lon,lat =', lon,lat
#   endif

      do j=2,lat-1
        jc=2*(j-1)
        do i=2,lon-1
          if (src(i,j) < spv) then
            ic=2*(i-1)
            wgt=0.D0       ; vlu=0.D0
            if (targ(ic,jc) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+targ(ic,jc)
            endif
            if (targ(ic+1,jc) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+targ(ic+1,jc)
            endif
            if (targ(ic+1,jc+1) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+targ(ic+1,jc+1)
            endif
            if (targ(ic,jc+1) < spv) then
              wgt=wgt+1.D0 ; vlu=vlu+targ(ic,jc+1)
            endif
            if (wgt > 0.D0) then
              res(i,j)=src(i,j)-vlu/wgt
            else
              res(i,j)=0.D0
            endif
          else
            res(i,j)=spv_set
          endif
        enddo
      enddo

      do j=1,lat
        if (src(1,j) < spv) then
          res(1,j)=0.D0
        else
          res(1,j)=spv_set
        endif
      enddo
      do j=1,lat
        if (src(lon,j) < spv) then
          res(lon,j)=0.D0
        else
          res(lon,j)=spv_set
        endif
      enddo
      do i=1,lon
        if (src(i,1) < spv) then
          res(i,1)=0.D0
        else
          res(i,1)=spv_set
        endif
      enddo
      do i=1,lon
        if (src(i,lat) < spv) then
          res(i,lat)=0.D0
        else
          res(i,lat)=spv_set
        endif
      enddo
      end
#  endif /* REF_WITH_CORR */

#  define U_TYPE
#  include "refine.F"
#  undef U_TYPE
#  define V_TYPE
#  include "refine.F"
# endif
#endif

