      program rescale_time

! Read user-specified timing variable which is expected to have valid
! cycle_length (stored as attribute), rescale it to new user-specified
! cycle_length,
!
!       time_var = time_var * desired_cycle_length/cycle_length
!
! and write both the modified time variable a the attribute back into
! the file.  Should be used as
!
!          rescale_time file.nc time_var desired_cycle_length

c--#define VERBOSE

      implicit none
      character(len=64) fname, time_var, str
      real(kind=8), allocatable, dimension(:):: time
      real(kind=8) new_cycle_length, cycle_length
      integer nargs, ncid, varid, vtype,ndims,dimids(4),natts,
     &        ntimes, ierr,i, lfnm,lvar,lenstr, read_str_dble

#include "netcdf.inc"

      ntimes=0 ; cycle_length=0.D0 ; new_cycle_length=0.D0
      nargs=iargc()
      if (nargs == 3) then
        call getarg(1,fname) ; lfnm=lenstr(fname)
        ierr=nf_open(fname(1:lfnm), nf_write, ncid)
        if (ierr == nf_noerr) then
          call getarg(2,time_var) ; lvar=lenstr(time_var)
          ierr=nf_inq_varid(ncid, time_var(1:lvar), varid)
          if (ierr == nf_noerr) then
            ierr=nf_inq_var(ncid, varid, str, vtype,  ndims,
     &                                        dimids, natts)
            if (ierr == nf_noerr) then
              if (ndims == 1) then
                ierr=nf_inq_dimlen(ncid, dimids(1), ntimes)
                if (ierr == nf_noerr) then
                  ierr=nf_get_att_double(ncid, varid, 'cycle_length',
     &                                                 cycle_length)
                  if (ierr == nf_noerr) then
                    call getarg(3,str)
                    ierr=read_str_dble(str,new_cycle_length)
                    if (ierr == 0) then
                      allocate(time(ntimes))
                      ierr=nf_get_var_double(ncid, varid, time)
                      if (ierr == nf_noerr) then
#ifdef VERBOSE
                        write(*,*) 'ntimes =', ntimes,
     &                     '  cycle_length =', cycle_length,
     &                 '  new_cycle_length =', new_cycle_length
                        write(*,*) 'time =', (time(i), i=1,ntimes)
#endif
                        do i=1,ntimes
                          time(i)=time(i)*new_cycle_length/cycle_length
                        enddo
#ifdef VERBOSE
                        write(*,*) 'rescaled =', (time(i), i=1,ntimes)
#endif
                      else
                        write(*,'(/1x,6A/12x,A/)')    '### ERROR: ',
     &                   'Cannot read variable ''', time_var(1:lvar),
     &                                   ''' from ''', fname(1:lfnm),
     &                                      '''.', nf_strerror(ierr)
                      endif
                    else
                      write(*,'(/1x,3A/)')     '### ERROR: Wrong  ',
     &                       'argument #3, desired cycle_length = ',
     &                                           str(1:lenstr(str))
                    endif
                  else
                    write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot ',
     &              'read attribute ''cycle_length'' for variable ''',
     &                  time_var(1:lvar), ''' from ''', fname(1:lfnm),
     &                                       '''.', nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,2A,I4,1x,3A/12x,A/)')   '### ERROR: ',
     &               'Cannot find length of dimension #',   dimids(1),
     &               'in ''', fname(1:lfnm), '''.', nf_strerror(ierr)

                endif
              else
                write(*,'(/1x,4A,I4,A/)')    '### ERROR: Variable ''',
     &           time_var(1:lvar), ''' must have a single dimension ',
     &                                      'instead of ', ndims, '.'
              endif
            else
              write(*,'(/1x,6A/12x,A/)')    '### ERROR: Cannot make ',
     &        'inquiry for variable ''', time_var(1:lvar), ''' in ''',
     &                        fname(1:lfnm), '''.', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,6A/12x,A/)')      '### ERROR: Cannot find ',
     &                    'variable ''', time_var(1:lvar), ''' in ''',
     &                        fname(1:lfnm), '''.', nf_strerror(ierr)
          endif
        else
          write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot open netCDF ',
     &                    'file ''', fname(1:lfnm), ''' for writing.',
     &                                              nf_strerror(ierr)
        endif

! At this moment reading is expected to be complete (unless an error
! occurred) and all the necessary variables have meaningful values, so
! write the updated versions of 'cycle_length' and time back into file.

        if ( ierr == nf_noerr .and. cycle_length > 0.D0 .and.
     &         ntimes > 0 .and. new_cycle_length > 0.D0 ) then
          ierr=nf_redef(ncid)
          if (ierr == nf_noerr) then
            ierr=nf_put_att_double(ncid, varid, 'cycle_length',
     &                         nf_double, 1, new_cycle_length)
            if (ierr == nf_noerr) then
              ierr=nf_enddef(ncid)
              if (ierr == nf_noerr) then
                ierr=nf_put_var_double(ncid, varid, time)
                if (ierr == nf_noerr) then
                  ierr=nf_close(ncid)
                  if (ierr == nf_noerr) then
                    write(*,'(/2(1x,A),2(1x,A,F16.8)/)')  'rescaled ',
     &                         time_var(1:lvar), ' from cycle_length',
     &                         cycle_length, ' to', new_cycle_length
                  else
                    write(*,'(/1x,4A/12x,A/)')   '### ERROR: Cannot ',
     &                   'close netCDF file ''', fname(1:lfnm), '''.',
     &                                              nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,6A/12x,A/)')     '### ERROR: Cannot ',
     &            'write variable ''', time_var(1:lvar), ''' into ''',
     &                        fname(1:lfnm), '''.', nf_strerror(ierr)
                endif
              else
                write(*,'(/1x,4A/12x,A/)')   '### ERROR: Cannot set ',
     &                                'netCDF file ''', fname(1:lfnm),
     &                  ''' back into input mode.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,6A/12x,A/)')   '### ERROR: Cannot write ',
     &                   'attribute ''cycle_length'' for variable ''',
     &                    time_var(1:lvar), ''' in ''', fname(1:lfnm),
     &                                       '''.', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot set netCDF ',
     &          'file ''', fname(1:lfnm), ''' into redefinition mode.',
     &                                              nf_strerror(ierr)
          endif
        endif
      else
        write(*,'(/1x,A/,/12x,A/)')   'This command should be used as',
     &       'rescale_time file.nc time_var_name desired_cycle_length'
        write(*,'(1x,3A/1x,A/,/12x,A/,/1x,2A/1x,A/)') 'It reads user-',
     &         'specified time variable ''time_var_name'' along with ',
     &             'its attribute', '''cycle_length'' and rescales it',
     &       'time_var = time_var * desired_cycle_length/cycle_length',
     &           'then writes the modified versions of time variable ',
     &             'and attribute cycle_length', 'back into the file.'
      endif
      end
