      program set_rmask

! A basic program to compute "rmask" from "raw" topography and place
! it into the same grid netCDF file. This operation is implemented as
! an independent step rather than called from within the procedure to
! compute topography in order to allow re-computation of initial "raw"
! topography stored in the file without disturbing "mask_rho", which
! may be subject to elaborate editing. by hand

      implicit none
      character(len=16) string
      character(len=80) grid_file
      real(kind=8) hmin
      real(kind=8), dimension(:,:), allocatable :: rmask
      integer nargs, iargc, ncgrd, Lm,Mm, i,j, ierr, lgrd,
     &                             lenstr, read_str_dble
#include "netcdf.inc"

      nargs=iargc()
      if (nargs==2) then
        call getarg(1,string)
        ierr=read_str_dble(string, hmin)
        if (ierr==0) then
          call getarg(2,grid_file)
          lgrd=lenstr(grid_file)

          ierr=nf_open (grid_file, nf_write, ncgrd)
          if (ierr==nf_noerr) then
            call roms_find_dims (ncgrd, grid_file, Lm,Mm, i)

            allocate (rmask(0:Lm+1,0:Mm+1))
            call get_var_by_name_double (ncgrd, 'hraw', rmask)

            do j=0,Mm+1
              do i=0,Lm+1
                if (rmask(i,j) > hmin) then
                  rmask(i,j)=1.D0
                else
                  rmask(i,j)=0.D0
                endif
              enddo
            enddo

            call put_var_by_name_double (ncgrd, 'mask_rho', rmask)
            ierr=nf_close (ncgrd)
          else
            write(*,'(/1x,4A/12x,A/)')     '### ERROR: Cannot open ',
     &              'ROMS grid netCDF file ''',    grid_file(1:lgrd),
     &              ''' in read-write mode.',      nf_strerror(ierr)
          endif
        else
          i=lenstr(string)
            write(*,'(/1x,4A/)')  '### ERROR: The first argument ''',
     &                           string(1:i), ''' must be a number.'
        endif
      else
       write(*,'(/1x,2A/ /8x,A/ /1x,2A/1x,A/)')  'Usage of "rmask" ',
     &   'should be:',   'rmask hmin roms_grid_file.nc',    'where ',
     &   '"hmin" is threshold depth: points with unsmoothed "hraw" ',
     &    'topography shallower than this value are set to "land".'
      endif
      end
