#if defined U_TYPE
# define spline_interp_TYPE spline_interp_u2d
# define spline_interp_TYPE_thread spline_interp_u2d_thread
# define spline_interp_TYPE_tile spline_interp_u2d_tile
# define xii xi_iu
# define etaj eta_ju
# define imin 1
# define jmin 0
#elif defined V_TYPE
# define spline_interp_TYPE spline_interp_v2d
# define spline_interp_TYPE_thread spline_interp_v2d_thread
# define spline_interp_TYPE_tile spline_interp_v2d_tile
# define xii xi_iv
# define etaj eta_jv
# define imin 0
# define jmin 1
#else
# define spline_interp_TYPE spline_interp_r2d
# define spline_interp_TYPE_thread spline_interp_r2d_thread
# define spline_interp_TYPE_tile spline_interp_r2d_tile
# define xii xi_ir
# define etaj eta_jr
# define imin 0
# define jmin 0
#endif


#if !defined U_TYPE && !defined V_TYPE
      module spline_work_arrays
       integer,save :: alloc_spl_wrk_size
       real(kind=8), dimension(:), allocatable :: srX,srY,sXY,sYX
      end module spline_work_arrays


      subroutine spline_2d(nx,ny, src, srX,srY,sXY,sYX)
      implicit none
      integer nx,ny, i,j
      real(kind=4), dimension(nx,ny) :: src
      real(kind=8), dimension(nx,ny) :: srX,srY,sXY,sYX
      real(kind=8) cff, CF(nx), FC(ny), error

      CF(1)=0.5D0
      do j=1,ny
        srX(1,j)=1.5D0*(src(2,j)-src(1,j))
      enddo
      do i=2,nx-1,+1
        CF(i)=1.D0/(4.D0-CF(i-1))
        cff=CF(i)
        do j=1,ny
          srX(i,j)=cff*(3.D0*(src(i+1,j)-src(i-1,j)) -srX(i-1,j))
        enddo
      enddo
      cff=1.D0/(2.D0-CF(nx-1))
      do j=1,ny
        srX(nx,j)=cff*(3.D0*(src(nx,j)-src(nx-1,j))-srX(nx-1,j))
      enddo
      do i=nx-1,1,-1
        cff=CF(i)
        do j=1,ny
          srX(i,j)=srX(i,j)-cff*srX(i+1,j)
        enddo
      enddo

      FC(1)=0.5D0
      do i=1,nx
        srY(i,1)=1.5D0*(src(i,2)-src(i,1))
        sXY(i,1)=1.5D0*(srX(i,2)-srX(i,1))
      enddo
      do j=2,ny-1,+1
        FC(j)=1.D0/(4.D0-FC(j-1))
        cff=FC(j)
        do i=1,nx
          srY(i,j)=cff*( 3.D0*(src(i,j+1)-src(i,j-1)) -srY(i,j-1))
          sXY(i,j)=cff*( 3.D0*(srX(i,j+1)-srX(i,j-1)) -sXY(i,j-1))
        enddo
      enddo
      cff=1.D0/(2.D0-FC(ny-1))
      do i=1,nx
        srY(i,ny)=cff*( 3.D0*(src(i,ny)-src(i,ny-1)) -srY(i,ny-1))
        sXY(i,ny)=cff*( 3.D0*(srX(i,ny)-srX(i,ny-1)) -sXY(i,ny-1))
      enddo
      do j=ny-1,1,-1
        cff=FC(j)
        do i=1,nx
          srY(i,j)=srY(i,j)-cff*srY(i,j+1)
          sXY(i,j)=sXY(i,j)-cff*sXY(i,j+1)
        enddo
      enddo

      CF(1)=0.5D0
      do j=1,ny
        sYX(1,j)=1.5D0*(srY(2,j)-srY(1,j))
      enddo
      do i=2,nx-1,+1
        CF(i)=1.D0/(4.D0-CF(i-1))
        cff=CF(i)
        do j=1,ny
          sYX(i,j)=cff*(3.D0*(srY(i+1,j)-srY(i-1,j)) -sYX(i-1,j))
        enddo
      enddo
      cff=1.D0/(2.D0-CF(nx-1))
      do j=1,ny
        sYX(nx,j)=cff*(3.D0*(srY(nx,j)-srY(nx-1,j))-sYX(nx-1,j))
      enddo
      do i=nx-1,1,-1
        cff=CF(i)
        do j=1,ny
          sYX(i,j)=sYX(i,j)-cff*sYX(i+1,j)
        enddo
      enddo

      error=0.D0
      do j=1,ny
        do i=1,nx
# ifdef VERBOSE
          cff=abs((sXY(i,j)-sYX(i,j))/(sXY(i,j)+sYX(i,j)))
          if (cff > error) error=cff
# endif
          sXY(1,j)=0.5D0*(sXY(i,j)+sYX(i,j))
        enddo
      enddo
# ifdef VERBOSE
      write(*,*) 'spline_2d: maximum mixed derivative error: ', error
# endif
      end

#endif


      subroutine spline_interp_TYPE(nx,ny, src, Lm,Mm,targ)
      use spline_work_arrays
      implicit none
      integer nx,ny, Lm,Mm
      real(kind=4), dimension(*) :: src, targ
!-->
      integer numthreads,trd, nsub_x,nsub_y, range,
     &      my_first,my_last, tile, istr,iend,jstr,jend
C$    integer omp_get_num_threads, omp_get_thread_num

      if (allocated(srX)) then
        if (nx*ny > alloc_spl_wrk_size) then
          deallocate(srX) ; deallocate(srY)
          deallocate(sXY) ; deallocate(sYX)
        endif
      endif
      if (.not.allocated(srX)) then
        alloc_spl_wrk_size=nx*ny
        allocate(srX(alloc_spl_wrk_size))
        allocate(srY(alloc_spl_wrk_size))
        allocate(sXY(alloc_spl_wrk_size))
        allocate(sYX(alloc_spl_wrk_size))
        write(*,*) 'spline_interp: allocated workspace ',
     &             'arrays, size =', alloc_spl_wrk_size
      endif

      call spline_2d(nx,ny, src, srX,srY,sXY,sYX)

C$OMP PARALLEL SHARED(nx,ny, src,srX,srY,sXY, Lm,Mm, targ)
C$OMP& PRIVATE(numthreads,trd, nsub_x,nsub_y, range,my_first,my_last,
C$OMP&                                    tile, istr,iend,jstr,jend)

      numthreads=1 ; trd=0
C$    numthreads=omp_get_num_threads() ; trd=omp_get_thread_num()

      call set_tiles(Lm,Mm, nsub_x,nsub_y)

c      nsub_x=1 ; nsub_y=1 !<-- for testing parallel correctness only
c      write(*,*) 'numthreads,trd =', numthreads,trd

      range=(nsub_x*nsub_y +numthreads-1)/numthreads
      my_first=trd*range
      my_last=min(my_first + range-1, nsub_x*nsub_y-1)

      do tile=my_first,my_last,+1
        call comp_tile_bounds(tile, Lm,Mm, nsub_x,nsub_y,
     &                                  istr,iend,jstr,jend)

        if (istr==1) istr=istr-1 ; if (iend==Lm) iend=iend+1
        if (jstr==1) jstr=jstr-1 ; if (jend==Mm) jend=jend+1

        call spline_interp_TYPE_tile( istr,iend,jstr,jend,
     &              nx,ny, src,srX,srY,sXY, Lm,Mm, targ)
      enddo
C$OMP END PARALLEL
      end


      subroutine spline_interp_TYPE_tile( istr,iend,jstr,jend,
     &                  nx,ny, src,srX,srY,sXY, Lm,Mm, targ)
      use horiz_inter_arrays
      implicit none
      integer istr,iend,jstr,jend, nx,ny, Lm,Mm
      real(kind=4) src(nx,ny), targ(imin:Lm+1,jmin:Mm+1)

      real(kind=8), dimension(nx,ny) :: srX,srY,sXY
      integer i,j, iL,iR, jL,jR
      real(kind=8) px,qx,pqx, HxR,HxL,GxR,GxL,
     &             py,qy,pqy, HyR,HyL,GyR,GyL
c**   real(kind=8) xi,eta
c**   write(*,*) 'istr,iend,jstr,jend =', istr,iend,jstr,jend

      istr=max(istr,imin)  ; jstr=max(jstr,jmin)

! Cubic Hermite spline interpolation in two dimensions.  Commented
! out (by kept here for reference) is the canonical symmetric form of
! Hermite basis functions defined on the interval -1/2 < xi < +1/2.
! The actually used code is optimized to remove repeated operations by
! expressing them in via px = 1/2 + xi and qx = 1/2 - xi resulting in
! a more compact form.

      do j=jstr,jend
        do i=istr,iend
          iL=int(xii(i,j)) ; px=xii(i,j)-dble(iL)
          iR=iL+1          ; qx=1.D0-px ; pqx=px*qx
          GxR=-px*pqx      ; HxR=px*(px+2.D0*pqx)
          GxL= qx*pqx      ; HxL=qx*(qx+2.D0*pqx)

c**          xi=xii(i,j)  -dble(iL)-0.5D0
c**          HxR=(0.5D0+xi)**2 * (2.D0-2.D0*xi)
c**          HxL=(0.5D0-xi)**2 * (2.D0+2.D0*xi)
c**          GxR=(0.5D0+xi)**2 * (xi-0.5D0)
c**          GxL=(0.5D0-xi)**2 * (xi+0.5D0)

          jL=int(etaj(i,j)) ; py=etaj(i,j)-dble(jL)
          jR=jL+1           ; qy=1.D0-py ; pqy=py*qy
          GyR=-py*pqy       ; HyR=py*(py+2.D0*pqy)
          GyL= qy*pqy       ; HyL=qy*(qy+2.D0*pqy)

c**          eta=etaj(i,j)-dble(jL)-0.5D0
c**          HyR=(0.5D0+eta)**2 * (2.D0-2.D0*eta)
c**          HyL=(0.5D0-eta)**2 * (2.D0+2.D0*eta)
c**          GyR=(0.5D0+eta)**2 * (eta-0.5D0)
c**          GyL=(0.5D0-eta)**2 * (eta+0.5D0)

          targ(i,j) = HxL*HyL*src(iL,jL) + HxR*HyL*src(iR,jL)
     &              + HxL*HyR*src(iL,jR) + HxR*HyR*src(iR,jR)

     &              + GxL*HyL*srX(iL,jL) + GxR*HyL*srX(iR,jL)
     &              + GxL*HyR*srX(iL,jR) + GxR*HyR*srX(iR,jR)

     &              + HxL*GyL*srY(iL,jL) + HxR*GyL*srY(iR,jL)
     &              + HxL*GyR*srY(iL,jR) + HxR*GyR*srY(iR,jR)

     &              + GxL*GyL*sXY(iL,jL) + GxR*GyL*sXY(iR,jL)
     &              + GxL*GyR*sXY(iL,jR) + GxR*GyR*sXY(iR,jR)
        enddo
      enddo
      end


#undef spline_interp_TYPE
#undef spline_interp_TYPE_tile
#undef spline_interp_TYPE_thread
#undef xii
#undef etaj
#undef imin
#undef jmin

#if !defined U_TYPE && !defined V_TYPE
# define U_TYPE
# include "spline_2d.F"
# undef U_TYPE
# define V_TYPE
# include "spline_2d.F"
#endif
