! Change units of user-specified time variable from seconds to days
! or vice versa.  This involves reading modifying, and writing back
! both the variable itself and its attribute "units".

#ifdef DAYS_TO_SECONDS
      program time_to_seconds
#else
      program time_to_days 
#endif
      implicit none
      character(len=64) time_var, fname, units, vname
      real(kind=8), allocatable, dimension(:) :: time
      integer nargs, iarg, ncid, varid, vtype, ndims,
     &        dimids(4), natts, alloc_size, ntimes, i,
     &                 ierr, latt, lfnm, lvar, lenstr
#include "phys_const.h"
#include "netcdf.inc"

      alloc_size=0 ; ntimes=1 !<-- default ; a single-record file may
      nargs=iargc()           !        not have time dimension at all 
      if (nargs > 1) then
        call getarg(1,time_var) ; lvar=lenstr(time_var) 
      else
        write(*,'(/1x,2A/2(/12x,A/))')    'These commands should be ',
     &  'used as', 'time_to_days time_var_name file1.nc file2.nc ...',
     &          'time_to_seconds time_var_name file1.nc file2.nc ...'
        stop
      endif

      do iarg=2,nargs
        call getarg(iarg,fname) ; lfnm=lenstr(fname)
        ierr=nf_open(fname(1:lfnm), nf_write, ncid)
        if (ierr == nf_noerr) then
          ierr=nf_inq_varid(ncid, time_var(1:lvar), varid)
          if (ierr == nf_noerr) then
            ierr=nf_inq_var(ncid, varid, vname, vtype, ndims,
     &                                         dimids, natts)
            if (ierr == nf_noerr) then
              if (ndims < 2) then
                if (ndims == 1) then
                  ierr=nf_inq_dimlen(ncid, dimids(1), ntimes)
                endif
                if (ierr == nf_noerr) then
                  ierr=nf_get_att_text(ncid, varid, 'units', units)
                  if (ierr == nf_noerr) then
                    latt=lenstr(units) 
#ifdef DAYS_TO_SECONDS
                    if (units(1:6) == 'second') then
                      write(*,'(/1x,5A/)')       'Time variable ''',
     &                  time_var(1:lvar), ''' in ''', fname(1:lfnm),
     &                ''' is already in seconds. No need to change.'
                      goto 1 
                    elseif (units(1:3) == 'day') then
#else
                    if (units(1:3) == 'day') then
                      write(*,'(/1x,5A/)')       'Time variable ''',
     &                  time_var(1:lvar), ''' in ''', fname(1:lfnm),
     &                  ''' is already in days. No need to change.'  
                      goto 1 
                    elseif (units(1:6) == 'second') then
#endif
                      if (ntimes > alloc_size) then               
                        if (allocated(time)) deallocate(time)
                        allocate(time(ntimes)); alloc_size=ntimes
                      endif
                      ierr=nf_get_var_double(ncid, varid, time)
                      if (ierr == nf_noerr) then

! Note that attrubute "units" may be just one word (single or plural) 
! "second", "seconds", "day", "days" or maybe more complicated, like
! say, "days since 00:00am January 1, 1900". The code segment below is
! designed to change only the first word (keeping plural ending "s" if
! present in the original version), and copy the rest of the string
! as is, if it is more that just a single word.

#ifdef DAYS_TO_SECONDS
                        do i=1,ntimes
                          time=time*day2sec
                        enddo
                        if (latt > 3) then
                          units='second'/ /units(4:latt)
                        else
                          units='seconds'
                        endif
#else
                        do i=1,ntimes
                          time=time*sec2day    
                        enddo
                        if (latt > 6) then
                          units='day'/ /units(7:latt)
                        else
                          units='days'
                        endif 
#endif
                        latt=lenstr(units)
                        ierr=nf_redef(ncid)
                        if (ierr == nf_noerr) then
                          ierr=nf_put_att_text(ncid, varid, 'units',
     &                                         latt, units(1:latt))
                          if (ierr == nf_noerr) then
                            ierr=nf_enddef(ncid)
                            if (ierr == nf_noerr) then
                              ierr=nf_put_var_double(ncid, varid, time)
                              if (ierr == nf_noerr) then
                                ierr=nf_close(ncid)
                                if (ierr == nf_noerr) then
                                  write(*,*)   'Changed units of ''',
     &                                  time_var(1:lvar), ''' in ''',
     &                                  fname(1:lfnm),  ''' into ''',
     &                                          units(1:latt), '''.'
                                  goto 1 !--> success
                                else
                                  write(*,'(/1x,4A)')  '### ERROR: ',
     &                                 'Cannot close netCDF file ''',
     &                                          fname(1:lfnm), '''.'
                                endif
                              else
                                write(*,'(/1x,6A)')    '### ERROR: ',
     &                                    'Cannot write variable ''',
     &                                time_var(1:lvar), ''' into ''',
     &                                          fname(1:lfnm), '''.'
                              endif
                            else
                              write(*,'(/1x,4A)')      '### ERROR: ',
     &                        'Cannot switch file ''', fname(1:lfnm),
     &                                         ''' into input mode.'
                            endif 
                          else
                            write(*,'(/1x,4A)') '### ERROR: Cannot ',
     &                      'add attribute ''units'' to variable ''',
     &                                       time_var(1:lvar), '''.'
                          endif
                        else
                          write(*,'(/1x,4A)')   '### ERROR: Cannot ',
     &                               'switch file ''', fname(1:lfnm),
     &                                 ''' into re-definition mode.'
                        endif 
                      else
                        write(*,'(/1x,6A)')     '### ERROR: Cannot ',
     &                          'read variable ''', time_var(1:lvar),
     &                            ''' from ''', fname(1:lfnm), '''.'
                      endif
                    else
                      write(*,'(/1x,6A)')   '### ERROR: Variable ''',
     &                   time_var(1:lvar), ''' has unknown units ''',
     &                                          units(1:latt), '''.'
                    endif
                  else
                    write(*,'(/1x,6A)')     '### ERROR: Cannot read ',
     &                          'attribute ''units'' for variable ''',
     &                time_var(1:lvar),''' in ''',fname(1:lfnm),'''.'
                  endif
                else
                  write(*,'(/1x,2A,I4,1x,3A)')   '### ERROR: Cannot ',
     &               'find length of dimension #', dimids(1), 'in ''',
     &                                           fname(1:lfnm), '''.'
                endif
              else
                write(*,'(/1x,4A,I4,A)')     '### ERROR: Variable ''',
     &                 time_var(1:lvar), ''' must have no more than ',
     &                        'one dimension instead of ', ndims, '.'
              endif
            else
              write(*,'(/1x,6A)')   '### ERROR: Cannot make inquiry ',
     &                'for variable ''', time_var(1:lvar), ''' in ''',
     &                                           fname(1:lfnm), '''.'
            endif
          else
            write(*,'(/1x,6A)')  '### ERROR: Cannot find variable ''',
     &             time_var(1:lvar), ''' in ''', fname(1:lfnm), '''.'
          endif
        else
          write(*,'(/1x,4A/12x,A/)') '### ERROR: Cannot open netCDF ',
     &                    'file ''', fname(1:lfnm), ''' for writing.'
        endif
        if (ierr /= nf_noerr) write(*,'(12x,A/)') nf_strerror(ierr)
        stop
   1    continue
      enddo 
      end
