      module smooth_topo_params
       character(len=64) method
       real(kind=8) :: hmin=0.D0, hmax=0.D0, r_max=0.D0
       integer :: iters_cond=0, iters_lin=0
      end

      subroutine write_topo(ncid, grid, xi_rho,eta_rho, h)

! Write smoothed topography (array "h") into netCDF grid file, as
! well as save parameters "hmin", "hmax", "r_max", smoothing method
! and number of iterations as attributes to the topographic variable
! variable: these may be useful if one needs to regenerate exactly
! the same version of topography.

! Depending on whether or not the grid file was previously subjected
! to parent-child topography matching at the boundary "r2r_match_topo",
! the smoothed but not yet matched version of topography is stored as
! netCDF variable "hsmth" rather than "h" read by ROMS code.  In this
! case the version of topography generated by the current procedure
! is saved into "hsmth" as well and needs to be re-matched by
! "r2r_match_topo" before in can be used by the model. 

      use smooth_topo_params
      implicit none
      character(len=*) grid
      integer xi_rho,eta_rho
      real(kind=8) h(xi_rho,eta_rho)
      integer ncid, varid, ierr, lgrd,lhnm,lmtd,lenstr
      character(len=6) hname

#include "netcdf.inc"

      lgrd=lenstr(grid)
      ierr=nf_inq_varid(ncid, 'hsmth', varid)
        if (ierr == nf_noerr) then
          write(*,'(/2x,2A)') 'Found variable ''hsmth'': using ',
     &                         'it to store smoothed topograpy.'
          hname='hsmth' ; lhnm=5
        else
          ierr=nf_inq_varid(ncid, 'h', varid)
          if (ierr == nf_noerr) then
            write(*,'(/1x,2A)') 'Variable ''hsmth'' not present, ',
     &                                  'but found ''h'' instead.'
          hname='h' ; lhnm=1
        endif
      endif

      if (ierr == nf_noerr) then
        ierr=nf_redef(ncid)
        if (ierr == nf_noerr) then
          ierr=nf_put_att_double(ncid, varid, 'hmin', nf_double,
     &                                                    1, hmin)
          if (ierr == nf_noerr) then
            ierr=nf_put_att_double(ncid, varid, 'hmax', nf_double,
     &                                                     1, hmax)
            if (ierr == nf_noerr) then
              ierr=nf_put_att_double(ncid, varid, 'r_max',nf_double,
     &                                                    1, r_max)
              if (ierr == nf_noerr) then
                lmtd=lenstr(method)
                ierr=nf_put_att_text(ncid, varid, 'method', lmtd,
     &                                             method(1:lmtd))
                if (ierr == nf_noerr) then
                  if (iters_cond > 0) then
                    ierr=nf_put_att_int(ncid, varid, 'iters_cond',
     &                                      nf_int, 1, iters_cond)
                  endif
                  if (ierr == nf_noerr) then
                    if (iters_lin > 0) then
                      ierr=nf_put_att_int (ncid, varid, 'iters_linear',
     &                                          nf_int, 1, iters_lin)
                    endif
                    if (ierr == nf_noerr) then
                      ierr=nf_enddef (ncid)
                      if (ierr == nf_noerr) then
                        ierr=nf_put_var_double(ncid, varid, h)
                        if (ierr == nf_noerr) then
                          ierr=nf_close(ncid)
                          if (ierr == nf_noerr) then
                            write(*,'(/1x,5A/)') 'Wrote topography ''',
     &                           hname(1:lhnm), ''' into grid file ''',
     &                                             grid(1:lgrd), '''.'
                          else
                            write(*,'(/1x,4A/12x,A/)')   '### ERROR: ',
     &                     'Cannot close netCDF file ''', grid(1:lgrd),
     &                                        '''.', nf_strerror(ierr)
                          endif
                        else
                          write(*,'(/1x,5A/12x,A/)')     '### ERROR: ',
     &                      'Cannot write variable ''h'' into netCDF ',
     &                  'file ''',grid(1:lgrd),'''.',nf_strerror(ierr)
                        endif
                      else
                        write(*,'(/1x,4A/12x,A/)')       '### ERROR: ',
     &                   'Cannot switch netCDF file ''', grid(1:lgrd),
     &                  ''' into data input mode.', nf_strerror(ierr)
                      endif
                    else
                      write(*,'(/1x,5A/12x,A/)')  '### ERROR: Cannot ',
     &                  'put attribute ''iters_linear'' for variable ',
     &                      '''h'' into netCDF file ''',  grid(1:lgrd),
     &                                       '''.', nf_strerror(ierr)
                    endif
                  else
                    write(*,'(/1x,5A/12x,A/)')    '### ERROR: Cannot ',
     &               'put attribute ''iters_cond'' for variable ''h'' ',
     &                   'into netCDF file ''',   grid(1:lgrd),  '''.',
     &                                              nf_strerror(ierr)
                  endif
                else
                  write(*,'(/1x,5A/12x,A/)')  '### ERROR: Cannot put ',
     &          'attribute ''method'' for variable ''h'' into netCDF ',
     &              'file ''', grid(1:lgrd), '''.', nf_strerror(ierr)
                endif
              else
                write(*,'(/1x,5A/12x,A/)')    '### ERROR: Cannot put ',
     &           'attribute ''r_max'' for variable ''h'' into netCDF ',
     &              'file ''', grid(1:lgrd), '''.', nf_strerror(ierr)
              endif
            else
              write(*,'(/1x,5A/12x,A/)')      '### ERROR: Cannot put ',
     &            'attribute ''hmax'' for variable ''h'' into netCDF ',
     &               'file ''', grid(1:lgrd),'''.', nf_strerror(ierr)
            endif
          else
            write(*,'(/1x,5A/12x,A/)')        '### ERROR: Cannot put ',
     &            'attribute ''hmin'' for variable ''h'' into netCDF ',
     &               'file ''', grid(1:lgrd),'''.', nf_strerror(ierr)
          endif
        else
          write(*,'(/1x,4A/12x,A/)')       '### ERROR: Cannot switch ',
     &    'netCDF file ''', grid(1:lgrd), ''' into redefinition mode.',
     &                                              nf_strerror(ierr)
        endif
      else
        write(*,'(/1x,4A/12x,A/)') '### ERROR: Neither ''hsmth'' nor ',
     &       '''h'' is present in netCDF file ''', grid(1:lgrd), '''.', 
     &                                              nf_strerror(ierr)
      endif
      end
